@@ -6,7 +6,7 @@
 
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -53,20 +53,21 @@
 	need to insert a different value (C->lhs[level][2]).
 */
 
-WORD execarg(PHEAD WORD *term, WORD level)
+int execarg(PHEAD WORD *term, WORD level)
 {
 	GETBIDENTITY
 	WORD *t, *r, *m, *v;
 	WORD *start, *stop, *rstop, *r1, *r2 = 0, *r3 = 0, *r4, *r5, *r6, *r7, *r8, *r9;
 	WORD *mm, *mstop, *rnext, *rr, *factor, type, ngcd, nq;
 	CBUF *C = cbuf+AM.rbufnum, *CC = cbuf+AT.ebufnum;
-	WORD i, j, k, oldnumlhs = AR.Cnumlhs, count, action = 0, olddefer = AR.DeferFlag;
+	WORD i, j, k, oldnumlhs = AR.Cnumlhs, count, olddefer = AR.DeferFlag;
+	int action = 0;
 	WORD oldnumrhs = CC->numrhs, size, pow, jj;
 	LONG oldcpointer = CC->Pointer - CC->Buffer, oldppointer = AT.pWorkPointer, lp;
 	WORD *oldwork = AT.WorkPointer, *oldwork2, scale, renorm;
 	WORD kLCM = 0, kGCD = 0, kGCD2, kkLCM = 0, jLCM = 0, jGCD, sign = 1;
 	int ii, didpolyratfun;
-	UWORD *EAscrat, *GCDbuffer = 0, *GCDbuffer2, *LCMbuffer, *LCMb, *LCMc;
+	UWORD *EAscrat, *GCDbuffer = 0, *GCDbuffer2 = 0, *LCMbuffer = 0, *LCMb = 0, *LCMc = 0;
 	AT.WorkPointer += *term;
 	start = C->lhs[level];
 	AR.Cnumlhs = start[2];
@@ -124,6 +125,16 @@ WORD execarg(PHEAD WORD *term, WORD level)
 /*
   	#[ Argument detection : + argument statement
 */
+/*
+	Allocate Numbers for MakeInteger here, for re-use in case multiple
+	functions are treated in the same term.
+*/
+	if ( type == TYPENORM4 ) {
+		GCDbuffer = NumberMalloc("execarg");
+		GCDbuffer2 = NumberMalloc("execarg");
+		LCMbuffer = NumberMalloc("execarg");
+		LCMb = NumberMalloc("execarg"); LCMc = NumberMalloc("execarg");
+	}
 	didpolyratfun = 0;
 	while ( t < rstop ) {
 		if ( *t >= FUNCTION && functions[*t-FUNCTION].spec <= 0 ) {
@@ -158,9 +169,19 @@ WORD execarg(PHEAD WORD *term, WORD level)
 					}
 RightNum:
 					if ( m[1] == 2 ) {
-						m += 2;
-						m += *m;
-						goto HaveTodo;
+#ifdef WITHFLOAT
+						if ( *t != FLOATFUN || TestFloat(t) == 0 )
+#endif
+						{
+							m += 2;
+							m += *m;
+							goto HaveTodo;
+						}
+#ifdef WITHFLOAT
+						else {
+							m += 2;
+						}
+#endif
 					}
 					else {
 						r = m + m[1];
@@ -336,11 +357,8 @@ HaveTodo:
 						For normalizing everything to integers we have to
 						determine for all elements of this argument the LCM of
 						the denominators and the GCD of the numerators.
+						The buffers have been allocated already.
 */
-						GCDbuffer = NumberMalloc("execarg");
-						GCDbuffer2 = NumberMalloc("execarg");
-						LCMbuffer = NumberMalloc("execarg");
-						LCMb = NumberMalloc("execarg"); LCMc = NumberMalloc("execarg");
 						r4 = r + *r;
 						r1 = r + ARGHEAD;
 /*
@@ -434,18 +452,10 @@ HaveTodo:
 								r3[jGCD+kLCM] = LCMbuffer[jGCD];
 							k = kLCM;
 						}
-/*						NumberFree(GCDbuffer,"execarg"); GCDbuffer = 0; */
-						NumberFree(GCDbuffer2,"execarg");
-						NumberFree(LCMbuffer,"execarg");
-						NumberFree(LCMb,"execarg"); NumberFree(LCMc,"execarg");
+
 						j = 2*k+1;
 /*
-						Now we have to correct the overal factor
-
-						We have a little problem here.
-						r3 is in GCDbuffer and we returned that.
-						At the same time we still use it.
-						This works as long as each worker has its own TermMalloc
+						Now we have to correct the overall factor
 */
 						if ( scale && ( factor == 0 || *factor > 0 ) )
 							goto ScaledVariety;
@@ -500,7 +510,7 @@ HaveTodo:
 						if ( j < 0 ) j = -j;
 						if ( type == TYPENORM && scale && ( factor == 0 || *factor ) ) {
 /*
-							Now we correct the overal factor
+							Now we correct the overall factor
 */
 ScaledVariety:;
 							size = term[*term-1];
@@ -725,6 +735,15 @@ do_shift:
 		}
 		t += t[1];
 	}
+/*
+		If TYPENORM4, we allocated Number buffers before the above while loop. Free them.
+*/
+	if ( type == TYPENORM4 ) {
+		NumberFree(GCDbuffer,"execarg");
+		NumberFree(GCDbuffer2,"execarg");
+		NumberFree(LCMbuffer,"execarg");
+		NumberFree(LCMb,"execarg"); NumberFree(LCMc,"execarg");
+	}
 	if ( didpolyratfun ) {
 		PolyFunDirty(BHEAD term);
 		didpolyratfun = 0;
@@ -1030,9 +1049,15 @@ do_shift:
 							continue;
 						}
 						else if ( *t == -MINVECTOR ) {
-							*r1++ = -VECTOR; t++; *r1++ = *t++;
-							*r1++ = -SNUMBER; *r1++ = -1;
-							*r1++ = -SNUMBER; *r1++ = 1;
+							if ( AC.OldFactArgFlag == NEWFACTARG ) {
+								*r1++ = -SNUMBER; *r1++ = -1;
+								*r1++ = -VECTOR; t++; *r1++ = *t++;
+							}
+							else {
+								*r1++ = -VECTOR; t++; *r1++ = *t++;
+								*r1++ = -SNUMBER; *r1++ = -1;
+								*r1++ = -SNUMBER; *r1++ = 1;
+							}
 							continue;
 						}
 					}
@@ -1139,35 +1164,35 @@ do_shift:
 						EAscrat = (UWORD *)(TermMalloc("execarg"));
 						if ( t + *t == r3 ) {
 							if ( factor == 0 || *factor > 2 ) {
-							if ( pow > 0 ) {
+							  if ( pow > 0 ) {
 								*r1++ = -SNUMBER; *r1++ = -1;
 								t = r5;
 								while ( t < r3 ) {
 									t += *t; t[-1] = -t[-1];
 								}
-							}
-							t = rr; *r1++ = *t++; *r1++ = 1; t++;
-							COPYARG(r1,t);
-							while ( t < m ) *r1++ = *t++;
+							  }
+							  t = rr; *r1++ = *t++; *r1++ = 1; t++;
+							  COPYARG(r1,t);
+							  while ( t < m ) *r1++ = *t++;
 							}
 						}
 						else {
-						 GETSTOP(t,r6);
-						 ngcd = t[t[0]-1];
-						 i = abs(ngcd)-1;
-						 while ( --i >= 0 ) EAscrat[i] = r6[i];
-						 t += *t;
-						 while ( t < r3 ) {
+						GETSTOP(t,r6);
+						ngcd = t[t[0]-1];
+						i = abs(ngcd)-1;
+						while ( --i >= 0 ) EAscrat[i] = r6[i];
+						t += *t;
+						while ( t < r3 ) {
 							GETSTOP(t,r6);
 							i = t[t[0]-1];
 							if ( AccumGCD(BHEAD EAscrat,&ngcd,(UWORD *)r6,i) ) goto execargerr;
 							if ( ngcd == 3 && EAscrat[0] == 1 && EAscrat[1] == 1 ) break;
 							t += *t;
-						 }
+						}
 /*
-	 					 if ( ngcd != 3 || EAscrat[0] != 1 || EAscrat[1] != 1 ) {
+	 					if ( ngcd != 3 || EAscrat[0] != 1 || EAscrat[1] != 1 )
 */
-						 {
+						{
 							if ( pow ) ngcd = -ngcd;
 							t = r5; r9 = r1; *r1++ = t[-ARGHEAD]; *r1++ = 1;
 							FILLARG(r1); ngcd = REDLENG(ngcd);
@@ -1215,9 +1240,26 @@ do_shift:
 									if ( ToFast(r9,r9) ) r1 = r9+2;
 								}
 							}
-	 					 }
+	 					}
 /*
-			#] Numerical factor :
+			#] Numerical factor : 
+						else {
+onetermnew:;
+
+							if ( factor == 0 || *factor > 2 ) {
+							  if ( pow > 0 ) {
+								*r1++ = -SNUMBER; *r1++ = -1;
+								t = r5;
+								while ( t < r3 ) {
+									t += *t; t[-1] = -t[-1];
+								}
+							  }
+							  t = rr; *r1++ = *t++; *r1++ = 1; t++;
+							  COPYARG(r1,t);
+							  while ( t < m ) *r1++ = *t++;
+							}
+						}
+onetermnew:;
 */
 						}
 						TermFree(EAscrat,"execarg");
@@ -1689,6 +1731,7 @@ oneterm:;
 					}
                 	} /* AC.OldFactArgFlag */
 				}
+/* r1 is fout in ons voorbeeld. */
 				r2[1] = r1 - r2;
 				action = 1;
 				v[2] = DIRTYFLAG;
@@ -1707,14 +1750,13 @@ oneterm:;
 		if ( AT.WorkPointer < t ) AT.WorkPointer = t;
 	}
 /*
-  	#] FACTARG :
+  	#] FACTARG : 
 */
 	AR.Cnumlhs = oldnumlhs;
 	if ( action && Normalize(BHEAD term) ) goto execargerr;
 	AT.WorkPointer = oldwork;
 	if ( AT.WorkPointer < term + *term ) AT.WorkPointer = term + *term;
 	AT.pWorkPointer = oldppointer;
-	if ( GCDbuffer ) NumberFree(GCDbuffer,"execarg");
 	return(action);
 execargerr:
 	AT.WorkPointer = oldwork;
@@ -1726,11 +1768,11 @@ execargerr:
 }
 
 /*
-  	#] execarg :
+  	#] execarg : 
   	#[ execterm :
 */
 
-WORD execterm(PHEAD WORD *term, WORD level)
+int execterm(PHEAD WORD *term, WORD level)
 {
 	GETBIDENTITY
 	CBUF *C = cbuf+AM.rbufnum;
@@ -1774,7 +1816,7 @@ WORD execterm(PHEAD WORD *term, WORD level)
 			buffer1 = 0;
 		}
 		AN.tryterm = 1;
-		if ( EndSort(BHEAD (WORD *)((VOID *)(&buffer1)),2) < 0 ) goto exectermerr;
+		if ( EndSort(BHEAD (WORD *)((void *)(&buffer1)),2) < 0 ) goto exectermerr;
 		tryterm = AN.tryterm; AN.tryterm = 0;
 		level = AR.Cnumlhs;
 	} while ( AR.Cnumlhs < maxisat );
@@ -2064,7 +2106,8 @@ int ArgFactorize(PHEAD WORD *argin, WORD *argout)
 			StoreTerm(BHEAD t);
 			t = tstop;
 		}
-		EndSort(BHEAD argin+ARGHEAD,0);
+		/* par = 1, in case the arg has more than SubTermsInSmall terms */
+		EndSort(BHEAD argin+ARGHEAD,1);
 		argin[*argin] = oldword;
 	}
 /*
@@ -2125,7 +2168,8 @@ int ArgFactorize(PHEAD WORD *argin, WORD *argout)
 			StoreTerm(BHEAD t);
 			t = tstop;
 		}
-		EndSort(BHEAD argfree+ARGHEAD,0);
+		/* par = 1, in case the arg has more than SubTermsInSmall terms */
+		EndSort(BHEAD argfree+ARGHEAD,1);
 		t = argfree+ARGHEAD;
 		while ( *t ) t += *t;
 		*argfree = t - argfree;
@@ -2217,7 +2261,8 @@ getout:
 			StoreTerm(BHEAD argextra);
 			t += *t; argextra += *argextra;
 		}
-		if ( EndSort(BHEAD argfree+ARGHEAD,0) ) { error = -2; goto getout; }
+		/* par = 1, in case the arg has more than SubTermsInSmall terms */
+		if ( EndSort(BHEAD argfree+ARGHEAD,1) < 0 ) { error = -2; goto getout; }
 		t = argfree + ARGHEAD;
 		while ( *t > 0 ) t += *t;
 		*argfree = t - argfree;
@@ -2240,63 +2285,64 @@ getout:
 		        Be careful: there should be more than one argument now.
 */
 	if ( error == 0 && action ) {
-	  a1 = a; NEXTARG(a1);
-	  if ( *a1 != 0 ) {
-		CBUF *C = cbuf+AC.cbufnum;
-		CBUF *CC = cbuf+AT.ebufnum;
-		WORD *oldworkpointer = AT.WorkPointer;
-		WORD *argcopy2 = TermMalloc("argcopy2"), *a1, *a2;
-		a1 = a; a2 = argcopy2;
-		while ( *a1 ) {
-			if ( *a1 < 0 ) {
-				if ( *a1 > -FUNCTION ) *a2++ = *a1++;
-				*a2++ = *a1++; *a2 = 0;
-				continue;
-			}
-			t = a1 + ARGHEAD;
-			tstop = a1 + *a1;
-			argextra = AT.WorkPointer;
-			NewSort(BHEAD0);
-			while ( t < tstop ) {
-				if ( ConvertFromPoly(BHEAD t,argextra,numxsymbol,CC->numrhs-startebuf+numxsymbol
-				,startebuf-numxsymbol,1) <= 0 ) {
-					TermFree(argcopy2,"argcopy2");
-					LowerSortLevel();
-					error = -3;
-					goto getout;
+		a1 = a; NEXTARG(a1);
+		if ( *a1 != 0 ) {
+			CBUF *C = cbuf+AC.cbufnum;
+			CBUF *CC = cbuf+AT.ebufnum;
+			WORD *oldworkpointer = AT.WorkPointer;
+			WORD *argcopy2 = TermMalloc("argcopy2"), *a1, *a2;
+			a1 = a; a2 = argcopy2;
+			while ( *a1 ) {
+				if ( *a1 < 0 ) {
+					if ( *a1 > -FUNCTION ) *a2++ = *a1++;
+					*a2++ = *a1++; *a2 = 0;
+					continue;
 				}
-				t += *t;
-				AT.WorkPointer = argextra + *argextra;
+				t = a1 + ARGHEAD;
+				tstop = a1 + *a1;
+				argextra = AT.WorkPointer;
+				NewSort(BHEAD0);
+				while ( t < tstop ) {
+					if ( ConvertFromPoly(BHEAD t,argextra,numxsymbol,CC->numrhs-startebuf+numxsymbol
+					,startebuf-numxsymbol,1) <= 0 ) {
+						TermFree(argcopy2,"argcopy2");
+						LowerSortLevel();
+						error = -3;
+						goto getout;
+					}
+					t += *t;
+					AT.WorkPointer = argextra + *argextra;
 /*
-				ConvertFromPoly leaves terms with subexpressions. Hence:
+					ConvertFromPoly leaves terms with subexpressions. Hence:
 */
-				if ( Generator(BHEAD argextra,C->numlhs) ) {
-					TermFree(argcopy2,"argcopy2");
-					LowerSortLevel();
-					error = -4;
-					goto getout;
+					if ( Generator(BHEAD argextra,C->numlhs) ) {
+						TermFree(argcopy2,"argcopy2");
+						LowerSortLevel();
+						error = -4;
+						goto getout;
+					}
 				}
+				AT.WorkPointer = oldworkpointer;
+				/* par = 1, in case the factor has more than SubTermsInSmall terms */
+				if ( EndSort(BHEAD a2+ARGHEAD,1) < 0 ) { error = -5; goto getout; }
+				t = a2+ARGHEAD; while ( *t ) t += *t;
+				*a2 = t - a2; a2[1] = 0; ZEROARG(a2);
+				ToFast(a2,a2); NEXTARG(a2);
+				a1 = tstop;
 			}
-			AT.WorkPointer = oldworkpointer;
-			if ( EndSort(BHEAD a2+ARGHEAD,0) ) { error = -5; goto getout; }
-			t = a2+ARGHEAD; while ( *t ) t += *t;
-			*a2 = t - a2; a2[1] = 0; ZEROARG(a2);
-			ToFast(a2,a2); NEXTARG(a2);
-			a1 = tstop;
+			i = a2 - argcopy2;
+			a2 = argcopy2; a1 = a;
+			NCOPY(a1,a2,i);
+			*a1 = 0;
+			TermFree(argcopy2,"argcopy2");
+/*
+			Erase the entries we made temporarily in cbuf[AT.ebufnum]
+*/
+			CC->numrhs = startebuf;
+		}
+		else {	/* no factorization. recover the argument from before step 3. */
+			for ( i = 0; i <= *argcopy; i++ ) a[i] = argcopy[i];
 		}
-		i = a2 - argcopy2;
-		a2 = argcopy2; a1 = a;
-		NCOPY(a1,a2,i);
-		*a1 = 0;
-		TermFree(argcopy2,"argcopy2");
-/*
-		Erase the entries we made temporarily in cbuf[AT.ebufnum]
-*/
-		CC->numrhs = startebuf;
-	  }
-	  else {	/* no factorization. recover the argument from before step 3. */
-		for ( i = 0; i <= *argcopy; i++ ) a[i] = argcopy[i];
-	  }
 	}
 /*
   	#] step 6 : 
@@ -2370,7 +2416,8 @@ return0:
 					StoreTerm(BHEAD t);
 					t = tstop;
 				}
-				EndSort(BHEAD a+ARGHEAD,0);
+				/* par = 1, in case the factor has more than SubTermsInSmall terms */
+				EndSort(BHEAD a+ARGHEAD,1);
 				a[*a] = oldword;
 				a += *a;
 			}
@@ -2493,7 +2540,7 @@ WORD FindArg(PHEAD WORD *a)
  *	If par == 1 it inserts in the cache defined with the FactorCache statement
  */
 
-WORD InsertArg(PHEAD WORD *argin, WORD *argout,int par)
+int InsertArg(PHEAD WORD *argin, WORD *argout,int par)
 {
 	CBUF *C;
 	WORD *a, i, bufnum;
