@@ -53,7 +53,7 @@
   	#] Explanations : 
   	#[ License :
  *
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -275,7 +275,7 @@ static int PF_CheckRecoveryFile()
 }
 #endif
 
-int CheckRecoveryFile()
+int CheckRecoveryFile(void)
 {
 	int ret = 0;
 #ifdef WITHMPI
@@ -330,7 +330,7 @@ int CheckRecoveryFile()
  *  Deletes the recovery files. It is called by CleanUp() in the case of a
  *  successful completion.
  */
-void DeleteRecoveryFile()
+void DeleteRecoveryFile(void)
 {
 	if ( done_snapshot ) {
 		remove(recoveryfile);
@@ -361,7 +361,7 @@ void DeleteRecoveryFile()
 /**
  *  Returns pointer to recovery filename. 
  */
-char *RecoveryFilename()
+char *RecoveryFilename(void)
 {
 	return(recoveryfile);
 }
@@ -396,7 +396,7 @@ static char *InitName(char *str, char *ext)
  *  This functions should only be called once to avoid memory leaks and after
  *  AM.TempDir has been initialized.
  */
-void InitRecovery()
+void InitRecovery(void)
 {
 	int lenpath = AM.TempDir ? strlen((char*)AM.TempDir)+1 : 0;
 #ifdef WITHMPI
@@ -840,6 +840,7 @@ static void print_M()
 	MesPrint("%d", AM.ggShortStatsMax);
 	MesPrint("--MARK 11");
 	MesPrint("%d", AM.FromStdin);
+	MesPrint("%d", AM.IgnoreDeprecation);
 	MesPrint("%%%% END M_const");
 /*	fflush(0); */
 }
@@ -1503,6 +1504,7 @@ int DoRecovery(int *moduletype)
 	R_COPY_S(AM.Path,UBYTE *);
 
 	R_SET(AM.FromStdin, BOOL);
+	R_SET(AM.IgnoreDeprecation, BOOL);
 
 #ifdef PRINTDEBUG
 	print_M();
@@ -2574,6 +2576,7 @@ static int DoSnapshot(int moduletype)
 	S_WRITE_S(AM.Path);
 
 	S_WRITE_B(&AM.FromStdin,sizeof(BOOL));
+	S_WRITE_B(&AM.IgnoreDeprecation,sizeof(BOOL));
 
 	/*#] AM :*/ 
 	/*#[ AC :*/
