@@ -14,7 +14,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -43,6 +43,7 @@
 */
 
 #include "form3.h"
+#include "comtool.h"
 
 /*
 	com1commands are the commands of which only part of the word has to
@@ -117,11 +118,16 @@ static KEYWORD com2commands[] = {
 	,{"chainin",        (TFUN)CoChainin,          STATEMENT,    PARTEST}
 	,{"chainout",       (TFUN)CoChainout,         STATEMENT,    PARTEST}
 	,{"chisholm",       (TFUN)CoChisholm,         STATEMENT,    PARTEST}
+#ifdef WITHFLOAT
+	,{"chop",           (TFUN)CoChop,             STATEMENT,    PARTEST}
+#endif
+	,{"clearflag",      (TFUN)CoClearUserFlag,    STATEMENT,    PARTEST}
 	,{"cleartable",     (TFUN)CoClearTable,       DECLARATION,  PARTEST}
 	,{"collect",        (TFUN)CoCollect,          SPECIFICATION,PARTEST}
 	,{"commuteinset",   (TFUN)CoCommuteInSet,     DECLARATION,  PARTEST}
 	,{"contract",       (TFUN)CoContract,         STATEMENT,    PARTEST}
-	,{"copyspectator"  ,(TFUN)CoCopySpectator,    DEFINITION,   PARTEST}
+	,{"copyspectator",  (TFUN)CoCopySpectator,    DEFINITION,   PARTEST}
+	,{"createall",      (TFUN)CoCreateAll,        STATEMENT,    PARTEST}
 	,{"createspectator",(TFUN)CoCreateSpectator,  DECLARATION,  PARTEST}
 	,{"ctable",         (TFUN)CoCTable,           DECLARATION,  PARTEST}
 	,{"deallocatetable",(TFUN)CoDeallocateTable,  DECLARATION,  PARTEST}
@@ -141,10 +147,14 @@ static KEYWORD com2commands[] = {
 	,{"endif",          (TFUN)CoEndIf,            STATEMENT,    PARTEST}
 	,{"endinexpression",(TFUN)CoEndInExpression,  STATEMENT,    PARTEST}
 	,{"endinside",      (TFUN)CoEndInside,        STATEMENT,    PARTEST}
+	,{"endmodel",       (TFUN)CoEndModel,         DECLARATION,  PARTEST}
 	,{"endrepeat",      (TFUN)CoEndRepeat,        STATEMENT,    PARTEST}
 	,{"endswitch",      (TFUN)CoEndSwitch,        STATEMENT,    PARTEST}
 	,{"endterm",        (TFUN)CoEndTerm,          STATEMENT,    PARTEST}
 	,{"endwhile",       (TFUN)CoEndWhile,         STATEMENT,    PARTEST}
+#ifdef WITHFLOAT
+	,{"evaluate",       (TFUN)CoEvaluate,         STATEMENT,    PARTEST}
+#endif
 	,{"exit",           (TFUN)CoExit,             STATEMENT,    PARTEST}
 	,{"extrasymbols",   (TFUN)CoExtraSymbols,     DECLARATION,  PARTEST}
 	,{"factarg",        (TFUN)CoFactArg,          STATEMENT,    PARTEST}
@@ -169,12 +179,14 @@ static KEYWORD com2commands[] = {
 	,{"many",           (TFUN)CoMany,             STATEMENT,    PARTEST}
 	,{"merge",          (TFUN)CoMerge,            STATEMENT,    PARTEST}
 	,{"metric",         (TFUN)CoMetric,           DECLARATION,  PARTEST}
+	,{"model",          (TFUN)CoModel,            DECLARATION,  PARTEST}
 	,{"moduleoption",   (TFUN)CoModuleOption,     ATENDOFMODULE,PARTEST}
 	,{"multi",          (TFUN)CoMulti,            STATEMENT,    PARTEST}
 	,{"multibracket",   (TFUN)CoMultiBracket,     STATEMENT,    PARTEST}
 	,{"ndrop",          (TFUN)CoNoDrop,           SPECIFICATION,PARTEST}
 	,{"nfactorize",     (TFUN)CoNFactorize,       TOOUTPUT,     PARTEST}
 	,{"nhide",          (TFUN)CoNoHide,           SPECIFICATION,PARTEST}
+	,{"nintohide",      (TFUN)CoNoIntoHide,       SPECIFICATION,PARTEST}
 	,{"normalize",      (TFUN)CoNormalize,        STATEMENT,    PARTEST}
 	,{"notinparallel",  (TFUN)CoNotInParallel,    SPECIFICATION,PARTEST}
 	,{"nskip",          (TFUN)CoNoSkip,           SPECIFICATION,PARTEST}
@@ -185,6 +197,7 @@ static KEYWORD com2commands[] = {
 	,{"on",             (TFUN)CoOn,               DECLARATION,  PARTEST}
 	,{"once",           (TFUN)CoOnce,             STATEMENT,    PARTEST}
 	,{"only",           (TFUN)CoOnly,             STATEMENT,    PARTEST}
+	,{"particle",       (TFUN)CoParticle,         DECLARATION,  PARTEST}
 	,{"polyfun",        (TFUN)CoPolyFun,          DECLARATION,  PARTEST}
 	,{"polyratfun",     (TFUN)CoPolyRatFun,       DECLARATION,  PARTEST}
 	,{"pophide",        (TFUN)CoPopHide,          SPECIFICATION,PARTEST}
@@ -202,12 +215,16 @@ static KEYWORD com2commands[] = {
 	,{"select",         (TFUN)CoSelect,           STATEMENT,    PARTEST}
 	,{"set",            (TFUN)CoSet,              DECLARATION,  PARTEST}
 	,{"setexitflag",    (TFUN)CoSetExitFlag,      STATEMENT,    PARTEST}
+	,{"setflag",        (TFUN)CoSetUserFlag,      STATEMENT,    PARTEST}
 	,{"shuffle",        (TFUN)CoMerge,            STATEMENT,    PARTEST}
 	,{"skip",           (TFUN)CoSkip,             SPECIFICATION,PARTEST}
 	,{"sort",           (TFUN)CoSort,             STATEMENT,    PARTEST}
 	,{"splitarg",       (TFUN)CoSplitArg,         STATEMENT,    PARTEST}
 	,{"splitfirstarg",  (TFUN)CoSplitFirstArg,    STATEMENT,    PARTEST}
 	,{"splitlastarg",   (TFUN)CoSplitLastArg,     STATEMENT,    PARTEST}
+#ifdef WITHFLOAT
+	,{"strictrounding", (TFUN)CoStrictRounding,   STATEMENT,    PARTEST}
+#endif
 	,{"stuffle",        (TFUN)CoStuffle,          STATEMENT,    PARTEST}
 	,{"sum",            (TFUN)CoSum,              STATEMENT,    PARTEST}
 	,{"switch",         (TFUN)CoSwitch,           STATEMENT,    PARTEST}
@@ -217,7 +234,14 @@ static KEYWORD com2commands[] = {
 	,{"term",           (TFUN)CoTerm,             STATEMENT,    PARTEST}
 	,{"testuse",        (TFUN)CoTestUse,          STATEMENT,    PARTEST}
 	,{"threadbucketsize",(TFUN)CoThreadBucket,    DECLARATION,  PARTEST}
+#ifdef WITHFLOAT
+	,{"tofloat",        (TFUN)CoToFloat,          STATEMENT,    PARTEST}
+#endif
 	,{"topolynomial",   (TFUN)CoToPolynomial,     STATEMENT,    PARTEST}
+#ifdef WITHFLOAT
+	,{"torat",          (TFUN)CoToRat,            STATEMENT,    PARTEST}
+	,{"torational",     (TFUN)CoToRat,            STATEMENT,    PARTEST}
+#endif
 	,{"tospectator",    (TFUN)CoToSpectator,      STATEMENT,    PARTEST}
 	,{"totensor",       (TFUN)CoToTensor,         STATEMENT,    PARTEST}
 	,{"tovector",       (TFUN)CoToVector,         STATEMENT,    PARTEST}
@@ -227,6 +251,7 @@ static KEYWORD com2commands[] = {
 	,{"tryreplace",     (TFUN)CoTryReplace,       STATEMENT,    PARTEST}
 	,{"unfactorize",    (TFUN)CoUnFactorize,      TOOUTPUT,     PARTEST}
 	,{"unhide",         (TFUN)CoUnHide,           SPECIFICATION,PARTEST}
+	,{"vertex",         (TFUN)CoVertex,           DECLARATION,  PARTEST}
 	,{"while",          (TFUN)CoWhile,            STATEMENT,    PARTEST}
 };
 
@@ -250,32 +275,30 @@ LONG insubexpbuffers = 0;
 	subexpbuffers = (SUBBUF *)Malloc1(256*sizeof(SUBBUF),"subexpbuffers");\
 	topsubexpbuffers = subexpbuffers+256; } insubexpbuffers = 0; }
 
-#if defined(ILP32)
-
-#define PUTNUMBER128(t,n) { if ( n >= 16384 ) { \
-				*t++ = n/(128*128); *t++ = (n/128)%128; *t++ = n%128; } \
-			else if ( n >= 128 ) { *t++ = n/128; *t++ = n%128; }      \
-			else *t++ = n; }
-#define PUTNUMBER100(t,n) { if ( n >= 10000 ) { \
-				*t++ = n/10000; *t++ = (n/100)%100; *t++ = n%100; } \
-			else if ( n >= 100 ) { *t++ = n/100; *t++ = n%100; }   \
-			else *t++ = n; }
-
-#elif ( defined(LLP64) || defined(LP64) )
-
-#define PUTNUMBER128(t,n) { if ( n >= 2097152 ) { \
+#if BITSINWORD == 32
+	#define PUTNUMBER128(t,n) { if ( n >= 2097152 ) { \
 				*t++ = ((n/128)/128)/128; *t++ = ((n/128)/128)%128; *t++ = (n/128)%128; *t++ = n%128; } \
 			else if ( n >= 16384 ) { \
 				*t++ = n/(128*128); *t++ = (n/128)%128; *t++ = n%128; } \
 			else if ( n >= 128 ) { *t++ = n/128; *t++ = n%128; }      \
 			else *t++ = n; }
-#define PUTNUMBER100(t,n) { if ( n >= 1000000 ) { \
+	#define PUTNUMBER100(t,n) { if ( n >= 1000000 ) { \
 				*t++ = ((n/100)/100)/100; *t++ = ((n/100)/100)%100; *t++ = (n/100)%100; *t++ = n%100; } \
 			else if ( n >= 10000 ) { \
 				*t++ = n/10000; *t++ = (n/100)%100; *t++ = n%100; } \
 			else if ( n >= 100 ) { *t++ = n/100; *t++ = n%100; }   \
 			else *t++ = n; }
-
+#elif BITSINWORD == 16
+	#define PUTNUMBER128(t,n) { if ( n >= 16384 ) { \
+				*t++ = n/(128*128); *t++ = (n/128)%128; *t++ = n%128; } \
+			else if ( n >= 128 ) { *t++ = n/128; *t++ = n%128; }      \
+			else *t++ = n; }
+	#define PUTNUMBER100(t,n) { if ( n >= 10000 ) { \
+				*t++ = n/10000; *t++ = (n/100)%100; *t++ = n%100; } \
+			else if ( n >= 100 ) { *t++ = n/100; *t++ = n%100; }   \
+			else *t++ = n; }
+#else
+	#error Only 64-bit and 32-bit platforms are supported.
 #endif
 
 /*
@@ -289,7 +312,7 @@ LONG insubexpbuffers = 0;
 		Search in table 2 can be binary.
 */
 
-VOID inictable()
+void inictable(void)
 {
 	KEYWORD *k = com1commands;
 	int i, j, ksize;
@@ -401,19 +424,32 @@ int ParenthesesTest(UBYTE *sin)
 /*
  		#] ParenthesesTest : 
  		#[ SkipAName :
-
-		Skips a name and gives a pointer to the object after the name.
-		If there is not a proper name, it returns a zero pointer.
-		In principle the brackets match already, so the `if ( *s == 0 )'
-		code is not really needed, but you never know how the program
-		is extended later.
 */
 
+/**
+ * Skips a name and gives a pointer to the character after the name.
+ * If there is not a proper name, it emits a compiler message and then returns
+ * a null pointer.
+ * This function supports formal names (e.g., `[x+a]`) and $-variables
+ * in addition to ordinary identifiers.
+ *
+ * @note The brackets are assumed to be matched already.
+ *
+ * @param[in]  s  Pointer to a null-terminated input string that starts
+ *                with a name.
+ * @return        Pointer to the character after the name, or a null pointer
+ *                if the name is invalid.
+ */
 UBYTE *SkipAName(UBYTE *s)
 {
 	UBYTE *t = s;
 	if ( *s == '[' ) {
 		SKIPBRA1(s)
+/*
+		In principle the brackets match already, so the `if ( *s == 0 )'
+		code is not really needed, but you never know how the program
+		is extended later.
+*/
 		if ( *s == 0 ) {
 			MesPrint("&Illegal name: '%s'",t);
 			return(0);
@@ -545,7 +581,7 @@ int CompileStatement(UBYTE *in)
 	}
 	else {
 		if ( ( k = findcommand(s) ) == 0 && IsIdStatement(s) == 0 ) {
-			MesPrint("&Unrecognized statement");
+			MesPrint("&Unrecognized statement %s",s);
 			return(1);
 		}
 		if ( k == 0 ) {	/* Id statement without id. Note: id must be in table */
@@ -594,8 +630,28 @@ int CompileStatement(UBYTE *in)
 		}
 	  }
 	  else if ( k->type > AC.compiletype ) {
-		if ( StrCmp((UBYTE *)(k->name),(UBYTE *)"format") != 0 )
-			AC.compiletype = k->type;
+		/*
+		 * We intentionally do NOT update "compiletype" for:
+		 * - Format statements (type = TOOUTPUT)
+		 * - ModuleOption statements (type = ATENDOFMODULE)
+		 *   with sum/maximum/minimum/local (i.e., $-variable-related options)
+		 *
+		 * This relaxes the ordering constraint, allowing statements with
+		 * type >= the current "compiletype" to follow.
+		 */
+		if ( StrCmp((UBYTE *)(k->name),(UBYTE *)"format") == 0 )
+			goto NoUpdateCompileType;
+		if ( StrCmp((UBYTE *)(k->name),(UBYTE *)"moduleoption") == 0 ) {
+			UBYTE *ss = s;
+			SkipSpaces(&ss);
+			if ( ConsumeOption(&ss,"sum")
+			|| ConsumeOption(&ss,"maximum")
+			|| ConsumeOption(&ss,"minimum")
+			|| ConsumeOption(&ss,"local") ) goto NoUpdateCompileType;
+		}
+		AC.compiletype = k->type;
+NoUpdateCompileType:
+		;
 	  }
 	  else if ( k->type < AC.compiletype ) {
 		switch ( k->type ) {
@@ -647,7 +703,7 @@ int CompileStatement(UBYTE *in)
  		#[ TestTables :
 */
 
-int TestTables()
+int TestTables(void)
 {
 	FUNCTIONS f = functions;
 	TABLES t;
@@ -770,8 +826,8 @@ int CompileSubExpressions(SBYTE *tokens)
 				Terminate(-1);
 			}
 			if ( subexpbuffers+insubexpbuffers >= topsubexpbuffers ) {
-				DoubleBuffer((void **)((VOID *)(&subexpbuffers))
-				,(void **)((VOID *)(&topsubexpbuffers)),sizeof(SUBBUF),"subexpbuffers");
+				DoubleBuffer((void **)((void *)(&subexpbuffers))
+				,(void **)((void *)(&topsubexpbuffers)),sizeof(SUBBUF),"subexpbuffers");
 			}
 			subexpbuffers[insubexpbuffers].subexpnum = num;
 			subexpbuffers[insubexpbuffers].buffernum = AC.cbufnum;
@@ -1719,6 +1775,19 @@ docoef:
 				}
 				*relo = 2; *t++ = SNUMBER; *t++ = 4; *t++ = y;
 				goto TryPower;
+#ifdef WITHFLOAT
+			case TFLOAT:
+				{ WORD *w;
+				s = ReadFloat(s);
+				i = AT.WorkPointer[1];
+				w = AT.WorkPointer;
+				NCOPY(t,w,i);
+/*
+Power?
+*/
+				}
+				break;
+#endif
 			case TDOLLAR:
 			{
 				WORD *powplace;
@@ -1826,6 +1895,11 @@ ErrorBraces:
 				funflag = 0;
 				deno = 1;
 				break;
+			case TCONJUGATE:
+				MesPrint("&Complex conjugate operator (%#) is not implemented");
+				error = -1;
+				deno = 1;
+				break;
 			default:
 				MesPrint("&Internal error in code generator. Unknown object: %d",c);
 				error = -1;
@@ -1941,8 +2015,8 @@ int CodeFactors(SBYTE *tokens)
 					Terminate(-1);
 				}
 				if ( subexpbuffers+insubexpbuffers >= topsubexpbuffers ) {
-					DoubleBuffer((void **)((VOID *)(&subexpbuffers))
-					,(void **)((VOID *)(&topsubexpbuffers)),sizeof(SUBBUF),"subexpbuffers");
+					DoubleBuffer((void **)((void *)(&subexpbuffers))
+					,(void **)((void *)(&topsubexpbuffers)),sizeof(SUBBUF),"subexpbuffers");
 				}
 				subexpbuffers[insubexpbuffers].subexpnum = subexp;
 				subexpbuffers[insubexpbuffers].buffernum = AC.cbufnum;
@@ -2182,8 +2256,8 @@ dopowerd:
 		Terminate(-1);
 	}
 	if ( subexpbuffers+insubexpbuffers >= topsubexpbuffers ) {
-		DoubleBuffer((void **)((VOID *)(&subexpbuffers))
-		,(void **)((VOID *)(&topsubexpbuffers)),sizeof(SUBBUF),"subexpbuffers");
+		DoubleBuffer((void **)((void *)(&subexpbuffers))
+		,(void **)((void *)(&topsubexpbuffers)),sizeof(SUBBUF),"subexpbuffers");
 	}
 	subexpbuffers[insubexpbuffers].subexpnum = subexp;
 	subexpbuffers[insubexpbuffers].buffernum = AC.cbufnum;
@@ -2207,7 +2281,7 @@ dopowerd:
 
 WORD GenerateFactors(WORD n,WORD inc)
 {
-	WORD subexp;
+	int subexp;
 	int i, error = 0;
 	SBYTE *s;
 	SBYTE *tokenbuffer = (SBYTE *)Malloc1(8*n*sizeof(SBYTE),"GenerateFactors");
@@ -2229,8 +2303,8 @@ WORD GenerateFactors(WORD n,WORD inc)
 		Terminate(-1);
 	}
 	if ( subexpbuffers+insubexpbuffers >= topsubexpbuffers ) {
-		DoubleBuffer((void **)((VOID *)(&subexpbuffers))
-		,(void **)((VOID *)(&topsubexpbuffers)),sizeof(SUBBUF),"subexpbuffers");
+		DoubleBuffer((void **)((void *)(&subexpbuffers))
+		,(void **)((void *)(&topsubexpbuffers)),sizeof(SUBBUF),"subexpbuffers");
 	}
 	subexpbuffers[insubexpbuffers].subexpnum = subexp;
 	subexpbuffers[insubexpbuffers].buffernum = AC.cbufnum;
