@@ -4,7 +4,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2013 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -39,7 +39,12 @@
   	#[ inicbufs :
 */
 
-int inicbufs()
+/**
+ * Creates a new compiler buffer and returns its ID number.
+ *
+ * @return  The ID number for the new compiler buffer.
+ */
+int inicbufs(void)
 {
 	int i, num = AC.cbufList.num;
 	CBUF *C = cbuf;
@@ -49,15 +54,15 @@ int inicbufs()
 	if ( i >= num ) C = (CBUF *)FromList(&AC.cbufList);
 	else num = i;
 	C->BufferSize = 2000;
-	C->Buffer = (WORD *)Malloc1(C->BufferSize*sizeof(WORD),"compiler buffer");
+	C->Buffer = (WORD *)Malloc1(C->BufferSize*sizeof(WORD),"compiler buffer-1");
 	C->Pointer = C->Buffer;
 	C->Top = C->Buffer + C->BufferSize;
 	C->maxlhs = 10;
-	C->lhs = (WORD **)Malloc1(C->maxlhs*sizeof(WORD *),"compiler buffer");
+	C->lhs = (WORD **)Malloc1(C->maxlhs*sizeof(WORD *),"compiler buffer-2");
 	C->numlhs = 0;
 	C->mnumlhs = 0;
 	C->maxrhs = 25;
-	C->rhs = (WORD **)Malloc1(C->maxrhs*(sizeof(WORD *)+2*sizeof(LONG)+2*sizeof(WORD)),"compiler buffer");
+	C->rhs = (WORD **)Malloc1(C->maxrhs*(sizeof(WORD *)+2*sizeof(LONG)+2*sizeof(WORD)),"compiler buffer-3");
 	C->CanCommu = (LONG *)(C->rhs+C->maxrhs);
 	C->NumTerms = C->CanCommu+C->maxrhs;
 	C->numdum = (WORD *)(C->NumTerms+C->maxrhs);
@@ -76,12 +81,17 @@ int inicbufs()
   	#[ finishcbuf :
 */
 
+/**
+ * Frees a compiler buffer.
+ *
+ * @param  num  The ID number for the buffer to be freed.
+ */
 void finishcbuf(WORD num)
 {
 	CBUF *C = cbuf+num;
-	if ( C->Buffer ) M_free(C->Buffer,"compiler buffer");
-	if ( C->rhs ) M_free(C->rhs,"compiler buffer");
-	if ( C->lhs ) M_free(C->lhs,"compiler buffer");
+	if ( C->Buffer ) M_free(C->Buffer,"compiler buffer-1");
+	if ( C->rhs ) M_free(C->rhs,"compiler buffer-3");
+	if ( C->lhs ) M_free(C->lhs,"compiler buffer-2");
 	if ( C->boomlijst ) M_free(C->boomlijst,"boomlijst");
 	C->Top = C->Pointer = C->Buffer = 0;
 	C->rhs = C->lhs = 0;
@@ -98,6 +108,11 @@ void finishcbuf(WORD num)
   	#[ clearcbuf :
 */
 
+/**
+ * Clears contents in a compiler buffer.
+ *
+ * @param  num  The ID number for the buffer to be cleared.
+ */
 void clearcbuf(WORD num)
 {
 	CBUF *C = cbuf+num;
@@ -118,13 +133,26 @@ void clearcbuf(WORD num)
   	#[ DoubleCbuffer :
 */
 
-WORD *DoubleCbuffer(int num, WORD *w)
+/**
+ * Doubles a compiler buffer.
+ *
+ * @param  num  The ID number for the buffer to be doubled.
+ * @param  w    The pointer to the end (exclusive) of the current buffer. The
+ *              contents in the range of [cbuf[num].Buffer,w) will be kept.
+ */
+WORD *DoubleCbuffer(int num, WORD *w,int par)
 {
 	CBUF *C = cbuf + num;
 	LONG newsize = C->BufferSize*2;
-	WORD *newbuffer = (WORD *)Malloc1(newsize*sizeof(WORD),"compiler buffer");
+	WORD *newbuffer = (WORD *)Malloc1(newsize*sizeof(WORD),"compiler buffer-4");
 	WORD *w1, *w2;
 	LONG offset, j, i;
+	DUMMYUSE(par)
+/*
+	MLOCK(ErrorMessageLock);
+		MesPrint(" doubleCbuffer: par = %d",par);
+	MUNLOCK(ErrorMessageLock);
+*/
 	w1 = C->Buffer; w2 = newbuffer;
 	i = w - w1;
 	j = i & 7;
@@ -151,13 +179,19 @@ WORD *DoubleCbuffer(int num, WORD *w)
   	#[ AddLHS :
 */
 
+/**
+ * Adds an LHS to a compiler buffer and returns the pointer to a buffer for the
+ * new LHS.
+ *
+ * @param  num  The ID number for the buffer to get another LHS.
+ */
 WORD *AddLHS(int num)
 {
 	CBUF *C = cbuf + num;
 	C->numlhs++;
 	if ( C->numlhs >= (C->maxlhs-2) ) {
 		WORD ***ppp = &(C->lhs);	/* to avoid compiler warning */
-		if ( DoubleList((VOID ***)ppp,&(C->maxlhs),sizeof(WORD *),
+		if ( DoubleList((void ***)ppp,&(C->maxlhs),sizeof(WORD *),
 		"statement lists") ) Terminate(-1);
 	}
 	C->lhs[C->numlhs] = C->Pointer;
@@ -170,6 +204,13 @@ WORD *AddLHS(int num)
   	#[ AddRHS :
 */
 
+/**
+ * Adds an RHS to a compiler buffer and returns the pointer to a buffer for the
+ * new RHS.
+ *
+ * @param  num   The ID number for the buffer to get another RHS.
+ * @param  type  If 0, the subexpression tree will be reallocated.
+ */
 WORD *AddRHS(int num, int type)
 {
 	LONG fullsize, *lold, newsize;
@@ -237,6 +278,13 @@ restart:;
   	#[ AddNtoL :
 */
 
+/**
+ * Adds an LHS with the given data to the current compiler buffer.
+ *
+ * @param  n      The length of the data.
+ * @param  array  The data to be added.
+ * @return        0 if succeeds.
+ */
 int AddNtoL(int n, WORD *array)
 {
 	int i;
@@ -245,7 +293,7 @@ int AddNtoL(int n, WORD *array)
 	MesPrint("LH: %a",n,array);
 #endif
 	AddLHS(AC.cbufnum);
-	while ( C->Pointer+n >= C->Top ) DoubleCbuffer(AC.cbufnum,C->Pointer);
+	while ( C->Pointer+n >= C->Top ) DoubleCbuffer(AC.cbufnum,C->Pointer,1);
 	for ( i = 0; i < n; i++ ) *(C->Pointer)++ = *array++;
 	return(0);
 }
@@ -258,7 +306,15 @@ int AddNtoL(int n, WORD *array)
 	more flexible (JV). Still to do with AddNtoL.
 */
 
-int AddNtoC(int bufnum, int n, WORD *array)
+/**
+ * Adds the given data to the last LHS/RHS in a compiler buffer.
+ *
+ * @param  bufnum  The ID number for the buffer where the data will be added.
+ * @param  n       The length of the data.
+ * @param  array   The data to be added.
+ * @return         0 if succeeds.
+ */
+int AddNtoC(int bufnum, int n, WORD *array,int par)
 {
 	int i;
 	WORD *w;
@@ -266,7 +322,7 @@ int AddNtoC(int bufnum, int n, WORD *array)
 #ifdef COMPBUFDEBUG
 	MesPrint("RH: %a",n,array);
 #endif
-	while ( C->Pointer+n+1 >= C->Top ) DoubleCbuffer(bufnum,C->Pointer);
+	while ( C->Pointer+n+1 >= C->Top ) DoubleCbuffer(bufnum,C->Pointer,50+par);
 	w = C->Pointer;
 	for ( i = 0; i < n; i++ ) *w++ = *array++;
 	C->Pointer = w;
@@ -623,4 +679,3 @@ LONG numcommute(WORD *terms, LONG *numterms)
 /*
   	#] numcommute : 
 */
-
