@@ -4,7 +4,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -328,88 +328,6 @@ NoWildArg:
 
 /*
   	#] DoCanonicalize : 
-  	#[ GenTopologies :
-
-	This function has the syntax
-		topologies_(nloops,    Number of loops
-			nlegs,             Number of legs
-			setvertexsizes,    A set which tells which vertices are allowed like {3,4}.
-			set_extmomenta,    The name of a set with the external momenta
-			set_intmomenta     The name of a set with the internal momenta
-			[,options])
-	The output will be using the built in functions vertex_ and edge_.
-
-	The test for whether this function can be evaluated is in TestSub (inside
-	file proces.c) (search for the string TOPOLOGIES). 
-	This passes the code -15 in AN.TeInFun to Generator, which then calls
-	the GenTopologies routine.
-*/
-
-WORD GenTopologies(PHEAD WORD *term,WORD level)
-{
-	WORD *t1, *tt1, *tstop, *t, *tt;
-	WORD *oldworkpointer = AT.WorkPointer;
-	WORD option1 = 0, option2 = 0, setoption = -1;
-	WORD retval;
-/*
-
-	We have to go through the testing procedure again, because there could
-	be more than one topologies_ function and not all have to be expandable.
-*/
-	tstop = term+*term; tstop -= ABS(tstop[-1]);
-	tt = term+1;
-	while ( tt < tstop ) {
-		t = tt; tt = t+t[1];
-		if ( *t != TOPOLOGIES ) continue;
-		tt = t + t[1]; t1 = t + FUNHEAD;
-		if ( t1+10 > tt || *t1 != -SNUMBER || t1[1] < 0 ||      /* loops */
-			t1[2] != -SNUMBER || ( t1[3] < 0 && t1[3] != -2 ) ||/* legs */
-			t1[4] != -SETSET || Sets[t1[5]].type != CNUMBER ||  /* set vertices */
-			t1[6] != -SETSET || Sets[t1[7]].type != CVECTOR ||  /* outvectors */
-			t1[8] != -SETSET || Sets[t1[9]].type != CVECTOR ) continue;
-		tt1 = t1 + 10;
-		if ( tt1+2 <= tt && tt1[0] == -SETSET ) {
-			if ( Sets[t1[5]].last-Sets[t1[5]].first !=
-				 Sets[tt1[1]].last-Sets[tt1[1]].first ) continue;
-			setoption = tt1[1]; tt1 += 2;
-		}
-		if ( tt1+2 <= tt && tt1[0] == -SNUMBER ) { option1 = tt1[1]; tt1 += 2; }
-		if ( tt1+2 <= tt && tt1[0] == -SNUMBER ) { option2 = tt1[1]; tt1 += 2; }
-		AT.setinterntopo = t1[9];
-		AT.setexterntopo = t1[7];
-		AT.TopologiesTerm = term;
-		AT.TopologiesStart = t;
-		AT.TopologiesLevel = level;
-		AT.TopologiesOptions[0] = option1;
-		AT.TopologiesOptions[1] = option2;
-		retval = GenerateTopologies(BHEAD t1[1],t1[3],t1[5],setoption);
-		AT.WorkPointer = oldworkpointer;
-		return(retval);
-	}
-	MLOCK(ErrorMessageLock);
-	MesPrint("Internal error: topologies_ function not encountered.");
-	MUNLOCK(ErrorMessageLock);
-	return(-1);
-
-}
-
-/*
-  	#] GenTopologies : 
-  	#[ GenDiagrams :
-*/
-
-WORD GenDiagrams(PHEAD WORD *term,WORD level)
-{
-#ifdef WITHPTHREADS
-	DUMMYUSE(B)
-#endif
-	DUMMYUSE(term)
-	DUMMYUSE(level)
-	return(0);
-}
-
-/*
-  	#] GenDiagrams : 
   	#[ DoTopologyCanonicalize :
 
 	term: The term
@@ -427,7 +345,7 @@ WORD GenDiagrams(PHEAD WORD *term,WORD level)
 
 int DoTopologyCanonicalize(PHEAD WORD *term,WORD vert,WORD edge,WORD *args)
 {
-	int nvert = 0, nvert2, i, ii, jj, flipnames = 0, nparts, level, num;
+	int nvert = 0, nvert2, i, ii, jj, flipnames = 0, nparts/*, level, num */;
 	WORD *tstop, *t, *tt, *tend, *td;
 	WORD *oldworkpointer = AT.WorkPointer;
 	WORD *termcopy = TermMalloc("TopologyCanonize1");
@@ -595,20 +513,18 @@ MesPrint("connectivity: %d -- %a",i,nvert,connect+i*nvert);
 */
 	WantAddPointers(nvert+1);
 	for ( i = 0; i < nvert2; i++ ) environ[i] = 0;
-	level = 0;
+	/* level = 0; */
 	pparts = partition;
 	while ( nparts < nvert ) {
 		nparts = DoShattering(BHEAD connect,environ,pparts,nvert);
 		if ( nparts < nvert ) { /* raise level and make a copy and split a part */
 			p = pparts + 2*nvert;
-			level++; 
+			/* level++; */
 			for ( i = 0; i < 2*nvert; i++ ) p[i] = pparts[i];
 			for ( ii = 0; ii < 2*nvert; ii += 2 ) {
 				if ( p[ii+1] == 0 ) { /* found a part with more than one */
-					num = 2; i = ii+2;
-					while ( p[i+1] == 0 ) { num++; i += 2; }
-
-
+					/* num = 2; */ i = ii+2;
+					while ( p[i+1] == 0 ) { /* num++; */ i += 2; }
 					p[ii+1] = -1; pparts = p;
 					break;
 				}
