@@ -5,7 +5,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -44,7 +44,7 @@
 		par == 0 after .sort
 */
 
-WORD CleanExpr(WORD par)
+int CleanExpr(WORD par)
 {
 	GETIDENTITY
 	WORD j, n, i;
@@ -165,6 +165,7 @@ WORD CleanExpr(WORD par)
 					e_out->numdummies = e_in->numdummies;
 					e_out->numfactors = e_in->numfactors;
 					e_out->vflags = e_in->vflags;
+					e_out->uflags = e_in->uflags;
 					e_out->sizeprototype = e_in->sizeprototype;
 				}
 #ifdef PARALLELCODE
@@ -207,10 +208,11 @@ WORD CleanExpr(WORD par)
 
 */
 
-WORD PopVariables()
+int PopVariables(void)
 {
 	GETIDENTITY
-	WORD i, j, retval;
+	WORD i, j;
+	int retval;
 	UBYTE *s;
 
 	retval = CleanExpr(1);
@@ -221,6 +223,10 @@ WORD PopVariables()
 	AC.CodesFlag = AM.gCodesFlag;
 	AC.NamesFlag = AM.gNamesFlag;
 	AC.StatsFlag = AM.gStatsFlag;
+#ifdef WITHFLOAT
+	AC.MaxWeight = AM.gMaxWeight;
+	AC.DefaultPrecision = AM.gDefaultPrecision;
+#endif
 	AC.OldFactArgFlag = AM.gOldFactArgFlag;
 	AC.TokensWriteFlag = AM.gTokensWriteFlag;
 	AC.extrasymbols = AM.gextrasymbols;
@@ -374,7 +380,7 @@ WORD PopVariables()
  		#[ MakeGlobal :
 */
 
-VOID MakeGlobal()
+void MakeGlobal(void)
 {
 	WORD i, j, *pp, *mm;
 	UWORD *p, *m;
@@ -384,6 +390,10 @@ VOID MakeGlobal()
 	AM.gCodesFlag = AC.CodesFlag;
 	AM.gNamesFlag = AC.NamesFlag;
 	AM.gStatsFlag = AC.StatsFlag;
+#ifdef WITHFLOAT
+	AM.gMaxWeight = AC.MaxWeight;
+	AM.gDefaultPrecision = AC.DefaultPrecision;
+#endif
     AM.gOldFactArgFlag = AC.OldFactArgFlag;
 	AM.gextrasymbols = AC.extrasymbols;
 	if ( AM.gextrasym ) { M_free(AM.gextrasym,"extrasym"); AM.gextrasym = 0; }
@@ -471,7 +481,7 @@ VOID MakeGlobal()
  		#[ TestDrop :
 */
 
-VOID TestDrop()
+void TestDrop(void)
 {
 	EXPRESSIONS e;
 	WORD j;
@@ -571,6 +581,7 @@ restart:;
 		AS.NumOldNumFactors = 20;
 		AS.OldNumFactors = (WORD *)Malloc1(AS.NumOldNumFactors*sizeof(WORD),"numfactors pointers");
 		AS.Oldvflags = (WORD *)Malloc1(AS.NumOldNumFactors*sizeof(WORD),"vflags pointers");
+		AS.Olduflags = (WORD *)Malloc1(AS.NumOldNumFactors*sizeof(WORD),"uflags pointers");
 	}
 	else if ( nexpr >= AS.NumOldNumFactors ) {
 		oldw = AS.OldNumFactors;
@@ -580,8 +591,12 @@ restart:;
 		oldw = AS.Oldvflags;
 		AS.Oldvflags = (WORD *)Malloc1(2*AS.NumOldNumFactors*sizeof(WORD),"vflags pointers");
 		for ( i = 0; i < AS.NumOldNumFactors; i++ ) AS.Oldvflags[i] = oldw[i];
-		AS.NumOldNumFactors = 2*AS.NumOldNumFactors;
 		M_free(oldw,"vflags pointers");
+		oldw = AS.Olduflags;
+		AS.Olduflags = (WORD *)Malloc1(2*AS.NumOldNumFactors*sizeof(WORD),"uflags pointers");
+		for ( i = 0; i < AS.NumOldNumFactors; i++ ) AS.Olduflags[i] = oldw[i];
+		M_free(oldw,"uflags pointers");
+		AS.NumOldNumFactors = 2*AS.NumOldNumFactors;
 	}
 /*
 	The next is needed when we Load a .sav file with lots of expressions.
@@ -590,6 +605,7 @@ restart:;
 	AS.OldOnFile[nexpr] = e->onfile;
 	AS.OldNumFactors[nexpr] = e->numfactors;
 	AS.Oldvflags[nexpr] = e->vflags;
+	AS.Olduflags[nexpr] = e->uflags;
 }
 
 /*
@@ -597,10 +613,10 @@ restart:;
  		#[ DoExecute :
 */
 
-WORD DoExecute(WORD par, WORD skip)
+int DoExecute(WORD par, WORD skip)
 {
 	GETIDENTITY
-	WORD RetCode = 0;
+	int RetCode = 0;
 	int i, oldmultithreaded = AS.MultiThreaded;
 #ifdef PARALLELCODE
 	int j;
@@ -787,7 +803,7 @@ WORD DoExecute(WORD par, WORD skip)
 			/* The user switched off the parallel execution explicitly. */
 		}
 		else if ( AC.mparallelflag & NOPARALLEL_DOLLAR ) {
-			if ( AC.WarnFlag >= 2 ) {  /* HighWarning */
+			if ( AC.WarnFlag >= 1 ) {  /* Warning */
 				int i, j, k, n;
 				UBYTE *s, *s1;
 				s = strDup1((UBYTE *)"","NOPARALLEL_DOLLAR s");
@@ -810,7 +826,7 @@ WORD DoExecute(WORD par, WORD skip)
 					s1 = AddToString(s1,(UBYTE *)"s",0);
 				s1 = AddToString(s1,(UBYTE *)": ",0);
 				s1 = AddToString(s1,s,0);
-				HighWarning((char *)s1);
+				Warning((char *)s1);
 				M_free(s,"NOPARALLEL_DOLLAR s");
 				M_free(s1,"NOPARALLEL_DOLLAR s1");
 			}
@@ -860,11 +876,6 @@ WORD DoExecute(WORD par, WORD skip)
 		RetCode = PF_BroadcastModifiedDollars();
 		if ( RetCode ) return RetCode;
 	}
-	/* Broadcast redefined preprocessor variables. */
-	if ( AC.numpfirstnum > 0 ) {
-		RetCode = PF_BroadcastRedefinedPreVars();
-		if ( RetCode ) return RetCode;
-	}
 	/* Broadcast the list of objects converted to symbols in AM.sbufnum. */
 	if ( AC.topolynomialflag & TOPOLYNOMIALFLAG ) {
 		RetCode = PF_BroadcastCBuf(AM.sbufnum);
@@ -1036,6 +1047,44 @@ if ( AC.SwitchInArray > 0 ) {
 	  AC.MultiBracketBuf = 0;
 	}
 
+	if ( AC.SortReallocateFlag ) {
+		/* Reallocate the sort buffers to reduce resident set usage */
+		/* AT.SS is the same as AT.S0 here */
+		SORTING* S = AT.S0;
+		M_free(S->lBuffer, "SortReallocate lBuffer+sBuffer");
+		S->lBuffer = Malloc1(sizeof(*(S->lBuffer))*(S->LargeSize+S->SmallEsize), "SortReallocate lBuffer+sBuffer");
+		S->lTop = S->lBuffer+S->LargeSize;
+		S->sBuffer = S->lTop;
+		if ( S->LargeSize == 0 ) { S->lBuffer = 0; S->lTop = 0; }
+		S->sTop = S->sBuffer + S->SmallSize;
+		S->sTop2 = S->sBuffer + S->SmallEsize;
+		S->sHalf = S->sBuffer + (LONG)((S->SmallSize+S->SmallEsize)>>1);
+
+#ifdef WITHPTHREADS
+		/* We have to re-set the pointers into master lBuffer in the SortBlocks */
+		UpdateSortBlocks(AM.totalnumberofthreads-1);
+
+		/* The SortBots do not have a real sort buffer to reallocate. */
+		/* AB[0] has been reallocated above already. */
+		for ( i = 1; i < AM.totalnumberofthreads; i++ ) {
+			SORTING* S = AB[i]->T.S0;
+			M_free(S->lBuffer, "SortReallocate lBuffer+sBuffer");
+			S->lBuffer = Malloc1(sizeof(*(S->lBuffer))*(S->LargeSize+S->SmallEsize), "SortReallocate lBuffer+sBuffer");
+			S->lTop = S->lBuffer+S->LargeSize;
+			S->sBuffer = S->lTop;
+			if ( S->LargeSize == 0 ) { S->lBuffer = 0; S->lTop = 0; }
+			S->sTop = S->sBuffer + S->SmallSize;
+			S->sTop2 = S->sBuffer + S->SmallEsize;
+			S->sHalf = S->sBuffer + (LONG)((S->SmallSize+S->SmallEsize)>>1);
+		}
+#endif
+	}
+	if ( AC.SortReallocateFlag == 2 ) {
+		/* The Flag was set for a single module by the preprocessor #sortreallocate,
+		   so turn it off again. */
+		AC.SortReallocateFlag = 0;
+	}
+
 	return(RetCode);
 }
 
@@ -1060,7 +1109,7 @@ if ( AC.SwitchInArray > 0 ) {
 	15-oct-1991
 */
 
-WORD PutBracket(PHEAD WORD *termin)
+int PutBracket(PHEAD WORD *termin)
 {
 	GETBIDENTITY
 	WORD *t, *t1, *b, i, j, *lastfun;
@@ -1070,7 +1119,9 @@ WORD PutBracket(PHEAD WORD *termin)
 	WORD *bbb = 0, *bind, *binst = 0, bwild = 0, *bss = 0, *bns = 0, bset = 0;
 	term1 = AT.WorkPointer+1;
 	term2 = (WORD *)(((UBYTE *)(term1)) + AM.MaxTer);
-	if ( ( (WORD *)(((UBYTE *)(term2)) + AM.MaxTer) ) > AT.WorkTop ) return(MesWork());
+	if ( ( (WORD *)(((UBYTE *)(term2)) + AM.MaxTer) ) > AT.WorkTop ) {
+		MesWork();
+	}
 	if ( AR.BracketOn < 0 ) {
 		t2 = term1; t1 = term2;		/* AntiBracket */
 	}
@@ -1086,6 +1137,9 @@ WORD PutBracket(PHEAD WORD *termin)
 
 	t = termin; tStopa = t + *t; i = *(t + *t -1); i = ABS(i);
 	if ( AR.PolyFun && AT.PolyAct ) tStop = termin + AT.PolyAct;
+#ifdef WITHFLOAT
+	else if ( AT.FloatPos ) tStop = termin + AT.FloatPos;
+#endif
 	else tStop = tStopa - i;
 	t++;
 	if ( AR.BracketOn < 0 ) {
@@ -1569,7 +1623,7 @@ NextSymbol2:;
  		#[ SpecialCleanup :
 */
 
-VOID SpecialCleanup(PHEAD0)
+void SpecialCleanup(PHEAD0)
 {
 	GETBIDENTITY
 	if ( AT.previousEfactor ) M_free(AT.previousEfactor,"Efactor cache");
@@ -1583,7 +1637,7 @@ VOID SpecialCleanup(PHEAD0)
 
 #ifndef WITHPTHREADS
 
-void SetMods()
+void SetMods(void)
 {
 	int i, n;
 	if ( AN.cmod != 0 ) M_free(AN.cmod,"AN.cmod");
@@ -1601,7 +1655,7 @@ void SetMods()
 
 #ifndef WITHPTHREADS
 
-void UnSetMods()
+void UnSetMods(void)
 {
 	if ( AN.cmod != 0 ) M_free(AN.cmod,"AN.cmod");
 	AN.cmod = 0;
@@ -1800,13 +1854,17 @@ int GetFirstBracket(WORD *term, int num)
  		#[ GetFirstTerm :
 */
 
-int GetFirstTerm(WORD *term, int num)
+/**
+ * Gets the first term of an expression. Routine should be thread safe.
+ *
+ * @param[out] term Pointer to the buffer where the term should be written.
+ * @param[in] num   The expression number from which to get the term.
+ * @param[in] pre   Denotes a pre-processor call (1) or not (0). Used to
+ *                  determine the correct source file for the expression.
+ * @return          First WORD of term, a negative value denotes an error.
+ */
+int GetFirstTerm(WORD *term, int num, int pre)
 {
-/*
-		Gets the first term of the expression 'num'
-		Puts it in term.
-		Routine should be thread-safe
-*/
 	GETIDENTITY
 	POSITION position, oldposition;
 	RENUMBER renumber;
@@ -1849,9 +1907,27 @@ int GetFirstTerm(WORD *term, int num)
 		}
 		else {
 			AR.GetOneFile = 0;
-			if ( Expressions[num].replace == NEWLYDEFINEDEXPRESSION )
-			     fi = AR.outfile;
-			else fi = AR.infile;
+			if ( Expressions[num].replace == NEWLYDEFINEDEXPRESSION ) {
+				/* During execution, if the expression has already been processed it
+				   will be in the outfile. If it has not, the usage is illegal according
+				   to the manual, though no error is given. */
+				if ( pre == 0 ) { fi = AR.outfile; }
+				/* During preprocessing, the expression certainly has not been processed
+				   yet. Print an error and terminate. */
+				else {
+					MesPrint("&isnumerical: expression is not yet defined!");
+					SETERROR(-1);
+				}
+			}
+			else {
+				/* During execution, we should use the definition as stored at the end
+				   of the previous module. This is in the infile. */
+				if ( pre == 0 ) { fi = AR.infile; }
+				/* During preprocessing, this function is called before the RevertScratch
+				   at the beginning of this module's execution. Thus the expressions are
+				   in the outfile of the previous module. */
+				else { fi = AR.outfile; }
+			}
 		}
 		if ( fi->handle >= 0 ) {
 			PUTZERO(oldposition);
@@ -2324,7 +2400,7 @@ LONG SizeOfExpression(WORD num)
  		#[ UpdatePositions :
 */
 
-void UpdatePositions()
+void UpdatePositions(void)
 {
 	EXPRESSIONS e = Expressions;
 	POSITION *old;
@@ -2354,12 +2430,17 @@ void UpdatePositions()
 			oldw = AS.Oldvflags;
 			AS.Oldvflags = (WORD *)Malloc1(NumExpressions*sizeof(WORD),"vflags pointers");
 			for ( i = 0; i < AS.NumOldNumFactors; i++ ) AS.Oldvflags[i] = oldw[i];
-			AS.NumOldNumFactors = NumExpressions;
 			M_free(oldw,"vflags pointers");
+			oldw = AS.Olduflags;
+			AS.Olduflags = (WORD *)Malloc1(NumExpressions*sizeof(WORD),"uflags pointers");
+			for ( i = 0; i < AS.NumOldNumFactors; i++ ) AS.Olduflags[i] = oldw[i];
+			M_free(oldw,"uflags pointers");
+			AS.NumOldNumFactors = NumExpressions;
 		}
 		else {
 			AS.OldNumFactors = (WORD *)Malloc1(NumExpressions*sizeof(WORD),"numfactors pointers");
 			AS.Oldvflags = (WORD *)Malloc1(NumExpressions*sizeof(WORD),"vflags pointers");
+			AS.Olduflags = (WORD *)Malloc1(NumExpressions*sizeof(WORD),"uflags pointers");
 			AS.NumOldNumFactors = NumExpressions;
 		}
 	}
@@ -2367,6 +2448,7 @@ void UpdatePositions()
 		AS.OldOnFile[i] = e[i].onfile;
 		AS.OldNumFactors[i] = e[i].numfactors;
 		AS.Oldvflags[i] = e[i].vflags;
+		AS.Olduflags[i] = e[i].uflags;
 	}
 }
 
