@@ -4,7 +4,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -149,10 +149,6 @@ Uncomment to get a self-consistent program:
 #else
 #define FORM_INLINE inline
 #endif
-/*
-	from declare.h:
-*/
-#define VOID void
 
 /* 
 	From form3.h:
@@ -169,7 +165,7 @@ extern int (*setKillModeForExternalChannel)(int signum, int sentToWholeGroup);
 #include "form3.h"
 #endif /*ifdef SELFTEST ... else*/
 /*
-pid_t  getExternalChannelPid(VOID);
+pid_t  getExternalChannelPid(void);
 */
 /*
   	#] Selftest initializing: 
@@ -213,7 +209,7 @@ int setKillModeForExternalChannelFailure(int signum, int sentToWholeGroup)
 	return(-1);
 }/*setKillModeForExternalChannelFailure*/
 
-int getcFromExtChannelFailure()
+int getcFromExtChannelFailure(void)
 {
 	return(-2);
 }/*getcFromExtChannelFailure*/
@@ -223,7 +219,7 @@ int (*setTerminatorForExternalChannel)(char *buffer) =
 	&setTerminatorForExternalChannelFailure;
 int (*setKillModeForExternalChannel)(int signum, int sentToWholeGroup) =
 	&setKillModeForExternalChannelFailure;
-int (*getcFromExtChannel)() = &getcFromExtChannelFailure;
+int (*getcFromExtChannel)(void) = &getcFromExtChannelFailure;
 #endif
 /*
   	#] FailureFunctions: 
@@ -236,8 +232,8 @@ int openExternalChannel(UBYTE *cmd, int daemonize, UBYTE *shellname, UBYTE *stde
 int initPresetExternalChannels(UBYTE *theline, int thetimeout) { DUMMYUSE(theline); DUMMYUSE(thetimeout); return(-1); };
 int closeExternalChannel(int n) { DUMMYUSE(n); return(-1); };
 int selectExternalChannel(int n) { DUMMYUSE(n); return(-1); };
-int getCurrentExternalChannel() { return(0); };
-void closeAllExternalChannels() {};
+int getCurrentExternalChannel(void) { return(0); };
+void closeAllExternalChannels(void) {};
 #else /*ifndef WITHEXTERNALCHANNEL*/
 /*
   	#] Stubs : 
@@ -284,7 +280,6 @@ typedef struct ExternalChannel {
 	int fsend;       /*stdin of the external process*/
 	int killSignal; /*signal to kill*/
 	int daemonize;/*0 --neither setsid nor daemonize, !=0 -- full daemonization*/
-	PADPOINTER(0,3,0,0);
 } EXTHANDLE;
 
 
@@ -343,7 +338,7 @@ int PutPreVar(UBYTE *a,UBYTE *b,UBYTE *c,int i)
 */
 
 /*Initialize one cell of handler:*/
-static FORM_INLINE VOID extHandlerInit(EXTHANDLE *h)
+static FORM_INLINE void extHandlerInit(EXTHANDLE *h)
 {
 	h->pid=-1;
 	h->gpid=-1;
@@ -356,7 +351,7 @@ static FORM_INLINE VOID extHandlerInit(EXTHANDLE *h)
 }/*extHandlerInit*/
 
 /* Copies each field of handler:*/
-static FORM_INLINE VOID extHandlerSwallowCopy(EXTHANDLE *to, EXTHANDLE *from)
+static FORM_INLINE void extHandlerSwallowCopy(EXTHANDLE *to, EXTHANDLE *from)
 {
 	to->pid=from->pid;
 	to->gpid=from->gpid;
@@ -376,7 +371,7 @@ static FORM_INLINE VOID extHandlerSwallowCopy(EXTHANDLE *to, EXTHANDLE *from)
 
 /*Allocates memory for fields of handler which have no fixed
  storage size and initializes some fields:*/
-static FORM_INLINE VOID 
+static FORM_INLINE void 
 extHandlerAlloc(EXTHANDLE *h, char *cmd, char *shellname, char *stderrname)
 {
 	h->IBfill=h->IBfull=h->INbuf=
@@ -399,7 +394,7 @@ extHandlerAlloc(EXTHANDLE *h, char *cmd, char *shellname, char *stderrname)
 }/*extHandlerAlloc*/
 
 /*Disallocates dynamically allocated fields of a handler:*/
-static FORM_INLINE VOID extHandlerFree(EXTHANDLE *h)
+static FORM_INLINE void extHandlerFree(EXTHANDLE *h)
 {
 	if(h->stderrname) M_free(h->stderrname,"External channel stderr name");
 	if(h->shellname) M_free(h->shellname,"External channel shell name");
@@ -410,7 +405,7 @@ static FORM_INLINE VOID extHandlerFree(EXTHANDLE *h)
 }/*extHandlerFree*/
 /* Closes all descriptors, kills the external process, frees all internal fields,
 BUT does NOT free the main container:*/
-static VOID destroyExternalChannel(EXTHANDLE *h)
+static void destroyExternalChannel(EXTHANDLE *h)
 {
 	/*Note, this function works in parallel mode correctly, see comments below.*/
 
@@ -514,7 +509,7 @@ int oldflags = fcntl (desc, F_GETFD, 0);
 /* Adds the integer fd to the array fifo of length top+1 so that 
 the array is ascendantly ordered. It is supposed that all 0 -- top-1
 elements in the array are already ordered:*/
-static VOID pushDescriptor(int *fifo, int top, int fd)
+static void pushDescriptor(int *fifo, int top, int fd)
 {
 	if ( top == 0 ) {
 		fifo[top] = fd;
@@ -536,7 +531,7 @@ static VOID pushDescriptor(int *fifo, int top, int fd)
 
 /*Close all descriptors greater or equal than startFrom except those
   listed in the ascendantly ordered array usedFd of length top:*/
-static FORM_INLINE VOID closeAllDescriptors(int startFrom, int *usedFd, int top)
+static FORM_INLINE void closeAllDescriptors(int startFrom, int *usedFd, int top)
 {
 int n,maxfd;
 	for(n=0;n<top; n++){
@@ -553,7 +548,7 @@ int n,maxfd;
 
 typedef int L_APIPE[2];
 /*Closes both pipe descriptors if not -1:*/
-static VOID closepipe(L_APIPE *thepipe)
+static void closepipe(L_APIPE *thepipe)
 {
   if( (*thepipe)[0] != -1) close ((*thepipe)[0]);
   if( (*thepipe)[1] != -1) close ((*thepipe)[1]);
@@ -625,7 +620,7 @@ static  FORM_INLINE char *addStr(char *to, char *from)
 
 
 /*Try to write (atomically) short buffer (of length count) to fd.
-  timeout is a timeout in millisecs. Returns number of writen bytes or -1:*/
+  timeout is a timeout in millisecs. Returns number of written bytes or -1:*/
 static FORM_INLINE ssize_t writeSome(int fd, char *buf, size_t count, int timeout)
 {
 	ssize_t res = 0;
@@ -636,7 +631,7 @@ static FORM_INLINE ssize_t writeSome(int fd, char *buf, size_t count, int timeou
 
 	/*Add O_NONBLOCK:*/
 	fcntl(fd,F_SETFL, flags | O_NONBLOCK);
-	/* important -- in order to avoid blocking of short rceiver buffer*/
+	/* important -- in order to avoid blocking of short receiver buffer*/
 
 	do{
 		FD_ZERO(&wfds);
@@ -674,7 +669,7 @@ static FORM_INLINE ssize_t writeSome(int fd, char *buf, size_t count, int timeou
 
 /*Try to read short buffer (of length not more than count)
   from fd. timeout is a timeout in millisecs. Returns number 
-  of writen bytes or -1: */
+  of written bytes or -1: */
 static FORM_INLINE ssize_t readSome(int fd, char *buf, size_t count, int timeout)
 {
 	ssize_t res = 0;
@@ -772,7 +767,7 @@ int setKillModeForExternalChannelOk(int signum, int sentToWholeGroup)
 returns EOF. If the external process is finished completely, the function closes 
 the channel (and returns EOF). If the external process was finished, the function
 returns EOF:*/
-int getcFromExtChannelOk()
+int getcFromExtChannelOk(void)
 {
 mysighandler_t oldPIPE = 0;
 EXTHANDLE *h;
@@ -1223,13 +1218,13 @@ static FORM_INLINE void *createExternalChannel(
 	int gpid = 0;
 	ECINFOSTRUCT *psetInfo;
 #ifdef WITHMPI
-	char statusbuf[2]={'\0','\0'};/*'\0' if run_cmd retuns ok, '!' othervise.*/
+	char statusbuf[2]={'\0','\0'};/*'\0' if run_cmd retuns ok, '!' otherwise.*/
 #endif
 	extHandlerInit(h);
 
 	h->pid=0;
 
-	if( cmd==NULL ){/*Instead of strting a new command, use preset channel:*/
+	if( cmd==NULL ){/*Instead of starting a new command, use preset channel:*/
 		psetInfo=(ECINFOSTRUCT *)shellname;
 		shellname=NULL;
 		h->killSignal=0;
@@ -1519,7 +1514,7 @@ int closeExternalChannel(int n)
   	#] closeExternalChannel : 
   	#[ closeAllExternalChannels :
 */
-void closeAllExternalChannels()
+void closeAllExternalChannels(void)
 {
 int i;
 	for(i=0; i<externalChannelsListFill; i++)
@@ -1542,7 +1537,7 @@ int i;
   	#[ getExternalChannelPid :
 */
 #ifdef SELFTEST
-pid_t getExternalChannelPid()
+pid_t getExternalChannelPid(void)
 {
   if(externalChannelsListTop!=0)
 		return(externalChannelsListTop ->pid);
@@ -1554,7 +1549,7 @@ pid_t getExternalChannelPid()
   	#[ getCurrentExternalChannel :
 */
 
-int getCurrentExternalChannel()
+int getCurrentExternalChannel(void)
 {
 
 	if ( externalChannelsListTop != 0 )
