@@ -4,7 +4,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2013 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -557,7 +557,7 @@ int FactorInExpr(PHEAD WORD *term, WORD level)
 	Load the first term in the workspace
 */
 	if ( GetTerm(BHEAD oldwork) == 0 ) {
-		SetScratch(file,&oldposition); /* We still need this untill Processor is clean */
+		SetScratch(file,&oldposition); /* We still need this until Processor is clean */
 		AR.DeferFlag = olddeferflag;
 		oldwork[0] = 4; oldwork[1] = 1; oldwork[2] = 1; oldwork[3] = 3; 
 		goto Complete;
@@ -870,4 +870,3 @@ onerror:
 /*
   	#] FactorInExpr : 
 */
-
