@@ -8,7 +8,7 @@
 
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -31,7 +31,7 @@
  *   You should have received a copy of the GNU General Public License along
  *   with FORM.  If not, see <http://www.gnu.org/licenses/>.
  */
-/* #] License : */
+/* #] License : */ 
  
 /**
  *	The next macros were introduced when TFORM was programmed. In the case of
@@ -161,41 +161,26 @@
 #define STATEMENT     4
 #define TOOUTPUT      5
 #define ATENDOFMODULE 6
+#define MIXED2        8  /* unused */
 #define MIXED         9
 
 /*
 	The typedefs are to allow the compilers to do better error checking.
 */
 
-/*
-	icc doesn't like the typedef void VOID; Neither does g++ on the apple
-	Hence we work the old fashioned way:
-*/
-#define VOID void
-
-#ifdef ANSI
-typedef VOID (*PVFUNWP)(WORD *);
-#ifdef INTELCOMPILER
-typedef VOID (*PVFUNV)();
-typedef int (*CFUN)();
-#else
-typedef VOID (*PVFUNV)(VOID);
-typedef int (*CFUN)(VOID);
-#endif
+typedef void (*PVFUNWP)(WORD *);
+typedef void (*PVFUNV)(void);
+typedef int (*CFUN)(void);
 typedef int (*TFUN)(UBYTE *);
 typedef int (*TFUN1)(UBYTE *,int);
-#else
-typedef VOID (*PVFUNWP)();
-typedef VOID (*PVFUNV)();
-typedef int (*CFUN)();
-typedef int (*TFUN)();
-typedef int (*TFUN1)();
-#endif
 
+/*
+	Flags used in the compiler's KEYWORD tables.
+*/
 
 #define NOAUTO 0
-#define PARTEST 1
-#define WITHAUTO 2
+#define PARTEST 1	/* parentheses test */
+#define WITHAUTO 2	/* auto-declaration */
 
 #define ALLVARIABLES -1
 #define SYMBOLONLY 1
@@ -229,6 +214,7 @@ typedef int (*TFUN1)();
 #define CDOLLAR 10
 #define CDUBIOUS 11
 #define CRANGE 12
+#define CMODEL 13
 #define CVECTOR1 21
 #define CDOUBLEDOT 22
 
@@ -278,6 +264,7 @@ typedef int (*TFUN1)();
 #define TSETNUM -38
 #define TSGAMMA -39
 #define TSETDOL -40
+#define TFLOAT -41
 
 #define TYPEISFUN 0
 #define TYPEISSUB 1
@@ -320,6 +307,18 @@ typedef int (*TFUN1)();
 #define SORTPOWERFIRST 2
 #define SORTANTIPOWER 3
 
+/* These control the output of WriteStats. The different sort
+ * types have differently formatted stats. These variables should
+ * not have arbitrary values since they are also used as array
+ * indices by WriteStats. */
+#define STATSSPLITMERGE 0
+#define STATSMERGETOFILE 1
+#define STATSPOSTSORT 2
+/* CHECKLOGTYPE implies that statistics will not be printed in
+ * the log file if AM.LogType = 1 (when running with -ll or -F). */
+#define NOCHECKLOGTYPE 0
+#define CHECKLOGTYPE 1
+
 #define NMIN4SHIFT 4
 /*
 	The next are the main codes.
@@ -452,14 +451,31 @@ typedef int (*TFUN1)();
 #define MOEBIUS 106
 #define TOPOLOGIES 107
 #define DIAGRAMS 108
-#define VERTEX 109
-#define EDGE 110
-/*#define ALLWILD 109 ???? */
-#define SIZEOFFUNCTION 111
+#define TOPO 109
+#define NODEFUNCTION 110
+#define EDGE 111
+#define SIZEOFFUNCTION 112
+#define BLOCK 113
+#define ONEPI 114
+#define PHI 115
+#define FLOATFUN 116
+#define TOFLOAT 117
+#define TORAT 118
+#define MZV 119
+#define EULER 120
+#define MZVHALF 121
+#define AGMFUNCTION 122
+#define GAMMAFUN 123
+#define EXPFUNCTION 124
+#define HPLFUNCTION 125
+#define MPLFUNCTION 126
+#define MAXBUILTINFUNCTION 126
 
-#define MAXBUILTINFUNCTION 111
 #define FIRSTUSERFUNCTION 150
 
+#define ALLFUNCTIONS -1
+#define ALLMZVFUNCTIONS -2
+
 /*
 	Note: if we add a builtin table we have to look also inside names.c
 	in the routine Globalize because there we assume there does not exist
@@ -475,8 +491,10 @@ typedef int (*TFUN1)();
 #define DIMENSIONSYMBOL 6
 #define FACTORSYMBOL 7
 #define SEPARATESYMBOL 8
+#define EESYMBOL 9
+#define EMSYMBOL 10
 
-#define BUILTINSYMBOLS 9
+#define BUILTINSYMBOLS 11
 #define FIRSTUSERSYMBOL 20
 
 #define BUILTININDICES 1
@@ -567,6 +585,18 @@ typedef int (*TFUN1)();
 #define TYPECANONICALIZE 80
 #define TYPESWITCH 81
 #define TYPEENDSWITCH 82
+#define TYPESETUSERFLAG 83
+#define TYPECLEARUSERFLAG 84
+#define TYPEALLLOOPS 85
+#define TYPEALLPATHS 86
+#ifdef WITHFLOAT
+#define TYPEEVALUATE 87
+#define TYPEEXPAND 88
+#define TYPETOFLOAT 89
+#define TYPETORAT 90
+#define TYPESTRICTROUNDING 91
+#define TYPECHOP 92
+#endif
 
 /*
 	The codes for the 'operations' that are part of TYPEOPERATION.
@@ -610,6 +640,10 @@ typedef int (*TFUN1)();
 #define NUMTOSYM 21
 #define NUMTOIND 22
 #define NUMTOSUB 23
+/*
+	Flag or-ed with ARGTOARG to give the number of arguments.
+*/
+#define EATTENSOR 0x2000
 
 /*
 	Dirty flags (introduced when functions got a field with a dirty flag)
@@ -692,8 +726,8 @@ typedef int (*TFUN1)();
  */
 
 /*@{*/
+#define VERTEXFUNCTION -1
 #define GENERALFUNCTION 0
-#define FASTFUNCTION 1
 #define TENSORFUNCTION 2
 #define GAMMAFUNCTION 4
 /*@}*/
@@ -748,7 +782,6 @@ typedef int (*TFUN1)();
 #define RATIONALMODE 0
 
 #define NUMSPECSETS 10
-#define EATTENSOR 0x2000
 
 #define ISZERO 1
 #define ISUNMODIFIED 2
@@ -810,6 +843,8 @@ typedef int (*TFUN1)();
 #define IFDOLLAREXTRA 8
 #define IFISFACTORIZED 9
 #define IFOCCURS 10
+#define IFUSERFLAG 11
+#define IFFLOATNUMBER 12
 #define GREATER	0
 #define GREATEREQUAL 1
 #define LESS 2
@@ -977,6 +1012,14 @@ typedef int (*TFUN1)();
 #define DROPARG     19
 #define SELECTARG   20
 #define DEDUPARG    21
+#define ZTOHARG     22
+#define HTOZARG     23
+/*
+#define ZTOSARG     24
+#define STOZARG     25
+#define HTOSARG     26
+#define STOHARG     27
+*/
 
 #define BASECODE 1
 #define YESLYNDON 1
@@ -1048,3 +1091,36 @@ typedef int (*TFUN1)();
 #define DENSETABLE 1
 #define SPARSETABLE 0
 
+// Diagram generator flags. They should be powers of two, since they are added
+// to pass to diagrams and masked in diawrap.cc to check the flags.
+// We use a "stringified" version of these when defining the FORM preprocessor
+// variables, so can't neatly use "1<<10" etc here.
+#define TOPOLOGIESONLY  1
+#define WITHOUTNODES    2
+#define WITHEDGES       4
+#define WITHBLOCKS      8
+#define WITHONEPISETS   16
+#define WITHSYMMETRIZEI 32
+#define WITHSYMMETRIZEF 64
+// This is not an "option" preprocessor var but is set by "external" particle definitions
+#define CHECKEXTERN     128
+// The "qgraf compatible" filtering flags:
+#define ONEPARTI        256
+#define ONEPARTR        512
+#define ONSHELL         1024
+#define OFFSHELL        2048
+#define NOSIGMA         4096
+#define SIGMA           8192
+#define NOSNAIL         16384
+#define SNAIL           32768
+#define NOTADPOLE       65536
+#define TADPOLE         131072
+#define SIMPLE          262144
+#define NOTSIMPLE       524288
+#define BIPART          1048576
+#define NONBIPART       2097152
+#define CYCLI           4194304
+#define CYCLR           8388608
+#define FLOOP           16777216
+#define NOTFLOOP        33554432
+
