@@ -4,7 +4,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -277,7 +277,7 @@ InVe:
 		Note that the whole setup asks for recursions.
 */
 
-WORD DoIfStatement(PHEAD WORD *ifcode, WORD *term)
+int DoIfStatement(PHEAD WORD *ifcode, WORD *term)
 {
 	GETBIDENTITY
 	WORD *ifstop, *ifp;
@@ -365,6 +365,10 @@ WORD DoIfStatement(PHEAD WORD *ifcode, WORD *term)
 					}
 				}
 				return(0);
+			case IFUSERFLAG:
+				if ( ( Expressions[AR.CurExpr].uflags & (1 << ifp[2]) ) != 0 )
+					return(1);
+				return(0);
 			default:
 /*
 				Now we have a subexpression. Test first for one with a single item.
@@ -402,6 +406,15 @@ WORD DoIfStatement(PHEAD WORD *ifcode, WORD *term)
 				cc = (UWORD *)(ifp + 3);
 				for ( i = 0; i < j; i++ ) coef2[i] = cc[i];
 				break;
+#ifdef WITHFLOAT
+			case IFFLOATNUMBER:
+/*
+				The sloppy solution is: Convert to rational.
+				This way we can write it over coef2,ncoef2
+*/
+				ncoef2 = FloatFunToRat(BHEAD coef2,ifp);
+				break;
+#endif
 			case MATCH:
 			case TYPEIF:
 				coef2[0] = HowMany(BHEAD ifp,term);
@@ -709,6 +722,15 @@ generic:;
 					coef2[1] = 1;
 				}
 				break;
+			case IFUSERFLAG:
+				{
+					ncoef2 = 0;
+					if ( ( Expressions[AR.CurExpr].uflags & (1 << ifp[2]) ) != 0 )
+						ncoef2 = 1;
+					coef2[0] = ncoef2;
+					coef2[1] = 1;
+				}
+				break;
 			default:
 				break;
 		}
@@ -1008,7 +1030,7 @@ WORD HowMany(PHEAD WORD *ifcode, WORD *term)
  		#[ DoubleIfBuffers :
 */
 
-VOID DoubleIfBuffers()
+void DoubleIfBuffers(void)
 {
 	int newmax, i;
 	WORD *newsumcheck;
@@ -1110,7 +1132,7 @@ SWITCHTABLE *FindCase(WORD nswitch, WORD ncase)
  		#[ DoubleSwitchBuffers :
 */
 
-int DoubleSwitchBuffers()
+int DoubleSwitchBuffers(void)
 {
 	int newmax, i;
 	SWITCH *newarray;
@@ -1159,7 +1181,7 @@ int DoubleSwitchBuffers()
 		Sorts an array of WORDs. No adding of equal objects.
 */
 
-VOID SwitchSplitMergeRec(SWITCHTABLE *array,WORD num,SWITCHTABLE *auxarray)
+void SwitchSplitMergeRec(SWITCHTABLE *array,WORD num,SWITCHTABLE *auxarray)
 {
 	WORD n1,n2,i,j,k;
 	SWITCHTABLE *t1,*t2, t;
@@ -1188,7 +1210,7 @@ VOID SwitchSplitMergeRec(SWITCHTABLE *array,WORD num,SWITCHTABLE *auxarray)
 */
 }
 
-VOID SwitchSplitMerge(SWITCHTABLE *array,WORD num)
+void SwitchSplitMerge(SWITCHTABLE *array,WORD num)
 {
 	SWITCHTABLE *auxarray = (SWITCHTABLE *)Malloc1(sizeof(SWITCHTABLE)*num/2,"SwitchSplitMerge");
 	SwitchSplitMergeRec(array,num,auxarray);
