@@ -6,7 +6,7 @@
 
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -93,6 +93,8 @@ FIXEDGLOBALS FG = {
 	,(UBYTE *)"1"
 	,(WORD)0
 	,(WORD)0
+	,(WORD)0
+	,(WORD)0
 
 /*  ASCII table of character types. Note that on some computers this
     table may be different from the ASCII table.
@@ -231,12 +233,27 @@ static struct fixedfun {
 	,{"perm_"       ,1 ,0             ,0               ,0} /* PERMUTATIONS */
 	,{"partitions_" ,1 ,0             ,0               ,0} /* PARTITIONS */
 	,{"mul_"        ,0 ,0             ,0               ,0} /* MULFUNCTION */
-	,{"moebius_"    ,0 ,0             ,0               ,0} /* MOEBIUS */
+    ,{"moebius_"    ,0 ,0             ,0               ,0} /* MOEBIUS */
 	,{"topologies_" ,0 ,0             ,0               ,0} /* TOPOLOGIES */
 	,{"diagrams_"   ,0 ,0             ,0               ,0} /* DIAGRAMS */
+	,{"topo_"       ,0 ,0             ,0               ,0} /* TOPO */
 	,{"node_"       ,0 ,0             ,0               ,0} /* VERTEX */
 	,{"edge_"       ,0 ,0             ,0               ,0} /* EDGE */
     ,{"sizeof_"     ,0 ,0             ,0               ,0} /* SIZEOFFUNCTION */
+	,{"block_"      ,0 ,0             ,0               ,0} /* BLOCK */
+	,{"onepi_"      ,0 ,0             ,0               ,0} /* ONEPI */
+	,{"phi_"        ,0 ,VERTEXFUNCTION,0               ,0} /* PHI */
+	,{"float_"      ,0 ,0             ,0               ,0} /* FLOATFUN */
+	,{"tofloat_"    ,0 ,0             ,0               ,0} /* TOFLOAT */
+	,{"torat_"      ,0 ,0             ,0               ,0} /* TORAT */
+	,{"mzv_"        ,0 ,0             ,0               ,0} /* MZV */
+	,{"euler_"      ,0 ,0             ,0               ,0} /* EULER */
+	,{"mzvhalf_"    ,0 ,0             ,0               ,0} /* MZVHALF */
+	,{"agm_"        ,0 ,0             ,0               ,0} /* AGMFUNCTION */
+	,{"gamma_"      ,0 ,0             ,0               ,0} /* GAMMAFUN */
+	,{"eexp_"       ,0 ,0             ,0               ,0} /* EXPFUNCTION */
+	,{"hpl_"        ,0 ,0             ,0               ,0} /* HPLFUNCTION */
+	,{"mpl_"        ,0 ,0             ,0               ,0} /* MPLFUNCTION */
 };
 
 FIXEDSET fixedsets[] = {
