@@ -6,7 +6,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2013 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -29,7 +29,7 @@
  *   You should have received a copy of the GNU General Public License along
  *   with FORM.  If not, see <http://www.gnu.org/licenses/>.
  */
-/* #] License : */
+/* #] License : */ 
 /*
   	#[ Includes :
 
@@ -45,7 +45,7 @@
 int withoutflush = 0;
 
 /*
-  	#] Includes :
+  	#] Includes : 
   	#[ Variables :
 */
  
@@ -53,12 +53,12 @@ static INDEXBLOCK scratchblock;
 static NAMESBLOCK scratchnamesblock;
  
 #define CFD(y,s,type,x,j) for(x=0,j=0;j<((int)sizeof(type));j++) \
-	x=(x<<8)+((*s++)&0x00FF); y=x;
+	{x=(x<<8)+((*s++)&0x00FF);} y=x;
 #define CTD(y,s,type,x,j) x=y;for(j=sizeof(type)-1;j>=0;j--){s[j]=x&0xFF; \
                       x>>=8;} s += sizeof(type);
 
 /*
-  	#] Variables :
+  	#] Variables : 
   	#[ Utilities :
  		#[ minosread :
 */
@@ -76,7 +76,7 @@ int minosread(FILE *f,char *buffer,MLONG size)
 }
 
 /*
- 		#] minosread :
+ 		#] minosread : 
  		#[ minoswrite :
 */
 
@@ -94,7 +94,7 @@ int minoswrite(FILE *f,char *buffer,MLONG size)
 }
 
 /*
- 		#] minoswrite :
+ 		#] minoswrite : 
  		#[ str_dup :
 */
 
@@ -113,7 +113,7 @@ char *str_dup(char *str)
 }
 
 /*
- 		#] str_dup :
+ 		#] str_dup : 
  		#[ convertblock :
 */
 
@@ -164,7 +164,7 @@ void convertblock(INDEXBLOCK *in,INDEXBLOCK *out,int mode)
 }
 
 /*
- 		#] convertblock :
+ 		#] convertblock : 
  		#[ convertnamesblock :
 */
 
@@ -190,7 +190,7 @@ void convertnamesblock(NAMESBLOCK *in,NAMESBLOCK *out,int mode)
 }
 
 /*
- 		#] convertnamesblock :
+ 		#] convertnamesblock : 
  		#[ convertiniinfo :
 */
 
@@ -218,16 +218,17 @@ void convertiniinfo(INIINFO *in,INIINFO *out,int mode)
 }
 
 /*
- 		#] convertiniinfo :
+ 		#] convertiniinfo : 
  		#[ LocateBase :
 */
 
-FILE *LocateBase(char **name, char **newname)
+FILE *LocateBase(char **name, char **newname, char *iomode)
 {
 	FILE *handle;
 	int  namesize, i;
 	UBYTE *s, *to, *u1, *u2, *indir;	
-	if ( ( handle = fopen(*name,"r+b") ) != 0 ) {
+	
+	if ( ( handle = fopen(*name,iomode) ) != 0 ) {
 		*newname = (char *)strDup1((UBYTE *)(*name),"LocateBase");
 		return(handle);
 	}
@@ -244,7 +245,7 @@ FILE *LocateBase(char **name, char **newname)
 		s = (UBYTE *)(*name);
 		while ( *s ) *to++ = *s++;
 		*to = 0;
-		if ( ( handle = fopen(*newname,"r+b") ) != 0 ) {
+		if ( ( handle = fopen(*newname,iomode) ) != 0 ) {
 			return(handle);
 		}
 		M_free(*newname,"LocateBase, incdir/file");
@@ -267,7 +268,7 @@ FILE *LocateBase(char **name, char **newname)
 			s = (UBYTE *)(*name);
 			while ( *s ) *to++ = *s++;
 			*to = 0;
-			if ( ( handle = fopen(*newname,"r+b") ) != 0 ) {
+			if ( ( handle = fopen(*newname,iomode) ) != 0 ) {
 				return(handle);
 			}
 			M_free(*newname,"LocateBase Path/file");
@@ -279,8 +280,8 @@ FILE *LocateBase(char **name, char **newname)
 }
 
 /*
- 		#] LocateBase :
-  	#] Utilities :
+ 		#] LocateBase : 
+  	#] Utilities : 
   	#[ ReadIndex :
 */
 
@@ -294,13 +295,11 @@ int ReadIndex(DBASE *d)
 	Allocate the pieces one by one (makes it easier to give it back)
 */
 	if ( d->info.numberofindexblocks <= 0 ) return(0);
-#ifndef WORDSIZE32
 	if ( sizeof(INDEXBLOCK)*d->info.numberofindexblocks > MAXINDEXSIZE ) {
 		MesPrint("We need more than %ld bytes for the index.\n",MAXINDEXSIZE);
 		MesPrint("The file %s may not be a proper database\n",d->name);
 		return(-1);
 	}
-#endif
 	size = sizeof(INDEXBLOCK *)*d->info.numberofindexblocks;
 	if ( ( ib = (INDEXBLOCK **)Malloc1(size,"tb,index") ) == 0 ) return(-1);
 	for ( i = 0; i < d->info.numberofindexblocks; i++ ) {
@@ -393,7 +392,7 @@ thisiswrong:
 }
 
 /*
-  	#] ReadIndex :
+  	#] ReadIndex : 
   	#[ WriteIndexBlock :
 */
 
@@ -414,7 +413,7 @@ int WriteIndexBlock(DBASE *d,MLONG num)
 }
 
 /*
-  	#] WriteIndexBlock :
+  	#] WriteIndexBlock : 
   	#[ WriteNamesBlock :
 */
 
@@ -435,7 +434,7 @@ int WriteNamesBlock(DBASE *d,MLONG num)
 }
 
 /*
-  	#] WriteNamesBlock :
+  	#] WriteNamesBlock : 
   	#[ WriteIndex :
 
 	Problem here is to get the links right.
@@ -500,7 +499,7 @@ int WriteIndex(DBASE *d)
 }
 
 /*
-  	#] WriteIndex :
+  	#] WriteIndex : 
   	#[ WriteIniInfo :
 */
 
@@ -517,7 +516,7 @@ int WriteIniInfo(DBASE *d)
 }
 
 /*
-  	#] WriteIniInfo :
+  	#] WriteIniInfo : 
   	#[ ReadIniInfo :
 */
 
@@ -540,19 +539,28 @@ int ReadIniInfo(DBASE *d)
 }
 
 /*
-  	#] ReadIniInfo :
+  	#] ReadIniInfo : 
   	#[ GetDbase :
 */
 
-DBASE *GetDbase(char *filename)
+DBASE *GetDbase(char *filename, MLONG rwmode)
 {
 	FILE *f;
 	DBASE *d;
 	char *newname;
-	if ( ( f = LocateBase(&filename,&newname) ) == 0 ) {
-		
-		return(NewDbase(filename,0));
+	if ( rwmode == 0 ) {
+		if ( ( f = LocateBase(&filename,&newname,"rb") ) == 0 ) {
+	
+			MesPrint("&Trying to open non-existent TableBase in readonly mode: %s", filename);
+			Terminate(-1);
+		}
+	} else {
+		if ( ( f = LocateBase(&filename,&newname,"r+b") ) == 0 ) {
+	
+			return(NewDbase(filename,0));
+		}	
 	}
+
 /*	setbuf(f,0); */
 	d = (DBASE *)From0List(&(AC.TableBaseList));
 	d->mode = 0;
@@ -562,6 +570,7 @@ DBASE *GetDbase(char *filename)
 	d->iblocks = 0;
 	d->nblocks = 0;
 	d->tablenames = 0;
+	d->rwmode = rwmode;
 
 	d->info.entriesinindex = 0;
 	d->info.numberofindexblocks = 0;
@@ -576,13 +585,15 @@ DBASE *GetDbase(char *filename)
 	d->handle = f;
 	if ( ReadIniInfo(d) || ReadIndex(d) ) { M_free(d,"index-d"); fclose(f); return(0); }
 	if ( ComposeTableNames(d) < 0 ) { FreeTableBase(d); fclose(f); return(0); }
+	// free allocation from previous str_dup
+	M_free(d->name, "from str_dup");
 	d->name = str_dup(filename);
 	d->fullname = newname;
 	return(d);
 }
 
 /*
-  	#] GetDbase :
+  	#] GetDbase : 
   	#[ NewDbase :
 
 	Creates a new database with 'number' entries in the index.
@@ -617,6 +628,7 @@ DBASE *NewDbase(char *name,MLONG number)
 	d->tablenamessize = 0;
 	d->topnumber = 0;
 	d->tablenamefill = 0;
+	d->rwmode = 1;
 
 	d->mode = 0;
 	if ( ( d->nblocks = (NAMESBLOCK **)Malloc1(sizeof(NAMESBLOCK *)*numnameblocks,
@@ -664,6 +676,8 @@ getout:
 		if ( i > 0 ) d->iblocks[i]->previousblock = d->iblocks[i-1]->position;
 		else d->iblocks[i]->previousblock = -1;
 		d->iblocks[i]->position = ftell(f);
+		// Initialise, to keep valgrind happy
+		d->iblocks[i]->flags = -1;
 /*----------change 10-feb-2003 */
 /*
 			Zero things properly. We don't want garbage in the file.
@@ -718,7 +732,7 @@ getout:
 }
 
 /*
-  	#] NewDbase :
+  	#] NewDbase : 
   	#[ FreeTableBase :
 */
 
@@ -747,7 +761,7 @@ void FreeTableBase(DBASE *d)
 }
 
 /*
-  	#] FreeTableBase :
+  	#] FreeTableBase : 
   	#[ ComposeTableNames :
 
 		The nameblocks are supposed to be in memory.
@@ -799,7 +813,7 @@ gotall:;
 }
 
 /*
-  	#] ComposeTableNames :
+  	#] ComposeTableNames : 
   	#[ OpenDbase :
 */
 
@@ -808,7 +822,7 @@ DBASE *OpenDbase(char *filename)
 	FILE *f;
 	DBASE *d;
 	char *newname;
-	if ( ( f = LocateBase(&filename,&newname) ) == 0 ) {
+	if ( ( f = LocateBase(&filename,&newname,"r+b") ) == 0 ) {
 		MesPrint("Cannot open file %s\n",filename);
 		return(0);
 	}
@@ -828,7 +842,7 @@ DBASE *OpenDbase(char *filename)
 }
 
 /*
-  	#] OpenDbase :
+  	#] OpenDbase : 
   	#[ AddTableName :
 
 	Adds a name of a table. Writes the namelist to disk.
@@ -912,7 +926,7 @@ MLONG AddTableName(DBASE *d,char *name,TABLES T)
 }
 
 /*
-  	#] AddTableName :
+  	#] AddTableName : 
   	#[ GetTableName :
 
 	Gets a name of a table.
@@ -945,7 +959,7 @@ MLONG GetTableName(DBASE *d,char *name)
 }
 
 /*
-  	#] GetTableName :
+  	#] GetTableName : 
   	#[ PutTableNames :
 
 	Takes the names string in d->tablenames and puts it in the nblocks
@@ -973,6 +987,8 @@ int PutTableNames(DBASE *d)
 		for ( i = 0; i < d->info.numberofnamesblocks; i++ ) {
 			nnew[i] = d->nblocks[i];
 		}
+		free(d->nblocks);
+		d->nblocks = nnew;
 		for ( ; i < numblocks; i++ ) {
 			if ( ( d->nblocks[i] = (NAMESBLOCK *)Malloc1(sizeof(NAMESBLOCK),
 			"additional names blocks ") ) == 0 ) {
@@ -1004,11 +1020,11 @@ int PutTableNames(DBASE *d)
 			firstdif = i;
 			for ( ; m < d->tablenamefill; m++ ) {
 				*t++ = *s++; j++;
-			}
-			if ( j >= NAMETABLESIZE ) {
-				i++;
-				t = d->nblocks[i]->names;
-				j = 0;
+				if ( j >= NAMETABLESIZE ) {
+					i++;
+					t = d->nblocks[i]->names;
+					j = 0;
+				}
 			}
 			*t = 0;
 			break;
@@ -1042,7 +1058,7 @@ int PutTableNames(DBASE *d)
 }
 
 /*
-  	#] PutTableNames :
+  	#] PutTableNames : 
   	#[ AddToIndex :
 */
 
@@ -1129,7 +1145,7 @@ dowrite:
 }
 
 /*
-  	#] AddToIndex :
+  	#] AddToIndex : 
   	#[ AddObject :
 */
 
@@ -1143,7 +1159,7 @@ MLONG AddObject(DBASE *d,MLONG tablenumber,char *arguments,char *rhs)
 }
 
 /*
-  	#] AddObject :
+  	#] AddObject : 
   	#[ FindTableNumber :
 */
 
@@ -1171,7 +1187,7 @@ MLONG FindTableNumber(DBASE *d,char *name)
 }
 
 /*
-  	#] FindTableNumber :
+  	#] FindTableNumber : 
   	#[ WriteObject :
 */
 
@@ -1229,12 +1245,21 @@ int WriteObject(DBASE *d,MLONG tablenumber,char *arguments,char *rhs,MLONG numbe
 	else buffer = 0;
 	if ( buffer ) {
 		ssize = size;
+#ifdef WITHZSTD
+		// Force the use of zlib for compressed Tablebase entries, so that tablebases created
+		// with zstd-supported FORM builds can be used by zstd-unsupported FORM builds.
+		const int old_isUsingZSTDcompression = ZWRAP_isUsingZSTDcompression();
+		ZWRAP_useZSTDcompression(0);
+#endif
 		if ( ( error = compress((Bytef *)buffer,&newsize,(Bytef *)rhs,ssize) ) != Z_OK ) {
 			MesPrint("Error = %d\n",error);
 			MesPrint("Due to error no compress used for element %s in file %s\n",arguments,d->name);
 			M_free(buffer,"tb,WriteObject");
 			buffer = 0;
 		}
+#ifdef WITHZSTD
+		ZWRAP_useZSTDcompression(old_isUsingZSTDcompression);
+#endif
 	}
 	if ( buffer ) {
 		rhs = buffer;
@@ -1260,7 +1285,7 @@ int WriteObject(DBASE *d,MLONG tablenumber,char *arguments,char *rhs,MLONG numbe
 }
 
 /*
-  	#] WriteObject :
+  	#] WriteObject : 
   	#[ ReadObject :
 
 	Returns a pointer to the proper rhs
@@ -1294,8 +1319,10 @@ char *ReadObject(DBASE *d,MLONG tablenumber,char *arguments)
 	s = d->tablenames; i = 1;
 	while ( *s ) {
 		if ( i == tablenumber ) break;
-		while ( *s ) s++; s++;
-		while ( *s ) s++; s++;
+		while ( *s ) s++;
+		s++;
+		while ( *s ) s++;
+		s++;
 		i++;
 	}
 	MesPrint("%s(%s) not found in tablebase %s\n",s,arguments,d->name);
@@ -1339,7 +1366,7 @@ foundelement:;
 }
 
 /*
-  	#] ReadObject :
+  	#] ReadObject : 
   	#[ ReadijObject :
 
 	Returns a pointer to the proper rhs
@@ -1391,7 +1418,7 @@ char *ReadijObject(DBASE *d,MLONG i,MLONG j,char *arguments)
 }
 
 /*
-  	#] ReadijObject :
+  	#] ReadijObject : 
   	#[ ExistsObject :
 
 	Returns 1 if Object exists
@@ -1421,10 +1448,10 @@ int ExistsObject(DBASE *d,MLONG tablenumber,char *arguments)
 }
 
 /*
-  	#] ExistsObject :
+  	#] ExistsObject : 
   	#[ DeleteObject :
 
-	Returns 1 if Object has been deleteted.
+	Returns 1 if Object has been deleted.
 	We leave a hole. Actually the object is still there but has been
 	inactivated. It can be reactivated by calling this routine again.
 */
@@ -1457,6 +1484,5 @@ int DeleteObject(DBASE *d,MLONG tablenumber,char *arguments)
 }
 
 /*
-  	#] DeleteObject :
+  	#] DeleteObject : 
 */
-
