@@ -6,7 +6,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -173,6 +173,8 @@ int CoModuleOption(UBYTE *s)
 		if ( StrCmp((UBYTE *)(option->name),(UBYTE *)("polyfun")) == 0
 		 || StrCmp((UBYTE *)(option->name),(UBYTE *)("polyratfun")) == 0 ) {
 			polyflag = 1;
+			t++;
+			t = SkipAName(t);
 		}
 		else polyflag = 0;
 		if ( option->flags > 0 ) return(error);
@@ -252,7 +254,7 @@ int CoModOption(UBYTE *s)
  		#[ SetSpecialMode :
 */
 
-VOID SetSpecialMode(int moduletype, int specialtype)
+void SetSpecialMode(int moduletype, int specialtype)
 {
 	DUMMYUSE(moduletype); DUMMYUSE(specialtype);
 }
@@ -261,7 +263,7 @@ VOID SetSpecialMode(int moduletype, int specialtype)
  		#] SetSpecialMode : 
  		#[ MakeGlobal :
 
-VOID MakeGlobal()
+void MakeGlobal()
 {
 }
 
@@ -271,6 +273,21 @@ VOID MakeGlobal()
 
 int ExecModule(int moduletype)
 {
+/*
+	Here we check module options and other things that should
+	be done at the start of a module.
+*/
+#ifdef WITHFLOAT
+	if ( ( AC.tMaxWeight != 0 && AC.tMaxWeight != AC.MaxWeight )
+	  || ( AC.tDefaultPrecision != 0 && AC.tDefaultPrecision != AC.DefaultPrecision ) ) {
+		AC.DefaultPrecision = AC.tDefaultPrecision;
+		AC.tDefaultPrecision = 0;
+		AC.MaxWeight = AC.tMaxWeight;
+		AC.tMaxWeight = 0;
+		ClearMZVTables();
+		SetupMZVTables();
+	}
+#endif
 	return(DoExecute(moduletype,0));
 }
 
@@ -279,7 +296,7 @@ int ExecModule(int moduletype)
  		#[ ExecStore :
 */
 
-int ExecStore()
+int ExecStore(void)
 {
 	return(0);
 }
@@ -294,7 +311,7 @@ int ExecStore()
 			What to do with files we write to
 */
 
-VOID FullCleanUp()
+void FullCleanUp(void)
 {
 	int j;
 
@@ -354,6 +371,10 @@ VOID FullCleanUp()
 	AC.ThreadSortFileSynch = AM.gThreadSortFileSynch = AM.ggThreadSortFileSynch;
 	AC.ShortStatsMax = AM.gShortStatsMax = AM.ggShortStatsMax;
 	AC.SizeCommuteInSet = AM.gSizeCommuteInSet = 0;
+#ifdef WITHFLOAT
+	AC.MaxWeight = AM.gMaxWeight = AM.ggMaxWeight;
+	AC.DefaultPrecision = AM.gDefaultPrecision = AM.ggDefaultPrecision;
+#endif
 
 	NumExpressions = 0;
 	if ( DeleteStore(0) < 0 ) {
@@ -412,7 +433,7 @@ int DoPolyfun(UBYTE *s)
 		*t = c;
 		return(-1);
 	}
-	if ( functions[funnum].spec != 0 || functions[funnum].commute != 0 ) {
+	if ( functions[funnum].spec > 0 || functions[funnum].commute != 0 ) {
 		MesPrint("@The PolyFun must be a regular commuting function!");
 		*t = c;
 		return(-1);
@@ -470,7 +491,7 @@ Error1:;
 		*t = c;
 		return(-1);
 	}
-	if ( functions[funnum].spec != 0 || functions[funnum].commute != 0 ) {
+	if ( functions[funnum].spec > 0 || functions[funnum].commute != 0 ) {
 Error2:;
 		MesPrint("@The PolyRatFun must be a regular commuting function!");
 		*t = c;
@@ -486,7 +507,7 @@ Error2:;
 		t = EndOfToken(s);
 		c = *t; *t = 0;
 		if ( GetName(AC.varnames,s,&funnum,WITHAUTO) != CFUNCTION ) goto Error1;
-		if ( functions[funnum].spec != 0 || functions[funnum].commute != 0 ) goto Error2;
+		if ( functions[funnum].spec > 0 || functions[funnum].commute != 0 ) goto Error2;
 		AR.PolyFunInv = funnum+FUNCTION;
 		*t = c;
 	}
@@ -638,6 +659,7 @@ UBYTE * DoModDollar(UBYTE *s, int type)
 	UBYTE *name, c;
 	WORD number;
 	MODOPTDOLLAR *md;
+	int nummodopt;
 	while ( *s == '$' ) {
 /*
 		Read the name of the dollar
@@ -650,8 +672,22 @@ UBYTE * DoModDollar(UBYTE *s, int type)
 			c = *s; *s = 0;
 			number = GetDollar(name);
 			if ( number < 0 ) {
+				UBYTE *s1;
 				number = AddDollar(s,0,0,0);
-				Warning("&Undefined $-variable in module statement");
+				s1 = strDup1((UBYTE *)"Undefined $-variable in module option; option ignored: $","Undefined $-variable in ModuleOption");
+				s1 =  AddToString(s1,name,0);
+				Warning((char *)s1);
+				M_free(s1,"Undefined $-variable in ModuleOption");
+			}
+			for ( nummodopt = 0; nummodopt < NumModOptdollars; nummodopt++ ) {
+				if ( number == ModOptdollars[nummodopt].number && type != ModOptdollars[nummodopt].type ) {
+					UBYTE *s1;
+					s1 = strDup1((UBYTE *)"Conflicting module options for $-variable; later option ignored: $","Conflicting $-variable in ModuleOption");
+					s1 = AddToString(s1,name,0);
+					Warning((char *)s1);
+					M_free(s1,"Conflicting $-variable in ModuleOption");
+					break;
+				}
 			}
 			md = (MODOPTDOLLAR *)FromList(&AC.ModOptDolList);
 			md->number = number;
@@ -756,7 +792,7 @@ int DonotinParallel(UBYTE *s)
  		#[ DoExecStatement :
 */
 
-int DoExecStatement()
+int DoExecStatement(void)
 {
 #ifdef WITHSYSTEM
 	FLUSHCONSOLE;
@@ -773,7 +809,7 @@ int DoExecStatement()
  		#[ DoPipeStatement :
 */
 
-int DoPipeStatement()
+int DoPipeStatement(void)
 {
 #ifdef WITHPIPE
 	FLUSHCONSOLE;
