@@ -8,7 +8,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -220,11 +220,11 @@ int PF_LibTerminate(int error)
 /**
  * Probes the next incoming message.
  * If src == PF_ANY_SOURCE this operation is blocking,
- * otherwise nonbloking.
+ * otherwise nonblocking.
  *
  * @param[in,out]  src  the source process number. In output, the process number of actual found source.
  * @return              the tag value of the next incoming message if found,
- *                      0 if a nonbloking probe (input src != PF_ANY_SOURCE) did not
+ *                      0 if a nonblocking probe (input src != PF_ANY_SOURCE) did not
  *                      find any messages. A negative returned value indicates an error.
  */
 int PF_Probe(int *src)
@@ -1022,7 +1022,7 @@ typedef struct longMultiStruct {
 } PF_LONGMULTI;
 
 static UBYTE *PF_longPackBuf = NULL;
-static VOID *PF_longPackSmallBuf = NULL;
+static void *PF_longPackSmallBuf = NULL;
 static int PF_longPackPos = 0;
 static int PF_longPackTop = 0;
 static PF_LONGMULTI *PF_longMultiRoot = NULL;
@@ -1244,7 +1244,7 @@ static int PF_longPackInit(void)
 	if ( ret != MPI_SUCCESS ) return(ret);
 
 	PF_longPackSmallBuf =
-			(VOID*)Malloc1(sizeof(UBYTE)*PF_longPackTop,"PF_longPackSmallBuf");
+			(void*)Malloc1(sizeof(UBYTE)*PF_longPackTop,"PF_longPackSmallBuf");
 
 	PF_longPackTop = PF_packsize;
 	PF_longMultiRoot =
@@ -1815,7 +1815,7 @@ int PF_LongMultiBroadcast(void)
 */
 		if ( PF_longPackN > 1 ) {
 			if ( PF_longMultiPreparePrefix() ) return(-1);
-			ret = MPI_Bcast((VOID*)PF_longMultiTop->buffer,
+			ret = MPI_Bcast((void*)PF_longMultiTop->buffer,
 			                PF_packsize,MPI_PACKED,MASTER,PF_COMM);
 			if ( ret != MPI_SUCCESS ) return(ret);
 /*
@@ -1830,7 +1830,7 @@ int PF_LongMultiBroadcast(void)
 			Just broadcast all the chunks:
 */
 		for ( i = 0; i < PF_longPackN; i++ ) {
-			ret = MPI_Bcast((VOID*)PF_longMultiTop->buffer,
+			ret = MPI_Bcast((void*)PF_longMultiTop->buffer,
 			                PF_packsize,MPI_PACKED,MASTER,PF_COMM);
 			if ( ret != MPI_SUCCESS ) return(ret);
 			PF_longMultiTop = PF_longMultiTop->next;
@@ -1845,11 +1845,11 @@ int PF_LongMultiBroadcast(void)
 		Get the first chunk; it can be either the only data chunk, or
 		an auxiliary chunk, if the data do not fit the single chunk:
 */
-	ret = MPI_Bcast((VOID*)PF_longMultiRoot->buffer,
+	ret = MPI_Bcast((void*)PF_longMultiRoot->buffer,
 	                PF_packsize,MPI_PACKED,MASTER,PF_COMM);
 	if ( ret != MPI_SUCCESS ) return(ret);
 
-	ret = MPI_Unpack((VOID*)PF_longMultiRoot->buffer,
+	ret = MPI_Unpack((void*)PF_longMultiRoot->buffer,
 	                 PF_packsize,
 	                 &(PF_longMultiRoot->packpos),
 	                 &PF_longPackN,1,MPI_INT,PF_COMM);
@@ -1872,7 +1872,7 @@ int PF_LongMultiBroadcast(void)
 		nPacks and lastLen fields. Get chunks:
 */
 	for ( PF_longMultiTop = PF_longMultiRoot->next, i = 0; i < PF_longPackN; i++ ) {
-		ret = MPI_Bcast((VOID*)PF_longMultiTop->buffer,
+		ret = MPI_Bcast((void*)PF_longMultiTop->buffer,
 		                PF_packsize,MPI_PACKED,MASTER,PF_COMM);
 		if ( ret != MPI_SUCCESS ) return(ret);
 		if ( i == 0 ) {   /* The first chunk, it contains extra "1". */
@@ -1880,7 +1880,7 @@ int PF_LongMultiBroadcast(void)
 /*
 				Extract this 1 into tmp and forget about it.
 */
-			ret = MPI_Unpack((VOID*)PF_longMultiTop->buffer,
+			ret = MPI_Unpack((void*)PF_longMultiTop->buffer,
 			                 PF_packsize,
 			                 &(PF_longMultiTop->packpos),
 			                 &tmp,1,MPI_INT,PF_COMM);
