@@ -9,7 +9,7 @@
 
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -46,7 +46,7 @@
 /*
   	#] Includes : 
   	#[ Utilities :
-  		#[ MPIDBG_RANK :
+ 		#[ MPIDBG_RANK :
 */
 
 static inline int MPIDBG_Get_rank(void) {
@@ -55,16 +55,16 @@ static inline int MPIDBG_Get_rank(void) {
 #define MPIDBG_RANK MPIDBG_Get_rank()
 
 /*
-  		#] MPIDBG_RANK : 
-  		#[ MPIDBG_Out :
+ 		#] MPIDBG_RANK : 
+ 		#[ MPIDBG_Out :
 */
 
 static inline void MPIDBG_Out(const char *file, int line, const char *func, const char *fmt, ...) {
 	char buf[1024];  /* Enough. */
 	va_list ap;
 	va_start(ap, fmt);
-	sprintf(buf, "*** [%d] %10s %4d @ %-16s: ", MPIDBG_RANK, file, line, func);
-	vsprintf(buf + strlen(buf), fmt, ap);
+	snprintf(buf,1024, "*** [%d] %10s %4d @ %-16s: ", MPIDBG_RANK, file, line, func);
+	vsnprintf(buf + strlen(buf),1024-strlen(buf), fmt, ap);
 	va_end(ap);
 	/* Assume fprintf with a line will work well even in multi-processes. */
 	fprintf(stderr, "%s\n", buf);
@@ -72,8 +72,8 @@ static inline void MPIDBG_Out(const char *file, int line, const char *func, cons
 #define MPIDBG_Out(...) MPIDBG_Out(file, line, func, __VA_ARGS__)
 
 /*
-  		#] MPIDBG_Out : 
-  		#[ MPIDBG_sprint_requests :
+ 		#] MPIDBG_Out : 
+ 		#[ MPIDBG_sprint_requests :
 */
 
 static inline void MPIDBG_sprint_requests(char *buf, int count, const MPI_Request *requests)
@@ -99,8 +99,8 @@ static inline void MPIDBG_sprint_requests(char *buf, int count, const MPI_Reques
 }
 
 /*
-  		#] MPIDBG_sprint_requests : 
-  		#[ MPIDBG_sprint_statuses :
+ 		#] MPIDBG_sprint_requests : 
+ 		#[ MPIDBG_sprint_statuses :
 */
 
 static inline void MPIDBG_sprint_statuses(char *buf, int count, const MPI_Request *old_requests, const MPI_Request *new_requests, const MPI_Status *statuses)
@@ -127,7 +127,7 @@ static inline void MPIDBG_sprint_statuses(char *buf, int count, const MPI_Reques
 }
 
 /*
-  		#] MPIDBG_sprint_statuses : 
+ 		#] MPIDBG_sprint_statuses : 
   	#] Utilities : 
   	#[ MPI APIs :
 */
@@ -139,7 +139,7 @@ static inline void MPIDBG_sprint_statuses(char *buf, int count, const MPI_Reques
 #define MPIDBG_EXTARG const char *file, int line, const char *func
 
 /*
-  		#[ MPI_Send :
+ 		#[ MPI_Send :
 */
 
 static inline int MPIDBG_Send(void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPIDBG_EXTARG)
@@ -157,8 +157,8 @@ static inline int MPIDBG_Send(void *buf, int count, MPI_Datatype datatype, int d
 #define MPI_Send(...) MPIDBG_Send(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Send : 
-  		#[ MPI_Recv :
+ 		#] MPI_Send : 
+ 		#[ MPI_Recv :
 */
 
 static inline int MPIDBG_Recv(void* buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status, MPIDBG_EXTARG)
@@ -178,8 +178,8 @@ static inline int MPIDBG_Recv(void* buf, int count, MPI_Datatype datatype, int s
 #define MPI_Recv(...) MPIDBG_Recv(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Recv : 
-  		#[ MPI_Bsend :
+ 		#] MPI_Recv : 
+ 		#[ MPI_Bsend :
 */
 
 static inline int MPIDBG_Bsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPIDBG_EXTARG)
@@ -197,8 +197,8 @@ static inline int MPIDBG_Bsend(void* buf, int count, MPI_Datatype datatype, int
 #define MPI_Bsend(...) MPIDBG_Bsend(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Bsend : 
-  		#[ MPI_Ssend :
+ 		#] MPI_Bsend : 
+ 		#[ MPI_Ssend :
 */
 
 static inline int MPIDBG_Ssend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPIDBG_EXTARG)
@@ -216,8 +216,8 @@ static inline int MPIDBG_Ssend(void* buf, int count, MPI_Datatype datatype, int
 #define MPI_Ssend(...) MPIDBG_Ssend(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Ssend : 
-  		#[ MPI_Rsend :
+ 		#] MPI_Ssend : 
+ 		#[ MPI_Rsend :
 */
 
 static inline int MPIDBG_Rsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPIDBG_EXTARG)
@@ -235,8 +235,8 @@ static inline int MPIDBG_Rsend(void* buf, int count, MPI_Datatype datatype, int
 #define MPI_Rsend(...) MPIDBG_Rsend(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Rsend : 
-  		#[ MPI_Isend :
+ 		#] MPI_Rsend : 
+ 		#[ MPI_Isend :
 */
 
 static inline int MPIDBG_Isend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request, MPIDBG_EXTARG)
@@ -254,8 +254,8 @@ static inline int MPIDBG_Isend(void* buf, int count, MPI_Datatype datatype, int
 #define MPI_Isend(...) MPIDBG_Isend(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Isend : 
-  		#[ MPI_Ibsend :
+ 		#] MPI_Isend : 
+ 		#[ MPI_Ibsend :
 */
 
 static inline int MPIDBG_Ibsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request, MPIDBG_EXTARG)
@@ -273,8 +273,8 @@ static inline int MPIDBG_Ibsend(void* buf, int count, MPI_Datatype datatype, int
 #define MPI_Ibsend(...) MPIDBG_Ibsend(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Ibsend : 
-  		#[ MPI_Issend :
+ 		#] MPI_Ibsend : 
+ 		#[ MPI_Issend :
 */
 
 static inline int MPIDBG_Issend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request, MPIDBG_EXTARG)
@@ -292,8 +292,8 @@ static inline int MPIDBG_Issend(void* buf, int count, MPI_Datatype datatype, int
 #define MPI_Issend(...) MPIDBG_Issend(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Issend : 
-  		#[ MPI_Irsend :
+ 		#] MPI_Issend : 
+ 		#[ MPI_Irsend :
 */
 
 static inline int MPIDBG_Irsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request, MPIDBG_EXTARG)
@@ -311,8 +311,8 @@ static inline int MPIDBG_Irsend(void* buf, int count, MPI_Datatype datatype, int
 #define MPI_Irsend(...) MPIDBG_Irsend(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Irsend : 
-  		#[ MPI_Irecv :
+ 		#] MPI_Irsend : 
+ 		#[ MPI_Irecv :
 */
 
 static inline int MPIDBG_Irecv(void* buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request, MPIDBG_EXTARG)
@@ -330,8 +330,8 @@ static inline int MPIDBG_Irecv(void* buf, int count, MPI_Datatype datatype, int
 #define MPI_Irecv(...) MPIDBG_Irecv(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Irecv : 
-  		#[ MPI_Wait :
+ 		#] MPI_Irecv : 
+ 		#[ MPI_Wait :
 */
 
 static inline int MPIDBG_Wait(MPI_Request *request, MPI_Status *status, MPIDBG_EXTARG)
@@ -353,8 +353,8 @@ static inline int MPIDBG_Wait(MPI_Request *request, MPI_Status *status, MPIDBG_E
 #define MPI_Wait(...) MPIDBG_Wait(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Wait : 
-  		#[ MPI_Test :
+ 		#] MPI_Wait : 
+ 		#[ MPI_Test :
 */
 
 static inline int MPIDBG_Test(MPI_Request *request, int *flag, MPI_Status *status, MPIDBG_EXTARG)
@@ -381,8 +381,8 @@ static inline int MPIDBG_Test(MPI_Request *request, int *flag, MPI_Status *statu
 #define MPI_Test(...) MPIDBG_Test(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Test : 
-  		#[ MPI_Waitany :
+ 		#] MPI_Test : 
+ 		#[ MPI_Waitany :
 */
 
 static inline int MPIDBG_Waitany(int count, MPI_Request *array_of_requests, int *index, MPI_Status *status, MPIDBG_EXTARG)
@@ -407,8 +407,8 @@ static inline int MPIDBG_Waitany(int count, MPI_Request *array_of_requests, int
 #define MPI_Waitany(...) MPIDBG_Waitany(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Waitany : 
-  		#[ MPI_Testany :
+ 		#] MPI_Waitany : 
+ 		#[ MPI_Testany :
 */
 
 static inline int MPIDBG_Testany(int count, MPI_Request *array_of_requests, int *index, int *flag, MPI_Status *status, MPIDBG_EXTARG)
@@ -438,8 +438,8 @@ static inline int MPIDBG_Testany(int count, MPI_Request *array_of_requests, int
 #define MPI_Testany(...) MPIDBG_Testany(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Testany : 
-  		#[ MPI_Waitall :
+ 		#] MPI_Testany : 
+ 		#[ MPI_Waitall :
 */
 
 static inline int MPIDBG_Waitall(int count, MPI_Request *array_of_requests, MPI_Status *array_of_statuses, MPIDBG_EXTARG)
@@ -462,8 +462,8 @@ static inline int MPIDBG_Waitall(int count, MPI_Request *array_of_requests, MPI_
 #define MPI_Waitall(...) MPIDBG_Waitall(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Waitall : 
-  		#[ MPI_Testall :
+ 		#] MPI_Waitall : 
+ 		#[ MPI_Testall :
 */
 
 static inline int MPIDBG_Testall(int count, MPI_Request *array_of_requests, int *flag, MPI_Status *array_of_statuses, MPIDBG_EXTARG)
@@ -491,8 +491,8 @@ static inline int MPIDBG_Testall(int count, MPI_Request *array_of_requests, int
 #define MPI_Testall(...) MPIDBG_Testall(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Testall : 
-  		#[ MPI_Waitsome :
+ 		#] MPI_Testall : 
+ 		#[ MPI_Waitsome :
 */
 
 static inline int MPIDBG_Waitsome(int incount, MPI_Request *array_of_requests, int *outcount, int *array_of_indices, MPI_Status *array_of_statuses, MPIDBG_EXTARG)
@@ -515,8 +515,8 @@ static inline int MPIDBG_Waitsome(int incount, MPI_Request *array_of_requests, i
 #define MPI_Waitsome(...) MPIDBG_Waitsome(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Waitsome : 
-  		#[ MPI_Testsome :
+ 		#] MPI_Waitsome : 
+ 		#[ MPI_Testsome :
 */
 
 static inline int MPIDBG_Testsome(int incount, MPI_Request *array_of_requests, int *outcount, int *array_of_indices, MPI_Status *array_of_statuses, MPIDBG_EXTARG)
@@ -539,8 +539,8 @@ static inline int MPIDBG_Testsome(int incount, MPI_Request *array_of_requests, i
 #define MPI_Testsome(...) MPIDBG_Testsome(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Testsome : 
-  		#[ MPI_Iprobe :
+ 		#] MPI_Testsome : 
+ 		#[ MPI_Iprobe :
 */
 
 static inline int MPIDBG_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status, MPIDBG_EXTARG)
@@ -565,8 +565,8 @@ static inline int MPIDBG_Iprobe(int source, int tag, MPI_Comm comm, int *flag, M
 #define MPI_Iprobe(...) MPIDBG_Iprobe(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Iprobe : 
-  		#[ MPI_Probe :
+ 		#] MPI_Iprobe : 
+ 		#[ MPI_Probe :
 */
 
 static inline int MPIDBG_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status, MPIDBG_EXTARG)
@@ -586,8 +586,8 @@ static inline int MPIDBG_Probe(int source, int tag, MPI_Comm comm, MPI_Status *s
 #define MPI_Probe(...) MPIDBG_Probe(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Probe : 
-  		#[ MPI_Cancel :
+ 		#] MPI_Probe : 
+ 		#[ MPI_Cancel :
 */
 
 static inline int MPIDBG_Cancel(MPI_Request *request, MPIDBG_EXTARG)
@@ -605,8 +605,8 @@ static inline int MPIDBG_Cancel(MPI_Request *request, MPIDBG_EXTARG)
 #define MPI_Cancel(...) MPIDBG_Cancel(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Cancel : 
-  		#[ MPI_Test_cancelled :
+ 		#] MPI_Cancel : 
+ 		#[ MPI_Test_cancelled :
 */
 
 static inline int MPIDBG_Test_cancelled(MPI_Status *status, int *flag, MPIDBG_EXTARG)
@@ -629,8 +629,8 @@ static inline int MPIDBG_Test_cancelled(MPI_Status *status, int *flag, MPIDBG_EX
 #define MPI_Test_cancelled(...) MPIDBG_Test_cancelled(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Test_cancelled : 
-  		#[ MPI_Barrier :
+ 		#] MPI_Test_cancelled : 
+ 		#[ MPI_Barrier :
 */
 
 static inline int MPIDBG_Barrier(MPI_Comm comm, MPIDBG_EXTARG)
@@ -648,8 +648,8 @@ static inline int MPIDBG_Barrier(MPI_Comm comm, MPIDBG_EXTARG)
 #define MPI_Barrier(...) MPIDBG_Barrier(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Barrier : 
-  		#[ MPI_Bcast :
+ 		#] MPI_Barrier : 
+ 		#[ MPI_Bcast :
 */
 
 static inline int MPIDBG_Bcast(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPIDBG_EXTARG)
@@ -667,7 +667,7 @@ static inline int MPIDBG_Bcast(void* buffer, int count, MPI_Datatype datatype, i
 #define MPI_Bcast(...) MPIDBG_Bcast(__VA_ARGS__, __FILE__, __LINE__, __func__)
 
 /*
-  		#] MPI_Bcast : 
+ 		#] MPI_Bcast : 
   	#] MPI APIs : 
 */
 
