@@ -8,7 +8,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -249,11 +249,13 @@ int AddName(NAMETREE *nametree, UBYTE *name, WORD type, WORD number, int *nodenu
 	3: call GetAutoName and return its values.
 */
 
-int GetName(NAMETREE *nametree, UBYTE *name, WORD *number, int par)
+int GetName(NAMETREE *nametree, UBYTE *namein, WORD *number, int par)
 {
 	NAMENODE *n;
 	int node, newnode, i;
-	UBYTE *s, *t, *u;
+	UBYTE *s, *t, *u, *name;
+/*	name = ConstructName(namein,0); */
+	name = namein;
 	if ( nametree->namenode == 0 || nametree->namefill == 0 ) goto NotFound;
 	newnode = nametree->headnode;
 	do {
@@ -358,7 +360,7 @@ UBYTE *GetFunction(UBYTE *s,WORD *funnum)
 		t1 = SkipAName(s);
 		c = *t1; *t1 = 0;
 		if ( ( ( type = GetName(AC.varnames,s,&numfun,WITHAUTO) ) != CFUNCTION )
-			|| ( functions[numfun].spec != 0 ) ) {
+			|| ( functions[numfun].spec > 0 ) ) {
 			MesPrint("&%s should be a regular function",s);
 			*funnum = 0;
 			if ( type < 0 ) {
@@ -552,7 +554,7 @@ int GetVar(UBYTE *name, WORD *type, WORD *number, int wantedtype, int par)
   	#[ EntVar :
 */
 
-WORD EntVar(WORD type, UBYTE *name, WORD x, WORD y, WORD z, WORD d)
+int EntVar(WORD type, UBYTE *name, WORD x, WORD y, WORD z, WORD d)
 {
 	switch ( type ) {
 		case CSYMBOL:
@@ -597,7 +599,7 @@ int GetDollar(UBYTE *name)
   	#[ DumpTree :
 */
 
-VOID DumpTree(NAMETREE *nametree)
+void DumpTree(NAMETREE *nametree)
 {
 	if ( nametree->headnode >= 0
 		&& nametree->namebuffer && nametree->namenode ) {
@@ -610,7 +612,7 @@ VOID DumpTree(NAMETREE *nametree)
   	#[ DumpNode :
 */
 
-VOID DumpNode(NAMETREE *nametree, WORD node, WORD depth)
+void DumpNode(NAMETREE *nametree, WORD node, WORD depth)
 {
 	NAMENODE *n;
 	int i;
@@ -691,7 +693,7 @@ int CompactifyTree(NAMETREE *nametree,WORD par)
   	#[ CopyTree :
 */
 
-VOID CopyTree(NAMETREE *newtree, NAMETREE *oldtree, WORD node, WORD par)
+void CopyTree(NAMETREE *newtree, NAMETREE *oldtree, WORD node, WORD par)
 {
 	NAMENODE *n, *m;
 	UBYTE *s, *t;
@@ -779,7 +781,7 @@ VOID CopyTree(NAMETREE *newtree, NAMETREE *oldtree, WORD node, WORD par)
   	#[ LinkTree :
 */
 
-VOID LinkTree(NAMETREE *tree, WORD offset, WORD numnodes)
+void LinkTree(NAMETREE *tree, WORD offset, WORD numnodes)
 {
 /*
 	Makes the tree into a binary tree
@@ -810,7 +812,7 @@ VOID LinkTree(NAMETREE *tree, WORD offset, WORD numnodes)
   	#[ MakeNameTree :
 */
 
-NAMETREE *MakeNameTree()
+NAMETREE *MakeNameTree(void)
 {
 	NAMETREE *n;
 	n = (NAMETREE *)Malloc1(sizeof(NAMETREE),"new nametree");
@@ -829,7 +831,7 @@ NAMETREE *MakeNameTree()
   	#[ FreeNameTree :
 */
 
-VOID FreeNameTree(NAMETREE *n)
+void FreeNameTree(NAMETREE *n)
 {
 	if ( n ) {
 		if ( n->namebuffer ) M_free(n->namebuffer,"nametree->namebuffer");
@@ -844,7 +846,7 @@ VOID FreeNameTree(NAMETREE *n)
   	#[ WildcardNames :
 */
 
-void ClearWildcardNames()
+void ClearWildcardNames(void)
 {
 	AC.NumWildcardNames = 0;
 }
@@ -1524,7 +1526,7 @@ illegsym:		*s = cc;
 			else goto illegsym;
 			*s = cc;
 			if ( *s != ')' || ( s[1] && s[1] != ',' && s[1] != '<' ) ) {
-				Warning("&Excess information in symmetric properties currently ignored");
+				Warning("Excess information in symmetric properties currently ignored");
 				s = SkipField(s,1);
 			}
 			else s++;
@@ -1545,8 +1547,7 @@ retry:;
 				if ( ( StrICont(par,(UBYTE *)"arguments") == 0 )
 				|| ( StrICont(par,(UBYTE *)"args") == 0 ) ) {}
 				else {
-					Warning("&Illegal information in number of arguments properties currently ignored");
-					error = 1;
+					Warning("Illegal information in number of arguments properties currently ignored");
 				}
 				*s = cc;
 			}
@@ -1569,8 +1570,7 @@ retry:;
 				if ( ( StrICont(par,(UBYTE *)"arguments") == 0 )
 				|| ( StrICont(par,(UBYTE *)"args") == 0 ) ) {}
 				else {
-					Warning("&Illegal information in number of arguments properties currently ignored");
-					error = 1;
+					Warning("Illegal information in number of arguments properties currently ignored");
 				}
 				*s = cc;
 			}
@@ -1597,10 +1597,14 @@ retry:;
 					MesPrint("&Function %s changed to tensor",name);
 					error = 1;
 				}
-				else if ( istensor == 0 && fun->spec ) {
+				else if ( istensor == 0 && fun->spec > 0 ) {
 					MesPrint("&Tensor %s changed to function",name);
 					error = 1;
 				}
+				else if ( fun->spec == VERTEXFUNCTION ) {
+					MesPrint("&Function or Tensor %s already declared as a Particle",name);
+					error = 1;
+				}
 				fun->spec = istensor;
 				if ( fun->symmetric != symtype ) {
 					fun->symmetric = symtype;
@@ -1649,6 +1653,11 @@ int CoCTensor(UBYTE *s) { return(CoFunction(s,0,2)); }
 			element. The first numind elements for the indices and the
 			last one for the element in cbuf[T->bufnum].rhs
 
+			If the number of dimensions is *<number>, there is not a fixed
+			number of dimensions. Just a maximum. In that case the first
+			index should be the number of other dimensions. This first index
+			does not count in <number>.
+
 		Complication: to preserve speed we need a prototype and a pattern
 		for each thread when we use WITHPTHREADS. This is because we write
 		into those when looking for the pattern.
@@ -1794,16 +1803,34 @@ IllForm:	MesPrint("&Illegal name or option in table declaration");
 		Now the regular compilation
 */
 		inp = p++;
-		ParseNumber(x,p)
-		if ( FG.cTable[p[-1]] != 1 || ( *p != ',' && *p != ')' ) ) {
-			p = inp;
-			MesPrint("&First argument in a sparse table must be a number of dimensions");
-			error = 1;
-			x = 1;
+		if ( *p == '<' ) {
+			WORD inc = 1;
+			p++;
+			if ( *p == '=' ) { inc++; p++; }
+/*
+			We will use one extra number for telling how many there really are.
+*/
+			x = 0;
+			while ( *p <= '9' && *p >= '0' ) x = 10*x + (*p++-'0');
+			x = -x-inc;
+			if ( x == -1 ) {
+				MesPrint("&Maximum number of dimensions in *-table should be at least one.");
+				error = 1;
+				goto FinishUp2;
+			}
+		}
+		else {
+			ParseNumber(x,p)
+			if ( FG.cTable[p[-1]] != 1 || ( *p != ',' && *p != ')' ) ) {
+				p = inp;
+				MesPrint("&First argument in a sparse table must be a number of dimensions");
+				error = 1;
+				x = 1;
+			}
 		}
 		T->numind = x;
-		T->mm = (MINMAX *)Malloc1(x*sizeof(MINMAX),"table dimensions");
-		T->flags = (WORD *)Malloc1(x*sizeof(WORD),"table flags");
+		T->mm = (MINMAX *)Malloc1(ABS(x)*sizeof(MINMAX),"table dimensions");
+		T->flags = (WORD *)Malloc1(ABS(x)*sizeof(WORD),"table flags");
 		mm = T->mm;
 		inp = p;
 		if ( *inp != ')' ) inp++;
@@ -1851,7 +1878,7 @@ IllForm:	MesPrint("&Illegal name or option in table declaration");
 	Now we redo the 'function part' and send it to the compiler.
 	The prototype has to be picked up properly.
 */
-	AT.WorkPointer++; /* We needs one extra word later */
+	AT.WorkPointer++; /* We need one extra word later */
 	OldWork = AT.WorkPointer;
 	oldcbufnum = AC.cbufnum;
 	AC.cbufnum = T->bufnum;
@@ -1939,7 +1966,7 @@ IllForm:	MesPrint("&Illegal name or option in table declaration");
 	Now we have to allocate space for prototype+pattern
 	In the case of TFORM we need extra pointers, because each worker has its own
 */
-	j = *w + T->numind*2-3;
+	j = *w + ABS(T->numind)*2-3;
 #ifdef WITHPTHREADS
 	{ int n;
 	T->prototypeSize = ((i+j)*sizeof(WORD)+2*sizeof(WORD *)) * AM.totalnumberofthreads;
@@ -1952,10 +1979,10 @@ IllForm:	MesPrint("&Illegal name or option in table declaration");
 	}
 	T->pattern[0] = t;
 	j--; w++;
-	w[1] += T->numind*2;
+	w[1] += ABS(T->numind)*2;
 	for ( k = 0; k < FUNHEAD; k++ ) *t++ = *w++;
 	j -= FUNHEAD;
-	for ( k = 0; k < T->numind; k++ ) { *t++ = -SNUMBER; *t++ = 0; j -= 2; }
+	for ( k = 0; k < ABS(T->numind); k++ ) { *t++ = -SNUMBER; *t++ = 0; j -= 2; }
 	for ( k = 0; k < j; k++ ) *t++ = *w++;
 	if ( sparseflag ) T->pattern[0][1] = t - T->pattern[0];
 	k = t - T->pattern[0];
@@ -1971,10 +1998,10 @@ IllForm:	MesPrint("&Illegal name or option in table declaration");
 	for ( k = 0; k < i; k++ ) T->prototype[k] = OldWork[k];
 	t = T->pattern;
 	j--; w++;
-	w[1] += T->numind*2;
+	w[1] += ABS(T->numind)*2;
 	for ( k = 0; k < FUNHEAD; k++ ) *t++ = *w++;
 	j -= FUNHEAD;
-	for ( k = 0; k < T->numind; k++ ) { *t++ = -SNUMBER; *t++ = 0; j -= 2; }
+	for ( k = 0; k < ABS(T->numind); k++ ) { *t++ = -SNUMBER; *t++ = 0; j -= 2; }
 	for ( k = 0; k < j; k++ ) *t++ = *w++;
 	if ( sparseflag ) T->pattern[1] = t - T->pattern;
 #endif
@@ -2007,6 +2034,7 @@ IllForm:	MesPrint("&Illegal name or option in table declaration");
 FinishUp:;
 	AT.WorkPointer = OldWork - 1;
 	AC.cbufnum = oldcbufnum;
+FinishUp2:;
 	if ( T->sparse ) ClearTableTree(T);
 	if ( ( sparseflag & 2 ) != 0 ) {
 		if ( T->spare == 0 ) { SpareTable(T); }
@@ -2134,7 +2162,10 @@ int DoElements(UBYTE *s, SETS set, UBYTE *name)
 	while ( *s ) {
 		if ( *s == ',' ) { s++; continue; }
 		sgn = 0;
-		while ( *s == '-' || *s == '+' ) { sgn ^= 1; s++; }
+		while ( *s == '-' || *s == '+' ) {
+			if ( *s == '-' ) sgn ^= 1;
+			s++;
+		}
 		cname = s;
 		if ( FG.cTable[*s] == 0 || *s == '_' || *s == '[' ) {
 			if ( ( s = SkipAName(s) ) == 0 ) {
@@ -2326,10 +2357,41 @@ int DoElements(UBYTE *s, SETS set, UBYTE *name)
 int CoSet(UBYTE *s)
 {
 	int type, error = 0, ordered = 0;
-	UBYTE *name, c, *ss;
+	UBYTE *name = s, c, *ss;
 	SETS set;
 	WORD numberofset, dim = MAXPOSITIVE;
-	name = s;
+#ifdef WITHFLOAT
+/*----------------------------------------------------------------*/
+	{
+		WORD numeq = 0;
+		LONG x;
+		ss = s;
+		while ( *ss && *ss != ':' ) { if ( *ss == '=' ) numeq++; ss++; }
+		if ( *ss == 0 && numeq == 1 ) { /* We have the Set var = value; variety */
+			while ( FG.cTable[*s] == 0 ) s++;
+			ss = s; c = *s; *s = 0;
+			if ( c != '=' ) {
+Proper:
+				MesPrint("&Proper syntax for value-set is `Set name = value'");
+				return(1);
+			}
+			x = 0; s++;
+			while ( *s >= '0' && *s <= '9' ) x = 10*x + (*s++-'0');
+			if ( *s ) goto Proper;
+			if ( StrICmp(name,(UBYTE *)"maxweight") == 0 ) {
+				AC.tMaxWeight = x;   /* Temporary. Made permanent later */
+			}
+			else if ( StrICmp(name,(UBYTE *)"defaultprecision") == 0 ) {
+				AC.tDefaultPrecision = x;   /* Temporary. Made permanent later */
+			}
+			else {
+				MesPrint("&Illegal subkey in value set: %s",name);
+				return(1);
+			}
+		}		
+	}
+/*----------------------------------------------------------------*/
+#endif
 	if ( ( s = SkipAName(s) ) == 0 ) {
 IllForm:MesPrint("&Illegal name for set");
 		return(1);
@@ -2715,6 +2777,7 @@ int AddExpression(UBYTE *name, int x, int y)
 #ifdef PARALLELCODE
 	expr->partodo = 0;
 #endif
+	expr->uflags = 0;
 	return(numexpr);
 }
 
@@ -2947,7 +3010,7 @@ void ResetVariables(int par)
 				if ( T->sparse ) {
 					T->totind = T->mdefined;
 					for ( j = 0, tp = T->tablepointers; j < T->totind; j++ ) {
-						tp += T->numind;
+						tp += ABS(T->numind);
 #if TABLEEXTENSION == 2
 						tp[0] = tp[1];
 #else
@@ -2962,7 +3025,7 @@ void ResetVariables(int par)
 						TABLES TT = T->spare;
 						TT->totind = TT->mdefined;
 						for ( j = 0, tp = TT->tablepointers; j < TT->totind; j++ ) {
-							tp += TT->numind;
+							tp += ABS(TT->numind);
 #if TABLEEXTENSION == 2
 							tp[0] = tp[1];
 #else
@@ -3064,7 +3127,7 @@ void ResetVariables(int par)
   	#[ RemoveDollars :
 */
 
-void RemoveDollars()
+void RemoveDollars(void)
 {
 	DOLLARS d;
 	CBUF *C = cbuf + AM.dbufnum;
@@ -3124,7 +3187,7 @@ void Globalize(int par)
 			if ( T->sparse ) {
 				T->mdefined = T->totind;
 				for ( j = 0, tp = T->tablepointers; j < T->totind; j++ ) {
-					tp += T->numind;
+					tp += ABS(T->numind);
 #if TABLEEXTENSION == 2
 					tp[1] = tp[0];
 #else
@@ -3136,7 +3199,7 @@ void Globalize(int par)
 					TABLES TT = T->spare;
 					TT->mdefined = TT->totind;
 					for ( j = 0, tp = TT->tablepointers; j < TT->totind; j++ ) {
-						tp += TT->numind;
+						tp += ABS(TT->numind);
 #if TABLEEXTENSION == 2
 						tp[1] = tp[0];
 #else
