@@ -5,7 +5,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2013 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -55,7 +55,7 @@ int NormPolyTerm(PHEAD WORD *term)
 	WORD *tcoef, ncoef, *tstop, *tfill, *t, *tt;
 	int equal, i;
 	WORD *r1, *r2, *r3, *r4, *r5, *rfirst, rv;
-	WORD *lnum = AT.n_llnum+1, nnum;		/* Scratch, originally for factorials */
+	WORD *lnum, nnum;		/* Scratch, originally for factorials */
 /*
 	One: find the coefficient
 */
@@ -64,7 +64,7 @@ int NormPolyTerm(PHEAD WORD *term)
 	tstop = tcoef - ABS(tcoef[-1]);
 	tfill = t = term + 1;
 	rfirst = 0;
-	if ( t >= tstop ) return(*term);
+	if ( t >= tstop ) { return(*term); }
 	while ( t < tstop ) {
 		switch ( *t ) {
 			case SYMBOL:
@@ -146,6 +146,7 @@ int NormPolyTerm(PHEAD WORD *term)
 					if ( t[3] < 0 ) goto NormInf;
 					goto NormZero;
 				}
+				lnum = TermMalloc("lnum");
 				lnum[0] = t[2];
 				nnum = 1;
 				if ( t[3] && RaisPow(BHEAD (UWORD *)lnum,&nnum,(UWORD)(ABS(t[3]))) ) goto FromNorm;
@@ -160,6 +161,7 @@ int NormPolyTerm(PHEAD WORD *term)
 				}
 				ncoef = INCLENG(ncoef);
 				t += t[1];
+				TermFree(lnum,"lnum");
 				break;
 			case LNUMBER:
 				ncoef = REDLENG(ncoef);
@@ -768,9 +770,10 @@ int ConvertFromPoly(PHEAD WORD *term, WORD *outterm, WORD from, WORD to, WORD of
 		Notice that in TFORM we should be in sequential mode.
 */
 
-WORD FindSubterm(WORD *subterm)
+int FindSubterm(WORD *subterm)
 {
-	WORD old[5], *ss, *term, number;
+	WORD old[5], *ss, *term;
+	int number;
 	CBUF *C = cbuf + AM.sbufnum;
 	LONG oldCpointer;
 	term = subterm-1;
@@ -789,7 +792,7 @@ WORD FindSubterm(WORD *subterm)
 
 	oldCpointer = C->Pointer-C->Buffer; /* Offset of course !!!!!*/
 	AddRHS(AM.sbufnum,1);
-	AddNtoC(AM.sbufnum,*term,term);
+	AddNtoC(AM.sbufnum,*term,term,8);
 	AddToCB(C,0)
 /*
 		See whether we have this one already. If not, insert it in the tree.
@@ -837,9 +840,10 @@ WORD FindSubterm(WORD *subterm)
 		Notice that in TFORM we should be in sequential mode.
 */
 
-WORD FindLocalSubterm(PHEAD WORD *subterm, WORD startebuf)
+int FindLocalSubterm(PHEAD WORD *subterm, WORD startebuf)
 {
-	WORD old[5], *ss, *term, number, i, j, *t1, *t2;
+	WORD old[5], *ss, *term, i, j, *t1, *t2;
+	int number;
 	CBUF *C = cbuf + AT.ebufnum;
 	term = subterm-1;
 	ss = subterm+subterm[1];
@@ -872,7 +876,7 @@ WORD FindLocalSubterm(PHEAD WORD *subterm, WORD startebuf)
 	Now we have to add it to cbuf[AT.ebufnum]
 */
 	AddRHS(AT.ebufnum,1);
-	AddNtoC(AT.ebufnum,*term,term);
+	AddNtoC(AT.ebufnum,*term,term,9);
 	AddToCB(C,0)
 	number = C->numrhs-startebuf+numxsymbol;
 wearehappy:
@@ -989,7 +993,7 @@ void PrintSubtermList(int from,int to)
 }
 
 /*
- 		#] PrintSubtermList :
+ 		#] PrintSubtermList : 
  		#[ PrintExtraSymbol :
 
 		Prints the definition of extra symbol num as the contents
@@ -1030,10 +1034,14 @@ void PrintExtraSymbol(int num, WORD *terms,int par)
 				num = MAXVARIABLES-num;
 			}
 			else {
-				out = StrCopy(VARNAME(symbols,num),out);
+				out = StrCopy(FindSymbol(num),out);
+/*				out = StrCopy(VARNAME(symbols,num),out); */
 				break;
 			}
+			/* fall through */
 		case EXTRASYMBOL:
+			out = StrCopy(FindExtraSymbol(num),out);
+/*
 			out = StrCopy((UBYTE *)AC.extrasym,out);
 			if ( AC.extrasymbols == 0 ) {
 				out = NumCopy(num,out);
@@ -1042,6 +1050,7 @@ void PrintExtraSymbol(int num, WORD *terms,int par)
 			else if ( AC.extrasymbols == 1 ) {
 				out = AddArrayIndex(num,out);
 			}
+*/
 			break;
 		case EXPRESSIONNUMBER:
 			out = StrCopy(EXPRNAME(num),out);
@@ -1084,25 +1093,28 @@ void PrintExtraSymbol(int num, WORD *terms,int par)
 		Notice that in TFORM we should be in sequential mode.
 */
 
-WORD FindSubexpression(WORD *subexpr)
+int FindSubexpression(WORD *subexpr)
 {
-	WORD *term, number;
+	WORD *term;
+	int number;
 	CBUF *C = cbuf + AM.sbufnum;
-	LONG oldCpointer = C->Pointer-C->Buffer; /* Offset of course !!!!!*/
+	LONG oldCpointer;
+
+	term = subexpr;
+	while ( *term ) term += *term;
+	number = term - subexpr;
 /*
 		We may have to add the subexpression to the tree.
 		This requires a lock.
 */
 	LOCK(AM.sbuflock);
 
+	oldCpointer = C->Pointer-C->Buffer; /* Offset of course !!!!!*/
 	AddRHS(AM.sbufnum,1);
-	term = subexpr;
-	while ( *term ) term += *term;
-	number = term - subexpr;
 /*
 		Add the terms to the compiler buffer. Paste on a zero.
 */
-	AddNtoC(AM.sbufnum,number,subexpr);
+	AddNtoC(AM.sbufnum,number,subexpr,10);
 	AddToCB(C,0)
 /*
 		See whether we have this one already. If not, insert it in the tree.
@@ -1199,4 +1211,30 @@ nocase:;
 
 /*
  		#] ExtraSymFun : 
+ 		#[ PruneExtraSymbols :
+*/
+
+int PruneExtraSymbols(WORD downto)
+{
+	CBUF *C = cbuf + AM.sbufnum;
+	if ( downto < C->numrhs && downto >= 0 ) {  /* !!!!! */
+		ClearTree(AM.sbufnum);
+		C->numrhs = downto;
+		if ( downto == 0 ) {
+			C->Pointer = C->Buffer;
+		}
+		else {
+			WORD *w = C->rhs[downto], i;
+			while ( *w ) w += *w;
+			C->Pointer = w+1;
+			for ( i = 1; i <= downto; i++ ) {
+				InsTree(AM.sbufnum,i);
+			}
+		}
+	}
+	return(0);
+}
+
+/*
+ 		#] PruneExtraSymbols : 
 */
