@@ -8,7 +8,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2013 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -55,7 +55,7 @@
 
 */
 
-WORD EpfFind(PHEAD WORD *term, WORD *params)
+int EpfFind(PHEAD WORD *term, WORD *params)
 {
 	GETBIDENTITY
 	WORD *t, *m, *r, n1 = 0, n2, min = -1, count, fac;
@@ -136,7 +136,7 @@ AllLev:
 	mstop = c2;
 	fac = t[1] - FUNHEAD;
 	m = AT.TMout;
-	*m++ = 3 + (min<<1);		/* The full length */
+	*m++ = 3 + (min*2);		/* The full length */
 	*m++ = CONTRACT;
 	if ( number < 0 ) *m++ = 1;
 	else *m++ = 0;
@@ -183,9 +183,9 @@ AllLev:
 		MUNLOCK(ErrorMessageLock);
 		SETERROR(-1)
 	}
-	tstop += (ABS(ncoef))<<1;
+	tstop += (ABS(ncoef))*2;
 	if ( sgn < 0 ) ncoef = -ncoef;
-	ncoef <<= 1;
+	ncoef *= 2;
 	*tstop++ = (ncoef<0)?(ncoef-1):(ncoef+1);
 	*term = WORDDIF(tstop,term);
 	return(1);
@@ -209,7 +209,7 @@ AllLev:
 
 */
 
-WORD EpfCon(PHEAD WORD *term, WORD *params, WORD num, WORD level)
+int EpfCon(PHEAD WORD *term, WORD *params, WORD num, WORD level)
 {
 	GETBIDENTITY
 	WORD *kron, *perm, *termout, *tstop, size2;
@@ -284,7 +284,7 @@ WORD EpfGen(WORD number, WORD *inlist, WORD *kron, WORD *perm, WORD sgn)
 	WORD i, *in2, k, a;
 	if ( !sgn ) {
 		in2 = inlist + number;
-		number <<= 1;
+		number *= 2;
 		for ( i = 1; i < number; i += 2 ) {
 			*perm++ = i;
 			*perm++ = i;
@@ -294,7 +294,7 @@ WORD EpfGen(WORD number, WORD *inlist, WORD *kron, WORD *perm, WORD sgn)
 		if ( number <= 0 ) return(0);
 		else return(1);
 	}
-	number <<= 1;
+	number *= 2;
 	i = number - 1;
 	while ( ( i -= 2 ) >= 0 ) {
 		if ( ( k = perm[i] ) != i ) {
@@ -415,11 +415,12 @@ WORD Trick(WORD *in, TRACES *t)
 
 */
 
-WORD Trace4no(WORD number, WORD *kron, TRACES *t)
+int Trace4no(WORD number, WORD *kron, TRACES *t)
 {
 	WORD i;
 	WORD *p, *m;
-	WORD retval, *stop, oldsign;
+	WORD *stop, oldsign;
+	int retval;
 	if ( !t->sgn ) {		/* Startup */
 		if ( ( number < 0 ) || ( number & 1 ) ) return(0);
 		if ( number <= 2 ) {
@@ -433,7 +434,7 @@ WORD Trace4no(WORD number, WORD *kron, TRACES *t)
 		t->sgn = 1;
 		{
 			WORD nhalf = number >> 1;
-			WORD ndouble = number << 1;
+			WORD ndouble = number * 2;
 			p = t->eers;
 			t->eers = p;	p += nhalf;
 			t->mepf = p;	p += nhalf;
@@ -691,13 +692,14 @@ NextE4:			if ( t->lc4 < 0 ) goto NextE3;
 
 */
 
-WORD Trace4(PHEAD WORD *term, WORD *params, WORD num, WORD level)
+int Trace4(PHEAD WORD *term, WORD *params, WORD num, WORD level)
 {
 	GETBIDENTITY
 	TRACES *t;
 	WORD *p, *m, number, i;
 	WORD *OldW;
 	WORD j, minimum, minimum2, *min, *stopper;
+	int ret;
 	OldW = AT.WorkPointer;
 	if ( AN.numtracesctack >= AN.intracestack ) {
 		number = AN.intracestack + 2;
@@ -727,7 +729,7 @@ WORD Trace4(PHEAD WORD *term, WORD *params, WORD num, WORD level)
 	}
 	t->inlist = AT.WorkPointer;
 	t->accup = t->accu = t->inlist + number;
-	t->perm = t->accu + (number<<1);
+	t->perm = t->accu + (number*2);
 	t->eers = t->perm + number;
 	if ( ( AT.WorkPointer += 19 * number ) >= AT.WorkTop ) {
 		MLOCK(ErrorMessageLock);
@@ -815,10 +817,10 @@ WORD Trace4(PHEAD WORD *term, WORD *params, WORD num, WORD level)
 		Now the trace is in normal order
 */
 	}
-	number = Trace4Gen(BHEAD t,number);
+	ret = Trace4Gen(BHEAD t,number);
 	AT.WorkPointer = OldW;
 	AN.numtracesctack--;
-	return(number);
+	return(ret);
 }
 
 /*
@@ -853,7 +855,7 @@ WORD Trace4(PHEAD WORD *term, WORD *params, WORD num, WORD level)
 
 */
 
-WORD Trace4Gen(PHEAD TRACES *t, WORD number)
+int Trace4Gen(PHEAD TRACES *t, WORD number)
 {
 	GETBIDENTITY
 	WORD *termout, *stop;
@@ -1379,12 +1381,13 @@ WORD TraceNno(WORD number, WORD *kron, TRACES *t)
  		#[ TraceN :				WORD TraceN(term,params,num,level)
 */
 
-WORD TraceN(PHEAD WORD *term, WORD *params, WORD num, WORD level)
+int TraceN(PHEAD WORD *term, WORD *params, WORD num, WORD level)
 {
 	GETBIDENTITY
 	TRACES *t;
 	WORD *p, *m, number, i;
 	WORD *OldW;
+	int ret;
 	if ( params[3] != GAMMA1 ) {
 		MLOCK(ErrorMessageLock);
 		MesPrint("Gamma5 not allowed in n-trace");
@@ -1426,10 +1429,10 @@ WORD TraceN(PHEAD WORD *term, WORD *params, WORD num, WORD level)
 	t->termp = term;
 	t->factor = params[4];
 	t->allsign = params[5];
-	number = TraceNgen(BHEAD t,number);
+	ret = TraceNgen(BHEAD t,number);
 	AT.WorkPointer = OldW;
 	AN.numtracesctack--;
-	return(number);
+	return(ret);
 }
 
 /*
@@ -1438,12 +1441,12 @@ WORD TraceN(PHEAD WORD *term, WORD *params, WORD num, WORD level)
 
 		This routine is a simplified version of Trace4Gen. We know here
 		only three cases: Adjacent objects, same objects and all different.
-		The othere difference lies of course in the struct which is now
+		The other difference lies of course in the struct which is now
 		not of type TRACES, but of type TRACES.
 
 */
 
-WORD TraceNgen(PHEAD TRACES *t, WORD number)
+int TraceNgen(PHEAD TRACES *t, WORD number)
 {
 	GETBIDENTITY
 	WORD *termout, *stop;
@@ -1828,7 +1831,7 @@ TracCall:
 
 */
 
-WORD Traces(PHEAD WORD *term, WORD *params, WORD num, WORD level)
+int Traces(PHEAD WORD *term, WORD *params, WORD num, WORD level)
 {
 	GETBIDENTITY
 	switch ( AT.TMout[2] ) {	/* Subtype gives dimension */
@@ -1850,7 +1853,7 @@ WORD Traces(PHEAD WORD *term, WORD *params, WORD num, WORD level)
  		#[ TraceFind :			WORD TraceFind(term,params)
 */
 
-WORD TraceFind(PHEAD WORD *term, WORD *params)
+int TraceFind(PHEAD WORD *term, WORD *params)
 {
 	GETBIDENTITY
 	WORD *p, *m, *to;
@@ -1960,7 +1963,7 @@ F7:					if ( AT.TMout[3] == GAMMA7 ) (AT.TMout[4])++;
 		is applied (SR is the reversed string).
 */
 
-WORD Chisholm(PHEAD WORD *term, WORD level)
+int Chisholm(PHEAD WORD *term, WORD level)
 {
 	GETBIDENTITY
 	WORD *t, *r, *m, *s, *tt, *rr;
@@ -2175,7 +2178,7 @@ ChisCall:
  		#[ TenVecFind :			WORD TenVecFind(term,params)
 */
 
-WORD TenVecFind(PHEAD WORD *term, WORD *params)
+int TenVecFind(PHEAD WORD *term, WORD *params)
 {
 	GETBIDENTITY
 	WORD *t, *w, *m, *tstop;
@@ -2309,7 +2312,7 @@ match:
  		#[ TenVec :				WORD TenVec(term,params,num,level)
 */
 
-WORD TenVec(PHEAD WORD *term, WORD *params, WORD num, WORD level)
+int TenVec(PHEAD WORD *term, WORD *params, WORD num, WORD level)
 {
 	GETBIDENTITY
 	WORD *t, *m, *w, *termout, *tstop, *outlist, *ou, *ww, *mm;
@@ -2504,5 +2507,3 @@ fromTenVec:
  		#] TenVec : 
   	#] Operations : 
 */
-
-/* temporary commentary for forcing cvs merge */
