@@ -11,7 +11,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -53,7 +53,7 @@
 			to many.
 		2:	Each symbol can have only one (wildcard) power, so
 			x^2*x^n? is illegal.
-		3:	when a single vector is used it replaces all occurences
+		3:	when a single vector is used it replaces all occurrences
 			of the vector. Therefore q*q(mu) or q*q(mu) cannot occur.
 			Also q*q cannot be done.
 		4:	Loose vector elements are replaced with p(mu), dotproducts
@@ -94,11 +94,12 @@
 	  are errors that occur in TFORM only and that may indicate problems.
 */
 
-WORD TestMatch(PHEAD WORD *term, WORD *level)
+int TestMatch(PHEAD WORD *term, WORD *level)
 {
 	GETBIDENTITY
 	WORD *ll, *m, *w, *llf, *OldWork, *StartWork, *ww, *mm, *t, *OldTermBuffer = 0;
-	WORD power = 0, match = 0, i, msign = 0, ll2;
+	WORD power = 0, i, msign = 0, ll2;
+	int match = 0;
 	int numdollars = 0, protosize, oldallnumrhs;
 	CBUF *C = cbuf+AM.rbufnum, *CC;
 	AT.idallflag = 0;
@@ -634,10 +635,10 @@ nextlevel:;
 
 /*
  		#] TestMatch : 
- 		#[ Substitute :			VOID Substitute(term,pattern,power)
+ 		#[ Substitute :			void Substitute(term,pattern,power)
 */
 
-VOID Substitute(PHEAD WORD *term, WORD *pattern, WORD power)
+void Substitute(PHEAD WORD *term, WORD *pattern, WORD power)
 {
 	GETBIDENTITY
 	WORD *TemTerm;
@@ -1162,11 +1163,11 @@ SubCoef:
  		#] Substitute : 
  		#[ FindSpecial :		WORD FindSpecial(term)
 
-	Routine to detect symplifications regarding the special functions
+	Routine to detect simplifications regarding the special functions
 	exponent, denominator.
 
 
-WORD FindSpecial(WORD *term)
+void FindSpecial(WORD *term)
 {
 	WORD *t;
 	WORD *tstop;
@@ -1187,14 +1188,13 @@ WORD FindSpecial(WORD *term)
 		}
 		t += *t;
 	} while ( t < tstop ); }
-	return(0);
 }
 
  		#] FindSpecial : 
  		#[ FindAll :			WORD FindAll(term,pattern,level,par)
 */
 
-WORD FindAll(PHEAD WORD *term, WORD *pattern, WORD level, WORD *par)
+int FindAll(PHEAD WORD *term, WORD *pattern, WORD level, WORD *par)
 {
 	GETBIDENTITY
 	WORD *t, *m, *r, *mm, rnum;
@@ -1855,7 +1855,7 @@ int TestSelect(WORD *term, WORD *setp)
 				}
 				s++;
 			}
-			if ( functions[*term-FUNCTION].spec ) {
+			if ( functions[*term-FUNCTION].spec > 0 ) {
 				n = term[1] - FUNHEAD;
 				t = term + FUNHEAD;
 dotensor:
@@ -1965,7 +1965,7 @@ dotensor:
 
 /*
  		#] TestSelect : 
- 		#[ SubsInAll :			VOID SubsInAll()
+ 		#[ SubsInAll :			void SubsInAll()
 
 		This routine takes a match in id,all and stores it away in
 		the AT.allbufnum 'compiler' buffer, after taking out the pattern.
@@ -1979,7 +1979,7 @@ dotensor:
 		(to be done later).
 */
 
-VOID SubsInAll(PHEAD0)
+void SubsInAll(PHEAD0)
 {
 	GETBIDENTITY
 	WORD *TemTerm;
@@ -2055,7 +2055,7 @@ VOID SubsInAll(PHEAD0)
 					AT.aebufnum.
 					Problem: the SUBEXPRESSION assumes automatically
 					that the compiler buffer is AT.ebufnum. We have to
-					correct that in TranferBuffer.
+					correct that in TransferBuffer.
 */
 					p = pstart + SUBEXPSIZE;
 					while ( p < fill ) {
@@ -2140,7 +2140,7 @@ VOID SubsInAll(PHEAD0)
 		wildcard resolutions adapted by an offset.
 */
 
-VOID TransferBuffer(int from,int to,int spectator)
+void TransferBuffer(int from,int to,int spectator)
 {
 	CBUF *C  = cbuf + spectator;
 	CBUF *Cf = cbuf + from;
