@@ -5,7 +5,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -36,6 +36,7 @@
 #include "poly.h"
 #include "polygcd.h"
 
+#include <cassert>
 #include <cctype>
 #include <cmath>
 #include <algorithm>
@@ -110,6 +111,9 @@ poly::poly (PHEAD const UWORD *a, WORD na, WORD modp, WORD modn):
 poly::poly (const poly &a, WORD modp, WORD modn):
 	size_of_terms(AM.MaxTer/(LONG)sizeof(WORD))	
 {
+#ifdef POLY_MOVE_DEBUG
+	std::cout << "poly copy ctor" << std::endl;
+#endif
 	POLY_GETIDENTITY(a);
 	POLY_STOREIDENTITY;
 	
@@ -1290,7 +1294,7 @@ void poly::divmod_one_term (const poly &a, const poly &b, poly &q, poly &r, bool
 		WORD nq,nr;
 	 
 		if (div) {
-			// if variables are divisable, divide coefficient
+			// if variables are divisible, divide coefficient
 			if (q.modp==0) {				
 				DivLong((UWORD *)&a[ai+1+AN.poly_num_vars], a[ai+a[ai]-1],
 								(UWORD *)&b[2+AN.poly_num_vars], b[b[1]],
@@ -1759,7 +1763,7 @@ void poly::divmod_heap (const poly &a, const poly &b, poly &q, poly &r, bool onl
 			ri += t[3];
 		}
 		else {
-			// divisable, so divide coefficient as well
+			// divisible, so divide coefficient as well
 			WORD nq, nr;
 	
 			if (q.modp==0) {
@@ -1933,7 +1937,9 @@ void poly::mod (const poly &a, const poly &b, poly &c) {
 
 // copy operator
 poly & poly::operator= (const poly &a) {
-
+#ifdef POLY_MOVE_DEBUG
+	std::cout << "poly copy assign" << std::endl;
+#endif
 	if (&a != this) {
 		modp = a.modp;
 		modn = a.modn;
@@ -2386,7 +2392,8 @@ const poly poly::simple_poly (PHEAD int x, const poly &a, int b, int p, int n) {
 
 // gets all variables in the expressions and stores them in AN.poly_vars
 // (it is assumed that AN.poly_vars=NULL)
-void poly::get_variables (PHEAD vector<WORD *> es, bool with_arghead, bool sort_vars) {
+void poly::get_variables (PHEAD const vector<WORD *> &es, bool with_arghead, bool sort_vars) {
+	assert(AN.poly_vars == NULL);
 
 	AN.poly_num_vars = 0;
 
@@ -2396,7 +2403,7 @@ void poly::get_variables (PHEAD vector<WORD *> es, bool with_arghead, bool sort_
 	
 	// extract all variables
 	for (int ei=0; ei<(int)es.size(); ei++) {
-		WORD *e = es[ei];
+		const WORD *e = es[ei];
 
 		// fast notation
 		if (*e == -SNUMBER) {
@@ -2408,7 +2415,14 @@ void poly::get_variables (PHEAD vector<WORD *> es, bool with_arghead, bool sort_
 				degrees.push_back(1);
 			}
 		}
-		else {		
+		// JD: Here we need to check for non-symbol/number terms in fast notation.
+		else if (*e < 0) {
+			MLOCK(ErrorMessageLock);
+			MesPrint ((char*)"ERROR: polynomials and polyratfuns must contain symbols only");
+			MUNLOCK(ErrorMessageLock);
+			Terminate(1);
+		}
+		else {
 			for (int i=with_arghead ? ARGHEAD : 0; with_arghead ? i<e[0] : e[i]!=0; i+=e[i]) {
 				if (i+1<i+e[i]-ABS(e[i+e[i]-1]) && e[i+1]!=SYMBOL) {
 					MLOCK(ErrorMessageLock);
@@ -2417,7 +2431,7 @@ void poly::get_variables (PHEAD vector<WORD *> es, bool with_arghead, bool sort_
 					Terminate(1);
 				}
 				
-				for (int j=i+3; j<i+e[i]-ABS(e[i+e[i]-1]); j+=2) 
+				for (int j=i+3; j<i+e[i]-ABS(e[i+e[i]-1]); j+=2) {
 					if (!var_to_idx.count(e[j])) {
 						vars.push_back(e[j]);
 						var_to_idx[e[j]] = AN.poly_num_vars++;
@@ -2426,10 +2440,11 @@ void poly::get_variables (PHEAD vector<WORD *> es, bool with_arghead, bool sort_
 					else {
 						degrees[var_to_idx[e[j]]] = MaX(degrees[var_to_idx[e[j]]], e[j+1]);
 					}
+				}
 			}
 		}
 	}
-				 
+
 	// make sure an eventual FACTORSYMBOL appear as last
 	if (var_to_idx.count(FACTORSYMBOL)) {
 		int i = var_to_idx[FACTORSYMBOL];
@@ -2488,7 +2503,7 @@ const poly poly::argument_to_poly (PHEAD WORD *e, bool with_arghead, bool sort_u
 	
 	poly res(BHEAD 0);
 
-	 // fast notation
+	// fast notation
 	if (*e == -SNUMBER) {
 		if (denpoly!=NULL) *denpoly = poly(BHEAD 1);
 		
@@ -2529,7 +2544,7 @@ const poly poly::argument_to_poly (PHEAD WORD *e, bool with_arghead, bool sort_u
 	UWORD *dum = NumberMalloc("poly::argument_to_poly");
 	den[0]=1;
 	
- 	for (int i=with_arghead ? ARGHEAD : 0; with_arghead ? i<e[0] : e[i]!=0; i+=e[i]) {
+	for (int i=with_arghead ? ARGHEAD : 0; with_arghead ? i<e[0] : e[i]!=0; i+=e[i]) {
 		int ncoe = ABS(e[i+e[i]-1]/2);
 		UWORD *coe = (UWORD *)&e[i+e[i]-ncoe-1];
 		while (ncoe>0 && coe[ncoe-1]==0) ncoe--;
@@ -2560,12 +2575,24 @@ const poly poly::argument_to_poly (PHEAD WORD *e, bool with_arghead, bool sort_u
 	}
 
 	res[0] = ri;
-	
-	// normalize, since the Form order is probably not the polynomial order
-	// for multiple variables
 
-	if (sort_univar || AN.poly_num_vars>1)
+	// JD: For univariate cases, check whether the ordering is correct or not.
+	// There are various ways to arrive here, but with an incorrect ordering, such
+	// as interactions with collect, moving a polyratfun out of another function
+	// argument, etc.
+	// If the ordering is not correct, make sure we normalize. In multivariate cases,
+	// always normalize as before.
+	if (sort_univar == false && AN.poly_num_vars == 1) {
+		if (res.number_of_terms() >= 2) {
+			if(res[2] < res.last_monomial()[2]) {
+				sort_univar = true;
+			}
+		}
+	}
+
+	if (sort_univar || AN.poly_num_vars>1) {
 		res.normalize();
+	}
 
 	NumberFree(den,"poly::argument_to_poly");
 	NumberFree(pro,"poly::argument_to_poly");
