@@ -5,7 +5,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -705,10 +705,8 @@ const poly polygcd::gcd_modular_sparse_interpolation (const poly &origa, const p
 	poly lred(sparse_interpolation_reduce_poly(lcgcd,x));
 
 	// set all coefficients to 1
-	int N=0;
 	for (int i=1; i<sred[0]; i+=sred[i]) {
 		sred[i+sred[i]-2] = sred[i+sred[i]-1] = 1;
-		N++;
 	}
 	
 	// generate random numbers and check there this set doesn't result
@@ -1390,7 +1388,7 @@ const map<vector<int>,int> polygcd::bracket_count(const poly &a, const vector<in
 
 struct BracketInfo {
 	std::vector<int> pattern;
-	int num_terms, dummy;
+	int num_terms, dummy = 0;
 	const poly* p;
 
 	BracketInfo(const std::vector<int>& pattern, int num_terms, const poly* p) : pattern(pattern), num_terms(num_terms), p(p) {}
