@@ -8,7 +8,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -127,7 +127,7 @@ WORD ReNumber(PHEAD WORD *term)
 
 */
 
-VOID FunLevel(PHEAD WORD *term)
+void FunLevel(PHEAD WORD *term)
 {
 	GETBIDENTITY
 	WORD *t, *tstop, *r, *fun;
@@ -222,7 +222,7 @@ VOID FunLevel(PHEAD WORD *term)
 					else {
 						if ( *t == -INDEX ) {
 							t++;
-							if ( *t >= AN.IndDum ) {
+							if ( *t > AN.IndDum ) {
 								if ( AN.NumFound >= AN.MaxRenumScratch ) AdjustRenumScratch(BHEAD0);
 								AN.NumFound++;
 								*AN.DumFound++ = *t;
@@ -361,7 +361,7 @@ int FullRenumber(PHEAD WORD *term, WORD par)
 				AT.WorkPointer = w;
 				if ( Normalize(BHEAD termtry) == 0 ) {
 					if ( *termtry == 0 ) goto Return0;
-					if ( ( ii = CompareTerms(termtry,best,0) ) > 0 ) {
+					if ( ( ii = CompareTerms(BHEAD termtry,best,0) ) > 0 ) {
 						t = termtry; w = best;
 						for ( ii = 0; ii < *termtry; ii++ ) *w++ = *t++;
 						i = 0; break; /* restart from beginning */
@@ -399,7 +399,7 @@ int FullRenumber(PHEAD WORD *term, WORD par)
 				AT.WorkPointer = w;
 				if ( Normalize(BHEAD termtry) == 0 ) {
 					if ( *termtry == 0 ) goto Return0;
-					if ( ( ii = CompareTerms(termtry,best,0) ) > 0 ) {
+					if ( ( ii = CompareTerms(BHEAD termtry,best,0) ) > 0 ) {
 						t = termtry; w = best;
 						for ( i = 0; i < *termtry; i++ ) *w++ = *t++;
 					}
@@ -433,7 +433,7 @@ Return0:
 		always been missing. Routine made 29-jan-2007.
 */
 
-VOID MoveDummies(PHEAD WORD *term, WORD shift)
+void MoveDummies(PHEAD WORD *term, WORD shift)
 {
 	GETBIDENTITY
 	WORD maxval = AN.IndDum;
@@ -1041,7 +1041,7 @@ outofrange:
  		#[ MultDo :
 */
 
-WORD MultDo(PHEAD WORD *term, WORD *pattern)
+int MultDo(PHEAD WORD *term, WORD *pattern)
 {
 	GETBIDENTITY
 	WORD *t, *r, i;
@@ -1069,7 +1069,7 @@ WORD MultDo(PHEAD WORD *term, WORD *pattern)
  		#[ TryDo :
 */
 
-WORD TryDo(PHEAD WORD *term, WORD *pattern, WORD level)
+int TryDo(PHEAD WORD *term, WORD *pattern, WORD level)
 {
 	GETBIDENTITY
 	WORD *t, *r, *m, i, j;
@@ -1089,7 +1089,7 @@ WORD TryDo(PHEAD WORD *term, WORD *pattern, WORD level)
 		if ( *r == 0 ) return(0);
 		ReNumber(BHEAD r); Normalize(BHEAD r);
 		if ( *r == 0 ) return(0);
-		if ( ( i = CompareTerms(term,r,0) ) < 0 ) {
+		if ( ( i = CompareTerms(BHEAD term,r,0) ) < 0 ) {
 			*AN.RepPoint = 1;
 			AR.expchanged = 1;
 			return(Generator(BHEAD r,level));
@@ -1116,7 +1116,7 @@ WORD TryDo(PHEAD WORD *term, WORD *pattern, WORD level)
 		because there is no knowing how long the field will be.
 */
 
-WORD DoDistrib(PHEAD WORD *term, WORD level)
+int DoDistrib(PHEAD WORD *term, WORD level)
 {
 	GETBIDENTITY
 	WORD *t, *m, *r = 0, *stop, *tstop, *termout, *endhead, *starttail, *parms;
@@ -1376,7 +1376,7 @@ redok:		while ( arg[j] == 1 && j >= 0 ) { j--; k++; }
 		Returns 1 if the arguments in the field are identical.
 */
 
-WORD EqualArg(WORD *parms, WORD num1, WORD num2)
+int EqualArg(WORD *parms, WORD num1, WORD num2)
 {
 	WORD *t1, *t2;
 	WORD i;
@@ -1402,7 +1402,7 @@ WORD EqualArg(WORD *parms, WORD num1, WORD num2)
  		#[ DoDelta3 :
 */
 
-WORD DoDelta3(PHEAD WORD *term, WORD level)
+int DoDelta3(PHEAD WORD *term, WORD level)
 {
 	GETBIDENTITY
 	WORD *t, *m, *m1, *m2, *stopper, *tstop, *termout, *dels, *taken;
@@ -1604,7 +1604,7 @@ nextk:;
 					arguments there will be no action.
 */
 
-WORD TestPartitions(WORD *tfun, PARTI *parti)
+int TestPartitions(WORD *tfun, PARTI *parti)
 {
 	WORD *tnext = tfun + tfun[1];
 	WORD *t, *tt;
@@ -1719,7 +1719,7 @@ No:
 	if we keep needing it.
 */
 
-WORD DoPartitions(PHEAD WORD *term, WORD level)
+int DoPartitions(PHEAD WORD *term, WORD level)
 {
 	WORD x, i, j, im, *fun, ndiff, siz, tensorflag = 0;
 	PARTI part = AT.partitions;
@@ -1733,7 +1733,7 @@ WORD DoPartitions(PHEAD WORD *term, WORD level)
 	Start with bubble sorting the list of arguments. And the list of partitions.
 */
 	fun = term + part.where;
-	if ( functions[*fun-FUNCTION].spec ) tensorflag = 1;
+	if ( functions[*fun-FUNCTION].spec > 0 ) tensorflag = 1;
 	for ( i = 1; i < part.numargs; i++ ) {
 		for ( j = i-1; j >= 0; j-- ) {
 			if ( CompArg(fun+part.args[j+1],fun+part.args[j]) >= 0 ) break;
@@ -1875,7 +1875,7 @@ WORD DoPartitions(PHEAD WORD *term, WORD level)
 			t2 = twhere+twhere[1];
 			to = termout = AT.WorkPointer;
 			if ( termout + *term + part.numpart*FUNHEAD + AM.MaxTal >= AT.WorkTop ) {
-				return(MesWork());
+				MesWork();
 			}
 			for ( i = 0; i < ncoeffnum; i++ ) coeff[i] = coeffnum[i];
 			ncoeff = ncoeffnum;
@@ -2004,7 +2004,7 @@ Done:
 	all permutations of the args. This should always fit!
 */
 
-WORD DoPermutations(PHEAD WORD *term, WORD level)
+int DoPermutations(PHEAD WORD *term, WORD level)
 {
 	PERMP perm;
 	WORD *oldworkpointer = AT.WorkPointer, *termout = AT.WorkPointer;
@@ -2092,7 +2092,7 @@ WORD DoPermutations(PHEAD WORD *term, WORD level)
 	groups (0,1) also cause double terms.
 */
 
-WORD DoShuffle(WORD *term, WORD level, WORD fun, WORD option)
+int DoShuffle(WORD *term, WORD level, WORD fun, WORD option)
 {
 	GETIDENTITY
 	SHvariables SHback, *SH = &(AN.SHvar);
@@ -2484,7 +2484,7 @@ Finicall:
 	need to know n1, n2, minval.
 */
 
-WORD DoStuffle(WORD *term, WORD level, WORD fun, WORD option)
+int DoStuffle(WORD *term, WORD level, WORD fun, WORD option)
 {
 	GETIDENTITY
 	SHvariables SHback, *SH = &(AN.SHvar);
