@@ -5,7 +5,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -35,24 +35,6 @@
 
 #include "form3.h"
 
-#ifdef ANSI
-#include <stdarg.h>
-#else
-#ifdef mBSD
-#include <varargs.h>
-#else
-#ifdef VMS
-#include <varargs.h>
-#else
-typedef UBYTE *va_list;
-#define va_dcl int va_alist;
-#define va_start(list) list = (UBYTE *) &va_alist
-#define va_end(list)
-#define va_arg(list,mode) (((mode *)(list += sizeof(mode)))[-1])
-#endif
-#endif
-#endif
-
 static int startinline = 0;
 static char fcontchar = '&';
 static int noextralinefeed = 0;
@@ -72,14 +54,14 @@ UBYTE *StrCopy(UBYTE *from, UBYTE *to)
 
 /*
  		#] StrCopy : 
- 		#[ AddToLine :			VOID AddToLine(s)
+ 		#[ AddToLine :			void AddToLine(s)
 
 	Puts the characters of s in the outputline. If the line becomes
 	filled it is written.
 
 */
 
-VOID AddToLine(UBYTE *s)
+void AddToLine(UBYTE *s)
 {
 	UBYTE *Out;
 	LONG num;
@@ -176,10 +158,10 @@ VOID AddToLine(UBYTE *s)
 
 /*
  		#] AddToLine : 
- 		#[ FiniLine :			VOID FiniLine()
+ 		#[ FiniLine :			void FiniLine()
 */
 
-VOID FiniLine()
+void FiniLine(void)
 {
 	UBYTE *Out;
 	WORD i;
@@ -258,13 +240,13 @@ VOID FiniLine()
 
 /*
  		#] FiniLine : 
- 		#[ IniLine :			VOID IniLine(extrablank)
+ 		#[ IniLine :			void IniLine(extrablank)
 
 	Initializes the output line for the type of output
 
 */
 
-VOID IniLine(WORD extrablank)
+void IniLine(WORD extrablank)
 {
 	UBYTE *Out;
 	Out = AO.OutputLine;
@@ -290,7 +272,7 @@ VOID IniLine(WORD extrablank)
 
 /*
  		#] IniLine : 
- 		#[ LongToLine :			VOID LongToLine(a,na)
+ 		#[ LongToLine :			void LongToLine(a,na)
 
 	Puts a Long integer in the output line. If it is only a single
 	word long it is put in the line as a single token.
@@ -300,7 +282,7 @@ VOID IniLine(WORD extrablank)
 
 static UBYTE *LLscratch = 0;
 
-VOID LongToLine(UWORD *a, WORD na)
+void LongToLine(UWORD *a, WORD na)
 {
 	UBYTE *OutScratch;
 	if ( LLscratch == 0 ) {
@@ -325,7 +307,7 @@ VOID LongToLine(UWORD *a, WORD na)
 
 /*
  		#] LongToLine : 
- 		#[ RatToLine :			VOID RatToLine(a,na)
+ 		#[ RatToLine :			void RatToLine(a,na)
 
 	Puts a rational number in the output line. The sign is ignored.
 
@@ -334,134 +316,122 @@ VOID LongToLine(UWORD *a, WORD na)
 static UBYTE *RLscratch = 0;
 static UWORD *RLscratE = 0;
 
-VOID RatToLine(UWORD *a, WORD na)
+void RatToLine(UWORD *a, WORD na)
 {
 	GETIDENTITY
 	WORD adenom, anumer;
+	ULONG maxInt;
+
+	if ( AC.OutputMode == CMODE ) {
+		// In C, integer literals over 2^32-1 are automatically promoted to longer types up to
+		// unsigned long long, however FORM has always given integers over 2^32-1 a floating suffix
+		// in C mode. Retain that behaviour here for now. In the future, maxInt could be a user-
+		// configurable parameter.
+		maxInt = 4294967295; 
+	}
+	else {
+		// In Fortran modes, literals over 2^31-1 should be printed with a float suffix
+		maxInt = 2147483647;
+	}
+
 	if ( na < 0 ) na = -na;
+
 	if ( AC.OutNumberType == RATIONALMODE ) {
-/*
-		We need some special provisions for the various Fortran modes.
-		In PFORTRAN we use
-				one     if denom = numerator = 1
-				integer     if denom = 1
-				(one/integer) if numerator = 1
-				((one*integer)/integer) in the general case
-*/
+
+		// Work out what is to be printed:
+		WORD isOne = 0, isOneOver = 0, isIntegral = 0;
+		WORD isLongNum = 0, isLongDen = 0;
+		UnPack(a,na,&adenom,&anumer);
+		if ( na == 1 && a[0] == 1 && a[1] == 1 ) { isOne = 1; isIntegral = 1; }
+		else if ( adenom == 1 && a[na] == 1 ) { isIntegral = 1; }
+		else if ( anumer == 1 && a[0] == 1 ) { isOneOver = 1; }
+		if ( anumer > 1 || ( anumer == 1 && a[0] > maxInt ) ) { isLongNum = 1; };
+		if ( adenom > 1 || ( adenom == 1 && a[na] > maxInt ) ) { isLongDen = 1; };
+
+		// Now sort out the float suffix for the numerator:
+		UBYTE* suffNum = (UBYTE*)"";
+		UBYTE* suffDen = (UBYTE*)"";
+		if ( isLongNum || !isIntegral || AC.Fortran90Kind || ( AO.DoubleFlag & 4 ) == 4 ) {
+			if ( AC.OutputMode == FORTRANMODE && AC.IsFortran90 == ISFORTRAN90 ) {
+				if ( AC.Fortran90Kind ) { suffNum = AC.Fortran90Kind; }
+				else { suffNum = (UBYTE*)"."; }
+			}
+			else if ( AC.OutputMode == FORTRANMODE || AC.OutputMode == CMODE ) {
+				if ( ( AO.DoubleFlag & 2 ) == 2 ) { suffNum = (UBYTE*)".Q0"; }
+				else if ( ( AO.DoubleFlag & 1 ) == 1 ) { suffNum = (UBYTE*)".D0"; }
+				else { suffNum = (UBYTE*)"."; }
+			}
+		}
+		// The same again, for the denominator:
+		if ( isLongDen || !isIntegral || AC.Fortran90Kind || ( AO.DoubleFlag & 4 ) == 4 ) {
+			if ( AC.OutputMode == FORTRANMODE && AC.IsFortran90 == ISFORTRAN90 ) {
+				if ( AC.Fortran90Kind ) { suffDen = AC.Fortran90Kind; }
+				else { suffDen = (UBYTE*)"."; }
+			}
+			else if ( AC.OutputMode == FORTRANMODE || AC.OutputMode == CMODE ) {
+				if ( ( AO.DoubleFlag & 2 ) == 2 ) { suffDen = (UBYTE*)".Q0"; }
+				else if ( ( AO.DoubleFlag & 1 ) == 1 ) { suffDen = (UBYTE*)".D0"; }
+				else { suffDen = (UBYTE*)"."; }
+			}
+		}
+		// In PFORTRAN mode, rationals don't get a suffix if the integers are not long
+		// Also the suffix is at least ".D0" (and never ".").
 		if ( AC.OutputMode == PFORTRANMODE ) {
-		  UnPack(a,na,&adenom,&anumer);
-		  if ( na == 1 && a[0] == 1 && a[1] == 1 ) {
-			AddToLine((UBYTE *)"one");
-			return;
-		  }
-		  if ( adenom == 1 && a[na] == 1 ) {
-			LongToLine(a,anumer);
-			if ( anumer > 1 ) {
-				if ( ( AO.DoubleFlag & 2 ) == 2 ) { AddToLine((UBYTE *)".Q0"); }
-				else { AddToLine((UBYTE *)".D0"); }
+			if ( isLongNum ) {
+				if ( ( AO.DoubleFlag & 2 ) == 2 ) { suffNum = (UBYTE*)".Q0"; }
+				else { suffNum = (UBYTE*)".D0"; }
 			}
-		  }
-		  else if ( anumer == 1 && a[0] == 1 ) {
-			a += na;
-			AddToLine((UBYTE *)"(one/");
-			LongToLine(a,adenom);
-			if ( adenom > 1 ) {
-				if ( ( AO.DoubleFlag & 2 ) == 2 ) { AddToLine((UBYTE *)".Q0"); }
-				else { AddToLine((UBYTE *)".D0"); }
+			if ( isLongDen ) {
+				if ( ( AO.DoubleFlag & 2 ) == 2 ) { suffDen = (UBYTE*)".Q0"; }
+				else { suffDen = (UBYTE*)".D0"; }
 			}
-			AddToLine((UBYTE *)")");
-		  }
-		  else {
-			if ( anumer > 1 || adenom > 1 ) {
+		}
+
+		// Finally, print the number:
+		// In PFORTRAN we use
+		//    one     if denom = numerator = 1
+		//    integer     if denom = 1
+		//    (one/integer) if numerator = 1
+		//    ((one*integer)/integer) in the general case
+		if ( AC.OutputMode == PFORTRANMODE ) {
+			if ( isOne ) {
+				AddToLine((UBYTE *)"one");
+			}
+			else if ( isOneOver ) {
+				AddToLine((UBYTE *)"(one/");
+				LongToLine(a+na,adenom);
+				AddToLine(suffDen);
+				AddToLine((UBYTE*)")");
+			}
+			else if ( isIntegral ) {
 				LongToLine(a,anumer);
-				if ( anumer > 1 ) {
-					if (  ( AO.DoubleFlag & 2 ) == 2 ) { AddToLine((UBYTE *)".Q0"); }
-					else { AddToLine((UBYTE *)".D0"); }
-				}
-				a += na;
-				AddToLine((UBYTE *)"/");
-				LongToLine(a,adenom);
-				if ( adenom > 1 ) {
-					if (  ( AO.DoubleFlag & 2 ) == 2 ) { AddToLine((UBYTE *)".Q0"); }
-					else { AddToLine((UBYTE *)".D0"); }
-				}
+				AddToLine(suffNum);
 			}
 			else {
+				// rational
 				AddToLine((UBYTE *)"((one*");
 				LongToLine(a,anumer);
-				a += na;
-				AddToLine((UBYTE *)")/");
-				LongToLine(a,adenom);
-				AddToLine((UBYTE *)")");
+				AddToLine(suffNum);
+				AddToLine((UBYTE*)")/");
+				LongToLine(a+na,adenom);
+				AddToLine(suffDen);
+				AddToLine((UBYTE*)")");
 			}
-		  }
 		}
+		// All other modes use the same printing code:
 		else {
-		  UnPack(a,na,&adenom,&anumer);
-		  LongToLine(a,anumer);
-		  a += na;
-		  if ( anumer && !( adenom == 1 && *a == 1 ) ) {
-			if ( AC.OutputMode == FORTRANMODE && AC.IsFortran90 == ISFORTRAN90 ) {
-				if ( AC.Fortran90Kind ) {
-					AddToLine(AC.Fortran90Kind);
-					AddToLine((UBYTE *)"/");
-				}
-				else {
-					AddToLine((UBYTE *)"./");
-				}
+			// Numerator:
+			LongToLine(a,anumer);
+			AddToLine(suffNum);
+			// Denominator, if it is not 1:
+			if (!isIntegral) {
+				AddToLine((UBYTE*)"/");
+				LongToLine(a+na,adenom);
+				AddToLine(suffDen);
 			}
-			else if ( AC.OutputMode == FORTRANMODE || AC.OutputMode == CMODE ) {
-				if ( ( AO.DoubleFlag & 2 ) == 2 ) { AddToLine((UBYTE *)".Q0/"); }
-				else if ( ( AO.DoubleFlag & 1 ) == 1 ) { AddToLine((UBYTE *)".D0/"); }
-				else { AddToLine((UBYTE *)"./"); }
-			}
-			else AddToLine((UBYTE *)"/");
-			LongToLine(a,adenom);
-			if ( AC.OutputMode == FORTRANMODE && AC.IsFortran90 == ISFORTRAN90 ) {
-				if ( AC.Fortran90Kind ) {
-					AddToLine(AC.Fortran90Kind);
-				}
-				else {
-					AddToLine((UBYTE *)".");
-				}
-			}
-			else if ( AC.OutputMode == FORTRANMODE || AC.OutputMode == CMODE ) {
-				if ( ( AO.DoubleFlag & 2 ) == 2 ) { AddToLine((UBYTE *)".Q0"); }
-				else if ( ( AO.DoubleFlag & 1 ) == 1 ) { AddToLine((UBYTE *)".D0"); }
-				else { AddToLine((UBYTE *)"."); }
-			}
-		  }
-		  else if ( ( anumer > 1 || ( AO.DoubleFlag & 4 ) == 4 ) && ( AC.OutputMode == FORTRANMODE
-		  || AC.OutputMode == CMODE ) ) {
-			if ( AC.OutputMode == FORTRANMODE && AC.IsFortran90 == ISFORTRAN90 ) {
-				if ( AC.Fortran90Kind ) {
-					AddToLine(AC.Fortran90Kind);
-				}
-				else {
-					AddToLine((UBYTE *)".");
-				}
-			}
-			else if ( ( AO.DoubleFlag & 2 ) == 2 ) { AddToLine((UBYTE *)".Q0"); }
-			else if ( ( AO.DoubleFlag & 1 ) == 1 ) { AddToLine((UBYTE *)".D0"); }
-			else { AddToLine((UBYTE *)"."); }
-		  }
-		  else if ( AC.OutputMode == FORTRANMODE && AC.IsFortran90 == ISFORTRAN90 ) {
-				if ( AC.Fortran90Kind ) {
-					AddToLine(AC.Fortran90Kind);
-				}
-				else {
-					AddToLine((UBYTE *)".");
-				}
-		  }
-		  else if ( ( AC.OutputMode == FORTRANMODE || AC.OutputMode == CMODE )
-		  && AO.DoubleFlag ) {
-			if ( anumer == 1 && adenom == 1 && a[0] == 1 &&
-				 ( AO.DoubleFlag & 4 ) == 0 ) {}
-			else if ( ( AO.DoubleFlag & 2 ) == 2 ) { AddToLine((UBYTE *)".Q0"); }
-			else if ( ( AO.DoubleFlag & 1 ) == 1 ) { AddToLine((UBYTE *)".D0"); }
-		  }
 		}
 	}
+
 	else {
 /*
 		This is the float mode
@@ -484,12 +454,12 @@ VOID RatToLine(UWORD *a, WORD na)
 			Divvy(BHEAD c,&na,&b,1);
 			UnPack(c,na,&adenom,&anumer);
 			exponent++;
-		}		
+		}
 		while ( BigLong(c,anumer,c+na,adenom) < 0 ) {
 			Mully(BHEAD c,&na,&b,1);
 			UnPack(c,na,&adenom,&anumer);
 			exponent--;
-		}		
+		}
 /*
 		Now division will give a number between 1 and 9
 */
@@ -523,7 +493,7 @@ VOID RatToLine(UWORD *a, WORD na)
 
 /*
  		#] RatToLine : 
- 		#[ TalToLine :			VOID TalToLine(x)
+ 		#[ TalToLine :			void TalToLine(x)
 
 	Writes the unsigned number x to the output as a single token.
 	Par indicates the number of leading blanks in the line.
@@ -531,7 +501,7 @@ VOID RatToLine(UWORD *a, WORD na)
 
 */
 
-VOID TalToLine(UWORD x)
+void TalToLine(UWORD x)
 {
 	UBYTE t[BITSINWORD/3+1];
 	UBYTE *s;
@@ -548,7 +518,7 @@ VOID TalToLine(UWORD x)
 
 /*
  		#] TalToLine : 
- 		#[ TokenToLine :		VOID TokenToLine(s)
+ 		#[ TokenToLine :		void TokenToLine(s)
 
 	Puts s in the output buffer. If it doesn't fit the buffer is
 	flushed first. This routine keeps tokens as one unit.
@@ -560,7 +530,7 @@ VOID TalToLine(UWORD x)
 	digits!
 */
 
-VOID TokenToLine(UBYTE *s)
+void TokenToLine(UBYTE *s)
 {
 	UBYTE *t, *Out;
 	LONG num, i = 0, j;
@@ -661,7 +631,7 @@ VOID TokenToLine(UBYTE *s)
 
 /*
  		#] TokenToLine : 
- 		#[ CodeToLine :			VOID CodeToLine(name,number,mode)
+ 		#[ CodeToLine :			void CodeToLine(name,number,mode)
 
 	Writes a name and possibly its number to output as a single token.
 
@@ -680,7 +650,7 @@ UBYTE *CodeToLine(WORD number, UBYTE *Out)
  		#[ MultiplyToLine :
 */
 
-void MultiplyToLine()
+void MultiplyToLine(void)
 {
 	int i;
 	if ( AO.CurrentDictionary > 0 && AO.CurDictSpecials > 0
@@ -722,10 +692,10 @@ UBYTE *AddArrayIndex(WORD num,UBYTE *out)
 
 /*
  		#] AddArrayIndex : 
- 		#[ PrtTerms :			VOID PrtTerms()
+ 		#[ PrtTerms :			void PrtTerms()
 */
 
-VOID PrtTerms()
+void PrtTerms(void)
 {
 	UWORD a[2];
 	WORD na;
@@ -806,7 +776,7 @@ void PrintTime(UBYTE *mess)
  		#] PrintTime : 
   	#] schryf-Utilities : 
  	#[ schryf-Writes :
- 		#[ WriteLists :			VOID WriteLists()
+ 		#[ WriteLists :			void WriteLists()
 
 	Writes the namelists. If mode > 0 also the internal codes are given.
 
@@ -819,7 +789,7 @@ static UBYTE *rsymname[] = {
 	 (UBYTE *)"(-cyclic)",(UBYTE *)"(-reversecyclic)"
 	,(UBYTE *)"(-symmetric)",(UBYTE *)"(-antisymmetric)" };
 
-VOID WriteLists()
+void WriteLists(void)
 {
 	GETIDENTITY
 	WORD i, j, k, *skip;
@@ -959,7 +929,10 @@ VOID WriteLists()
 				else if ( ( functions[i].complex & VARTYPECOMPLEX ) == VARTYPECOMPLEX ) {
 					Out = StrCopy((UBYTE *)"#c",Out);
 				}
-				if ( functions[i].spec >= TENSORFUNCTION ) {
+				if ( functions[i].spec == VERTEXFUNCTION ) {
+					Out = StrCopy((UBYTE *)"(Particle)",Out);
+				}
+				else if ( functions[i].spec >= TENSORFUNCTION ) {
 					Out = StrCopy((UBYTE *)"(Tensor)",Out);
 				}
 				if ( functions[i].symmetric > 0 ) {
@@ -1039,7 +1012,7 @@ VOID WriteLists()
 				element = Sets[i].first;
 				LastElement = Sets[i].last;
 				type = Sets[i].type;
-				do {
+				while ( element < LastElement ) {
 					TokenToLine((UBYTE *)" ");
 					number = SetElements[element++];
 					switch ( type ) {
@@ -1107,7 +1080,7 @@ VOID WriteLists()
 							TokenToLine(OutScr);
 							break;
 					}
-				} while ( element < LastElement );
+				}
 			}
 		}
 		*skip = 0;
@@ -1424,13 +1397,13 @@ void WriteDictionary(DICTIONARY *dict)
 
 /*
  		#] WriteDictionary : 
- 		#[ WriteArgument :		VOID WriteArgument(WORD *t)
+ 		#[ WriteArgument :		void WriteArgument(WORD *t)
 
 		Write a single argument field. The general field goes to
 		WriteExpression and the fast field is dealt with here.
 */
 
-VOID WriteArgument(WORD *t)
+void WriteArgument(WORD *t)
 {
 	UBYTE buffer[180];
 	UBYTE *Out;
@@ -1548,7 +1521,7 @@ UBYTE *specfunnames[NUMSPECS] = {
 	, (UBYTE *)"invfac" };
 */
 
-WORD WriteSubTerm(WORD *sterm, WORD first)
+int WriteSubTerm(WORD *sterm, WORD first)
 {
 	UBYTE buffer[80];
 	UBYTE *Out, closepar[2] = { (UBYTE)')', 0};
@@ -1767,6 +1740,8 @@ WORD WriteSubTerm(WORD *sterm, WORD first)
 				if ( !first ) MultiplyToLine();
 				if ( AC.OutputMode == CMODE && t[2] != 1 )
 					TokenToLine((UBYTE *)"pow(");
+				if ( AC.OutputMode == MATHEMATICAMODE )
+					TokenToLine((UBYTE *)"(");
 				Out = StrCopy(FindVector(*t),buffer);
 /*				Out = StrCopy(VARNAME(vectors,*t - AM.OffsetVector),buffer); */
 				t++;
@@ -1776,6 +1751,10 @@ WORD WriteSubTerm(WORD *sterm, WORD first)
 				else *Out++ = '.';
 				Out = StrCopy(FindVector(*t),Out);
 /*				Out = StrCopy(VARNAME(vectors,*t - AM.OffsetVector),Out); */
+				if ( AC.OutputMode == MATHEMATICAMODE ) {
+					*Out++ = ')';
+					*Out = 0;
+				}
 				t++;
 				if ( *t != 1 ) WrtPower(Out,*t);
 				t++;
@@ -1951,9 +1930,13 @@ WORD WriteSubTerm(WORD *sterm, WORD first)
 
 */
 
-WORD WriteInnerTerm(WORD *term, WORD first)
+int WriteInnerTerm(WORD *term, WORD first)
 {
 	WORD *t, *s, *s1, *s2, n, i, pow;
+#ifdef WITHFLOAT
+	int FloatChars = 0;
+	GETIDENTITY
+#endif
 	t = term;
 	s = t+1;
 	GETCOEF(t,n);
@@ -2005,6 +1988,31 @@ WORD WriteInnerTerm(WORD *term, WORD first)
 		else                            RatToLine((UWORD *)t,n);
 		first = 0;
 	}
+#ifdef WITHFLOAT
+/*
+	Check whether there is a 'proper' float_ function and no raw mode.
+	If so, print as float.
+	Raw mode is indicated as AO.FloatPrec < 0.
+	AO.FloatPrec == 0 indicated as many digits as the precision allows.
+*/
+	else if ( AO.FloatPrec >= 0 && AT.aux_ != 0 ) {
+		WORD *ss = s;
+		while ( ss < t ) {
+			if ( *ss == FLOATFUN ) {
+				if ( ( FloatChars = PrintFloat(ss,AO.FloatPrec) ) != 0 ) {
+					TokenToLine(AO.floatspace);
+					if ( AC.IsFortran90 == ISFORTRAN90 && AC.Fortran90Kind ) { 
+						AddToLine(AC.Fortran90Kind);
+					}
+					first = 0;
+				}
+				break;
+			}
+			ss += ss[1];
+		}
+		if ( ss >= t ) first = 1;
+	}
+#endif
 	else first = 1;
 	while ( s < t ) {
 		if ( lowestlevel && ( (AO.PrintType & (PRINTONEFUNCTION | PRINTALL)) == PRINTONEFUNCTION ) ) {
@@ -2050,6 +2058,11 @@ WORD WriteInnerTerm(WORD *term, WORD first)
 /*
  		#] NEWGAMMA : 
 */
+#ifdef WITHFLOAT
+		if ( *s == FLOATFUN && AO.FloatPrec >= 0 && AT.aux_ != 0 ) {
+		}
+		else 
+#endif
 		{
 			if ( *s >= FUNCTION && AC.funpowers > 0
 			&& functions[*s-FUNCTION].spec == 0 && ( AC.funpowers == ALLFUNPOWERS ||
@@ -2117,7 +2130,7 @@ WORD WriteInnerTerm(WORD *term, WORD first)
 
 */
 
-WORD WriteTerm(WORD *term, WORD *lbrac, WORD first, WORD prtf, WORD br)
+int WriteTerm(WORD *term, WORD *lbrac, WORD first, WORD prtf, WORD br)
 {
 	WORD *t, *stopper, *b, n;
 	int oldIsFortran90 = AC.IsFortran90, i;
@@ -2439,7 +2452,7 @@ WrtTmes:				t = term;
 
 */
 
-WORD WriteExpression(WORD *terms, LONG ltot)
+int WriteExpression(WORD *terms, LONG ltot)
 {
 	WORD *stopper;
 	WORD first, btot;
@@ -2470,7 +2483,7 @@ WORD WriteExpression(WORD *terms, LONG ltot)
 		Writes all expressions that should be written
 */
 
-WORD WriteAll()
+int WriteAll(void)
 {
 	GETIDENTITY
 	WORD lbrac, first;
@@ -2488,7 +2501,7 @@ WORD WriteAll()
 		 */
 		for ( n = 0; n < NumExpressions; n++ ) {
 			e = &Expressions[n];
-			if ( !e->printflag & PRINTON ) continue;
+			if ( (!e->printflag) & PRINTON ) continue;
 			switch ( e->status ) {
 				case LOCALEXPRESSION:
 				case GLOBALEXPRESSION:
@@ -2599,6 +2612,9 @@ WORD WriteAll()
 				TokenToLine((UBYTE *)")");
 			}
 			TOKENTOLINE(" =","=");
+			if ( AC.OutputMode == MATHEMATICAMODE ) {
+				TOKENTOLINE(" (","(");
+			}
 			lbrac = 0;
 			AO.InFbrack = 0;
 			if ( AC.OutputMode == FORTRANMODE || AC.OutputMode == PFORTRANMODE )
@@ -2652,6 +2668,9 @@ WORD WriteAll()
 					PrtTerms();
 					TOKENTOLINE(" )",")")
 				}
+				if ( AC.OutputMode == MATHEMATICAMODE ) {
+					TokenToLine((UBYTE *)")");
+				}
 				if ( AC.OutputMode != FORTRANMODE && AC.OutputMode != PFORTRANMODE )
 					TokenToLine((UBYTE *)";");
 			}
@@ -2690,7 +2709,7 @@ AboWrite:
 		Writes one expression from the preprocessor
 */
 
-WORD WriteOne(UBYTE *name, int alreadyinline, int nosemi, WORD plus)
+int WriteOne(UBYTE *name, int alreadyinline, int nosemi, WORD plus)
 {
 	GETIDENTITY
 	WORD number;
