@@ -4,7 +4,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -57,6 +57,9 @@ SETUPPARAMETERS setupparameters[] =
 	,{(UBYTE *)"compresssize",          NUMERICALVALUE, 0, (LONG)COMPRESSBUFFER}
 	,{(UBYTE *)"constindex",            NUMERICALVALUE, 0, (LONG)NUMFIXED}
 	,{(UBYTE *)"continuationlines",     NUMERICALVALUE, 0, (LONG)FORTRANCONTINUATIONLINES}
+#ifdef WITHFLOAT
+	,{(UBYTE *)"defaultprecision",      NUMERICALVALUE, 0, (LONG)DEFAULTPRECISION}
+#endif
 	,{(UBYTE *)"define",                   DEFINEVALUE, 0, (LONG)0}
 	,{(UBYTE *)"dotchar",                  STRINGVALUE, 0, (LONG)dotchar}
 	,{(UBYTE *)"factorizationcache",    NUMERICALVALUE, 0, (LONG)FBUFFERSIZE}
@@ -72,6 +75,9 @@ SETUPPARAMETERS setupparameters[] =
 	,{(UBYTE *)"maxnumbersize",         NUMERICALVALUE, 0, (LONG)0}
 /*	,{(UBYTE *)"maxnumbersize",         NUMERICALVALUE, 0, (LONG)MAXNUMBERSIZE} */
 	,{(UBYTE *)"maxtermsize",           NUMERICALVALUE, 0, (LONG)MAXTER}
+#ifdef WITHFLOAT
+	,{(UBYTE *)"maxweight",             NUMERICALVALUE, 0, (LONG)MAXWEIGHT}
+#endif
 	,{(UBYTE *)"maxwildcards",          NUMERICALVALUE, 0, (LONG)MAXWILDC}
 	,{(UBYTE *)"nospacesinnumbers",         ONOFFVALUE, 0, (LONG)0}
 	,{(UBYTE *)"numstorecaches",        NUMERICALVALUE, 0, (LONG)NUMSTORECACHES}
@@ -123,7 +129,7 @@ SETUPPARAMETERS setupparameters[] =
  		#[ DoSetups :
 */
 
-int DoSetups()
+int DoSetups(void)
 {
 	UBYTE *setbuffer, *s, *t, *u /*, c */;
 	int errors = 0;
@@ -265,7 +271,7 @@ restart:;
 				}
 				*t = 0;
 				if ( sp->flags == USEDFLAG && sp->value != 0 )
-						M_free((VOID *)(sp->value),"Process option");
+						M_free((void *)(sp->value),"Process option");
 				sp->value = (LONG)strDup1(s2,"Process option");
 				sp->flags = USEDFLAG;
 				break;
@@ -283,7 +289,7 @@ restart:;
 					break;
 				}
 				if ( sp->flags == USEDFLAG && sp->value != 0 )
-					M_free((VOID *)(sp->value),"Process option");
+					M_free((void *)(sp->value),"Process option");
 				sp->value = (LONG)strDup1(s2,"Process option");
 				sp->flags = USEDFLAG;
 				break;
@@ -332,7 +338,8 @@ SETUPPARAMETERS *GetSetupPar(UBYTE *s)
 {
 	int hi, med, lo, i;
 	lo = 0;
-	hi = sizeof(setupparameters)/sizeof(SETUPPARAMETERS);
+	// -1: remove possibility of out-of-bounds read in StrICmp
+	hi = sizeof(setupparameters)/sizeof(SETUPPARAMETERS) - 1;
 	do {
 		med = ( hi + lo ) / 2;
 		i = StrICmp(s,(UBYTE *)setupparameters[med].parameter);
@@ -348,7 +355,7 @@ SETUPPARAMETERS *GetSetupPar(UBYTE *s)
  		#[ RecalcSetups :
 */
 
-int RecalcSetups()
+int RecalcSetups(void)
 {
 	SETUPPARAMETERS *sp, *sp1;
 
@@ -395,7 +402,7 @@ int RecalcSetups()
  		#[ AllocSetups :
 */
 
-int AllocSetups()
+int AllocSetups(void)
 {
 	SETUPPARAMETERS *sp;
 	LONG LargeSize, SmallSize, SmallEsize, TermsInSmall, IOsize;
@@ -567,7 +574,8 @@ int AllocSetups()
 #endif
 	AM.S0 = 0;
 	AM.S0 = AllocSort(LargeSize,SmallSize,SmallEsize,TermsInSmall
-					,MaxPatches,MaxFpatches,IOsize);
+					,MaxPatches,MaxFpatches,IOsize,0);
+	/* AM.S0->file.ziosize was already set to a (larger) value by AllocSort, here it is re-set. */
 #ifdef WITHZLIB
 	AM.S0->file.ziosize = IOsize;
 #ifndef WITHPTHREADS
@@ -598,6 +606,8 @@ int AllocSetups()
 	AM.SMaxFpatches = sp->value;
 	sp = GetSetupPar((UBYTE *)"subsortiosize");
 	AM.SIOsize = sp->value;
+	/* As for IOsize, make sure SIOsize is at least as large as AM.MaxTer. */
+	if ( AM.SIOsize < AM.MaxTer ) { AM.SIOsize = AM.MaxTer; sp->value = AM.SIOsize; }
 	sp = GetSetupPar((UBYTE *)"spectatorsize");
 	AM.SpectatorSize = sp->value;
 /*
@@ -692,6 +702,12 @@ int AllocSetups()
 	sp = GetSetupPar((UBYTE *)"wtimestats");
 	if ( sp->value == 2 ) sp->value = AM.ggWTimeStatsFlag;
 	AC.WTimeStatsFlag = AM.gWTimeStatsFlag = AM.ggWTimeStatsFlag = sp->value;
+#ifdef WITHFLOAT
+	sp = GetSetupPar((UBYTE *)"maxweight");
+	AC.MaxWeight = AM.gMaxWeight = AM.ggMaxWeight = sp->value;
+	sp = GetSetupPar((UBYTE *)"defaultprecision");
+	AC.DefaultPrecision = AM.gDefaultPrecision = AM.ggDefaultPrecision = sp->value;
+#endif
 	sp = GetSetupPar((UBYTE *)"sorttype");
 	if ( StrICmp((UBYTE *)"lowfirst",(UBYTE *)sp->value) == 0 ) {
 		AC.lSortType = SORTLOWFIRST;
@@ -714,6 +730,8 @@ int AllocSetups()
 	AM.NumStoreCaches = sp->value;
 	sp = GetSetupPar((UBYTE *)"sizestorecache");
 	AM.SizeStoreCache = sp->value;
+	/* Make sure this is a multiple of sizeof(WORD). */
+	AM.SizeStoreCache = ((AM.SizeStoreCache+sizeof(WORD)-1)/sizeof(WORD))*sizeof(WORD);
 #ifndef WITHPTHREADS
 /*
 	Install the store caches (15-aug-2006 JV)
@@ -728,7 +746,7 @@ int AllocSetups()
 		AT.StoreCache = AT.StoreCacheAlloc;
 		sa = AT.StoreCache;
 		for ( j = 0; j < AM.NumStoreCaches; j++ ) {
-			sb = (STORECACHE)(VOID *)((UBYTE *)sa+size);
+			sb = (STORECACHE)(void *)((UBYTE *)sa+size);
 			if ( j == AM.NumStoreCaches-1 ) {
 				sa->next = 0;
 			}
@@ -779,7 +797,7 @@ int AllocSetups()
 	and have that included into the LaTeX file.
 */
 
-VOID WriteSetup()
+void WriteSetup(void)
 {
 	int n = sizeof(setupparameters)/sizeof(SETUPPARAMETERS);
 	SETUPPARAMETERS *sp;
@@ -833,21 +851,32 @@ VOID WriteSetup()
 		Routine allocates a complete struct for sorting.
 		To be used for the main allocation of the sort buffers, and
 		in a later stage for the function and subroutine sort buffers.
+		The level arg denotes a main buffer allocation (0) or a sub-buffer
+		allocation (1), used only for printing warning messages for buffer
+		size adjustments in DEBUGGING mode.
 */
-
-SORTING *AllocSort(LONG LargeSize, LONG SmallSize, LONG SmallEsize, LONG TermsInSmall,
-                   int MaxPatches, int MaxFpatches, LONG IOsize)
+SORTING *AllocSort(LONG inLargeSize, LONG inSmallSize, LONG inSmallEsize, LONG inTermsInSmall,
+                   int inMaxPatches, int inMaxFpatches, LONG inIOsize, int level)
 {
-	LONG allocation,longer,terms2insmall,sortsize,longerp;
+	DUMMYUSE(level); /* This is only used in DEBUGGING mode */
+	LONG LargeSize = inLargeSize;
+	LONG SmallSize = inSmallSize;
+	LONG SmallEsize = inSmallEsize;
+	LONG TermsInSmall = inTermsInSmall;
+	int MaxPatches = inMaxPatches;
+	int MaxFpatches = inMaxFpatches;
+	LONG IOsize = inIOsize;
+
+	LONG longer,terms2insmall,sortsize,longerp;
 	LONG IObuffersize = IOsize;
 	LONG IOtry;
 	SORTING *sort;
-	int i = 0, j = 0;
+	int fname2Size = 0, j = 0;
 	char *s;
 	if ( AM.S0 != 0 ) {
-		s = FG.fname2; i = 0;
-		while ( *s ) { s++; i++; }
-		i += 16;
+		s = FG.fname2; fname2Size = 0;
+		while ( *s ) { s++; fname2Size++; }
+		fname2Size += 16;
 	}
 	if ( MaxFpatches < 4 ) MaxFpatches = 4;
 	longer = MaxPatches > MaxFpatches ? MaxPatches : MaxFpatches;
@@ -862,7 +891,6 @@ SORTING *AllocSort(LONG LargeSize, LONG SmallSize, LONG SmallEsize, LONG TermsIn
 	terms2insmall = 2*TermsInSmall;  /* Used to be just + 100 rather than *2 */
 	if ( SmallEsize < (SmallSize*3)/2 ) SmallEsize = (SmallSize*3)/2;
 	if ( LargeSize > 0 && LargeSize < 2*SmallSize ) LargeSize = 2*SmallSize;
-/*	if ( SmallEsize < 3*AM.MaxTer ) SmallEsize = 3*AM.MaxTer; */
 	SmallEsize = (SmallEsize+15) & (-16L);
 	if ( LargeSize < 0 ) LargeSize = 0;
 	sortsize = sizeof(SORTING);
@@ -884,24 +912,27 @@ SORTING *AllocSort(LONG LargeSize, LONG SmallSize, LONG SmallEsize, LONG TermsIn
 
 	IOtry = ((LargeSize+SmallEsize)/MaxFpatches-2*AM.MaxTer)/sizeof(WORD)-COMPINC;
 
-	if ( (LONG)(IObuffersize*sizeof(WORD)) < IOtry )
-		IObuffersize = (IOtry+sizeof(WORD)-1)/sizeof(WORD);
+	/* Here both IObuffersize and IOtry are in units of sizeof(WORD). */
+	if ( IObuffersize < IOtry ) {
+		IObuffersize = IOtry;
+	}
 
-	allocation =
-		 3*sizeof(POSITION)*(LONG)longer				/* Filepositions!! */
-		+2*sizeof(WORD *)*longer
-		+2*(longerp*(sizeof(WORD *)+sizeof(WORD)))
-		+(3*longerp+2)*sizeof(WORD)
-#ifdef WITHZLIB
-		+(2*longerp+4)*sizeof(WORD)
+#if DEBUGGING
+	char *prefix;
+	if ( level == 0 ) { prefix = ""; }
+	else { prefix = "Sub"; }
+	if ( LargeSize != inLargeSize ) { MesPrint("Warning: %sLargeSize adjusted: %l -> %l", prefix, inLargeSize, LargeSize); }
+	if ( SmallSize != inSmallSize ) { MesPrint("Warning: %sSmallSize adjusted: %l -> %l", prefix, inSmallSize, SmallSize); }
+	if ( SmallEsize != inSmallEsize ) {MesPrint("Warning: %sSmallEsize adjusted: %l -> %l", prefix, inSmallEsize, SmallEsize); }
+	if ( TermsInSmall != inTermsInSmall ) { MesPrint("Warning: %sTermsInSmall adjusted: %l -> %l", prefix, inTermsInSmall, TermsInSmall); }
+	if ( MaxPatches != inMaxPatches ) { MesPrint("Warning: MaxPatches adjusted: %d -> %d", inMaxPatches, MaxPatches); }
+	if ( MaxFpatches != inMaxFpatches ) {MesPrint("Warning: MaxFPatches adjusted: %d -> %d", inMaxFpatches, MaxFpatches); }
+	/* This one is always changed if the LargeSize has not been... */
+	/* if ( IObuffersize != inIOsize/(LONG)sizeof(WORD) ) { MesPrint("Warning: IOsize adjusted: %l -> %l", inIOsize/sizeof(WORD), IObuffersize); } */
 #endif
-		+terms2insmall*sizeof(WORD *)
-		+terms2insmall*sizeof(WORD *)/2
-		+LargeSize
-		+SmallEsize
-		+sortsize
-		+IObuffersize*sizeof(WORD) + i + 16;
-	sort = (SORTING *)Malloc1(allocation,"sort buffers");
+
+	/* Allocate separate buffers for most struct members, for better testing and debugging with valgrind. */
+	sort = Malloc1(sizeof(*sort), "AllocSort: sorting struct");
 
 	sort->LargeSize = LargeSize/sizeof(WORD);
 	sort->SmallSize = SmallSize/sizeof(WORD);
@@ -911,33 +942,38 @@ SORTING *AllocSort(LONG LargeSize, LONG SmallSize, LONG SmallEsize, LONG TermsIn
 	sort->TermsInSmall = TermsInSmall;
 	sort->Terms2InSmall = terms2insmall;
 
-	sort->sPointer = (WORD **)(sort+1);
-	sort->SplitScratch = sort->sPointer + terms2insmall;
-	sort->Patches = (WORD **)(sort->SplitScratch + terms2insmall/2);
-	sort->pStop = sort->Patches+longer;
-	sort->poina = sort->pStop+longer;
-	sort->poin2a = sort->poina + longerp;
-	sort->fPatches = (POSITION *)(sort->poin2a+longerp);
-	sort->fPatchesStop = sort->fPatches + longer;
-	sort->inPatches = sort->fPatchesStop + longer;
-	sort->tree = (WORD *)(sort->inPatches + longer);
-	sort->used = sort->tree+longerp;
+	sort->sPointer     = Malloc1(sizeof(*(sort->sPointer    ))*terms2insmall, "AllocSort: sPointer");
+	sort->Patches      = Malloc1(sizeof(*(sort->Patches     ))*longer, "AllocSort: Patches");
+	sort->pStop        = Malloc1(sizeof(*(sort->pStop       ))*longer, "AllocSort: pStop");
+	sort->poina        = Malloc1(sizeof(*(sort->poina       ))*longerp, "AllocSort: poina");
+	sort->poin2a       = Malloc1(sizeof(*(sort->poin2a      ))*longerp, "AllocSort: poin2a");
+
+	sort->fPatches     = Malloc1(sizeof(*(sort->fPatches    ))*longer, "AllocSort: fPatches");
+	sort->fPatchesStop = Malloc1(sizeof(*(sort->fPatchesStop))*longer, "AllocSort: fPatchesStop");
+	sort->inPatches    = Malloc1(sizeof(*(sort->inPatches   ))*longer, "AllocSort: inPatches");
+	sort->tree         = Malloc1(sizeof(*(sort->tree        ))*longerp, "AllocSort: tree");
+	sort->used         = Malloc1(sizeof(*(sort->used        ))*longerp, "AllocSort: used");
+
 #ifdef WITHZLIB
-	sort->fpcompressed = sort->used+longerp;
-	sort->fpincompressed = sort->fpcompressed+longerp+2;
-	sort->ktoi = sort->fpincompressed+longerp+2;
+	sort->fpcompressed   = Malloc1(sizeof(*(sort->fpcompressed  ))*(longerp+2), "AllocSort: fpcompressed");
+	sort->fpincompressed = Malloc1(sizeof(*(sort->fpincompressed))*(longerp+2), "AllocSort: fpincompressed");
 	sort->zsparray = 0;
-#else
-	sort->ktoi = sort->used + longerp;
 #endif
-	sort->lBuffer = (WORD *)(sort->ktoi + longerp + 2);
+
+	sort->ktoi         = Malloc1(sizeof(*(sort->ktoi))*(longerp+2), "AllocSort: ktoi");
+
+	// The combined Large buffer and Small buffer (+ extension) are used.
+	// They must be allocated together.
+	sort->lBuffer      = Malloc1(sizeof(*(sort->lBuffer))*(sort->LargeSize+sort->SmallEsize), "AllocSort: lBuffer+sBuffer");
 	sort->lTop = sort->lBuffer+sort->LargeSize;
+
 	sort->sBuffer = sort->lTop;
 	if ( sort->LargeSize == 0 ) { sort->lBuffer = 0; sort->lTop = 0; }
 	sort->sTop = sort->sBuffer + sort->SmallSize;
 	sort->sTop2 = sort->sBuffer + sort->SmallEsize;
 	sort->sHalf = sort->sBuffer + (LONG)((sort->SmallSize+sort->SmallEsize)>>1);
-	sort->file.PObuffer = (WORD *)(sort->sTop2);
+
+	sort->file.PObuffer = Malloc1(IObuffersize*sizeof(*(sort->file.PObuffer))+fname2Size+16, "AllocSort: PObuffer");
 	sort->file.POstop = sort->file.PObuffer+IObuffersize;
 	sort->file.POsize = IObuffersize * sizeof(WORD);
 	sort->file.POfill = sort->file.POfull = sort->file.PObuffer;
@@ -948,7 +984,6 @@ SORTING *AllocSort(LONG LargeSize, LONG SmallSize, LONG SmallEsize, LONG TermsIn
 	sort->file.pthreadslock = dummylock;
 #endif
 #ifdef WITHZLIB
-/*	sort->file.ziosize = IOsize; */
 	sort->file.ziosize = IObuffersize*sizeof(WORD);
 	sort->file.ziobuffer = 0;
 #endif
@@ -970,7 +1005,7 @@ SORTING *AllocSort(LONG LargeSize, LONG SmallSize, LONG SmallEsize, LONG TermsIn
  		#[ AllocSortFileName :
 */
 
-VOID AllocSortFileName(SORTING *sort)
+void AllocSortFileName(SORTING *sort)
 {
 	GETIDENTITY
 	char *s, *t;
@@ -982,10 +1017,10 @@ VOID AllocSortFileName(SORTING *sort)
 	while ( *s ) *t++ = *s++;
 #ifdef WITHPTHREADS
 	t[-2] = 'F';
-	sprintf(t-1,"%d.%d",identity,AN.filenum);
+	snprintf(t-1,FG.fname2size-(t-1-sort->file.name),"%d.%d",identity,AN.filenum);
 #else
 	t[-2] = 'f';
-	sprintf(t-1,"%d",AN.filenum);
+	snprintf(t-1,FG.fname2size-(t-1-sort->file.name),"%d",AN.filenum);
 #endif
 	AN.filenum++;
 }
@@ -1008,7 +1043,8 @@ FILEHANDLE *AllocFileHandle(WORD par,char *name)
 	if ( par == 0 ) { i += 16; Ssize = AM.SIOsize; }
 	else { s = name; while ( *s ) { i++; s++; } i+= 2; Ssize = AM.SpectatorSize; }
 
-	allocation = sizeof(FILEHANDLE) + (Ssize+1)*sizeof(WORD) + i*sizeof(char);
+	allocation = sizeof(FILEHANDLE) + (Ssize+1)*sizeof(WORD) + i*sizeof(char)
+			+FG.fname2size+20;
 	fh = (FILEHANDLE *)Malloc1(allocation,"FileHandle");
 
 	fh->PObuffer = (WORD *)(fh+1);
@@ -1027,10 +1063,10 @@ FILEHANDLE *AllocFileHandle(WORD par,char *name)
 		while ( *s ) *t++ = *s++;
 #ifdef WITHPTHREADS
 		t[-2] = 'F';
-		sprintf(t-1,"%d-%d",identity,AN.filenum);
+		snprintf(t-1,FG.fname2size-(t-1-fh->name),"%d-%d",identity,AN.filenum);
 #else
 		t[-2] = 'f';
-		sprintf(t-1,"%d",AN.filenum);
+		snprintf(t-1,FG.fname2size-(t-1-fh->name),"%d",AN.filenum);
 #endif
 		AN.filenum++;
 	  }
@@ -1072,7 +1108,7 @@ void DeAllocFileHandle(FILEHANDLE *fh)
  		#[ MakeSetupAllocs :
 */
 
-int MakeSetupAllocs()
+int MakeSetupAllocs(void)
 {
 	if ( RecalcSetups() || AllocSetups() ) return(1);
 	else return(0);
@@ -1092,7 +1128,7 @@ int MakeSetupAllocs()
 
 #define SETBUFSIZE 257
 
-int TryFileSetups()
+int TryFileSetups(void)
 {
 	LONG oldstreamposition;
 	int oldstream;
@@ -1131,7 +1167,7 @@ int TryFileSetups()
 		if ( c == ENDOFINPUT ) break;
 		if ( c == LINEFEED ) continue;
 		if ( c == 0 || c == ENDOFINPUT ) break;
-		while ( c != LINEFEED ) {
+		while ( c != LINEFEED && c != ENDOFINPUT ) {
 			*s++ = c;
 			c = GetInput();
 			if ( c != LINEFEED && c != '\r' ) continue;
@@ -1178,7 +1214,7 @@ eoi:
  		#[ TryEnvironment :
 */
 
-int TryEnvironment()
+int TryEnvironment(void)
 {
 	char *s, *t, *u, varname[100];
 	int i,imax = sizeof(setupparameters)/sizeof(SETUPPARAMETERS);
