@@ -5,7 +5,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -53,10 +53,11 @@
 		non-commuting objects are involved.
 */
 
-WORD MatchE(PHEAD WORD *pattern, WORD *fun, WORD *inter, WORD par)
+int MatchE(PHEAD WORD *pattern, WORD *fun, WORD *inter, WORD par)
 {
 	GETBIDENTITY
-	WORD *m, *t, *r, i, retval;
+	WORD *m, *t, *r, i;
+	int retval;
 	WORD *mstop, *tstop, j, newvalue, newfun;
 	WORD fixvec[MAXMATCH],wcvec[MAXMATCH],fixind[MAXMATCH],wcind[MAXMATCH];
 	WORD tfixvec[MAXMATCH],tfixind[MAXMATCH];
@@ -142,8 +143,8 @@ WORD MatchE(PHEAD WORD *pattern, WORD *fun, WORD *inter, WORD par)
 				else return(1);
 			}
 /*			newter = instart; */
-			i = ScanFunctions(BHEAD newpat,inter,par);
-			return(i);
+			retval = ScanFunctions(BHEAD newpat,inter,par);
+			return(retval);
 		}
 /*
 		Now the recursion
@@ -440,7 +441,7 @@ NoCaseB:		m = AN.WildValue;
 		hit in the assignment of wildcards though.
 */
 
-WORD Permute(PERM *perm, WORD first)
+int Permute(PERM *perm, WORD first)
 {
 	WORD *s, c, i, j;
 	if ( first ) {
@@ -474,7 +475,7 @@ WORD Permute(PERM *perm, WORD first)
 	Like Permute, but works on an array of pointers
 */
 
-WORD PermuteP(PERMP *perm, WORD first)
+int PermuteP(PERMP *perm, WORD first)
 {
 	WORD **s, *c, i, j;
 	if ( first ) {
@@ -506,7 +507,7 @@ WORD PermuteP(PERMP *perm, WORD first)
   	#[ Distribute :
 */
 
-WORD Distribute(DISTRIBUTE *d, WORD first)
+int Distribute(DISTRIBUTE *d, WORD first)
 {
 	WORD *to, *from, *inc, *from2, i, j;
 	if ( first ) {
@@ -531,7 +532,7 @@ WORD Distribute(DISTRIBUTE *d, WORD first)
 	inc = d->cycle;
 	from = inc + d->n;
 	while ( *inc ) { j++; inc++; }
-	while ( !*inc && inc < from ) { i++; inc++; }
+	while ( inc < from && !*inc ) { i++; inc++; }
 	if ( inc >= from ) return(1);
 	d->sign ^= ((i&j)-j+1) & 1;
 	*inc = 0;
@@ -799,8 +800,8 @@ int MatchCy(PHEAD WORD *pattern, WORD *fun, WORD *inter, WORD par)
 					if ( *p == FUNNYWILD ) {
 						p++; wc = 1;
 						AN.argaddress = t;
-						if ( CheckWild(BHEAD *p,ARGTOARG,funnycount|EATTENSOR,t) ) break;
-						AddWild(BHEAD *p,ARGTOARG,funnycount|EATTENSOR);
+						if ( CheckWild(BHEAD *p,ARGTOARG|EATTENSOR,funnycount,t) ) break;
+						AddWild(BHEAD *p,ARGTOARG|EATTENSOR,funnycount);
 						j += funnycount-1; t += funnycount-1;
 					}
 					else if ( *p >= AM.OffsetIndex + WILDOFFSET
@@ -957,8 +958,8 @@ int MatchCy(PHEAD WORD *pattern, WORD *fun, WORD *inter, WORD par)
 					if ( *p == FUNNYWILD ) {
 						p++; wc = thewildcards[renum[*p]];
 						AN.argaddress = t;
-						if ( CheckWild(BHEAD *p,ARGTOARG,wc|EATTENSOR,t) ) break;
-						AddWild(BHEAD *p,ARGTOARG,wc|EATTENSOR);
+						if ( CheckWild(BHEAD *p,ARGTOARG|EATTENSOR,wc,t) ) break;
+						AddWild(BHEAD *p,ARGTOARG|EATTENSOR,wc);
 						j += wc-1; t += wc-1; wc = 1;
 					}
 					else if ( *p >= AM.OffsetIndex + WILDOFFSET
