@@ -5,7 +5,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -59,6 +59,7 @@
 
 #include "form3.h"
 #include "minos.h"
+#include "comtool.h"
 
 /* static UBYTE *sparse = (UBYTE *)"sparse"; */
 static UBYTE *tablebase = (UBYTE *)"tablebase";
@@ -102,7 +103,7 @@ void ClearTableTree(TABLES T)
 int InsTableTree(TABLES T, WORD *tp)
 {
 	COMPTREE *boomlijst, *q, *p, *s;
-	WORD *v1, *v2, *v3;
+	WORD *v1, *v2, *v3, xstop;
 	int ip, iq, is;
 	if ( T->numtree + 1 >= T->MaxTreeSize ) {
 		if ( T->MaxTreeSize == 0 ) ClearTableTree(T);
@@ -129,10 +130,12 @@ int InsTableTree(TABLES T, WORD *tp)
 		return(T->numtree);
 	}
 	ip = q->right;
+	if ( T->numind >= 0 ) xstop = T->numind;
+	else xstop = *tp + 1;
 	while ( ip >= 0 ) {
 		p = boomlijst + ip;
 		v1 = T->tablepointers + p->value;
-		v2 = tp; v3 = tp + T->numind;
+		v2 = tp; v3 = tp + xstop;
 		while ( *v1 == *v2 && v2 < v3 ) { v1++; v2++; }
 		if ( v2 >= v3 ) return(-p->value);
 		if ( *v1 > *v2 ) {
@@ -267,7 +270,7 @@ void RedoTableTree(TABLES T, int newsize)
 	ClearTableTree(T);
 	for ( i = 0, tp = T->tablepointers; i < newsize; i++ ) {
 		InsTableTree(T,tp);
-		tp += T->numind+TABLEEXTENSION;
+		tp += ABS(T->numind)+TABLEEXTENSION;
 	}
 }
 
@@ -288,15 +291,20 @@ void RedoTableTree(TABLES T, int newsize)
 int FindTableTree(TABLES T, WORD *tp, int inc)
 {
 	COMPTREE *boomlijst = T->boomlijst, *q = boomlijst + T->rootnum, *p;
-	WORD *v1, *v2, *v3;
+	WORD *v1, *v2, *v3, xstop;
 	int ip, iq;
 	if ( q->right == -1 ) return(-1);
 	ip = q->right;
 	if ( inc > 1 ) tp += inc-1;
+	if ( T->numind >= 0 ) xstop = T->numind;
+	else {	/* We have to read the number of arguments first */
+		if ( *tp <= 0 ) return(-1); /* Cannot be! */
+		xstop = *tp+1;
+	}
 	while ( ip >= 0 ) {
 		p = boomlijst + ip;
 		v1 = T->tablepointers + p->value;
-		v2 = tp; v3 = v1 + T->numind;
+		v2 = tp; v3 = v1 + xstop;
 		while ( *v1 == *v2 && v1 < v3 ) { v1++; v2 += inc; }
 		if ( v1 == v3 ) {
 			p->usage++;
@@ -323,10 +331,11 @@ int FindTableTree(TABLES T, WORD *tp, int inc)
   	#[ DoTableExpansion :
 */
 
-WORD DoTableExpansion(WORD *term, WORD level)
+int DoTableExpansion(WORD *term, WORD level)
 {
 	GETIDENTITY
-	WORD *t, *tstop, *stopper, *termout, *m, *mm, *tp, *r;
+	WORD *t, *tstop, *stopper, *termout, *m, *mm, *tp, *r, xx;
+	WORD numsubexp, numbuf;
 	TABLES T = 0;
 	int i, j, num;
 	AN.TeInFun = AR.TePos = 0;
@@ -340,11 +349,11 @@ WORD DoTableExpansion(WORD *term, WORD level)
 		if ( T == 0 ) { t += t[1]; continue; }
 		if ( T->spare ) T = T->spare;
 		if ( t[1] == FUNHEAD+2 && t[FUNHEAD+1] <= -FUNCTION ) break;
-		if ( t[1] < FUNHEAD+1+2*T->numind ) { t += t[1]; continue; }
-		for ( i = 0; i < T->numind; i++ ) {
+		if ( t[1] < FUNHEAD+1+2*ABS(T->numind) ) { t += t[1]; continue; }
+		for ( i = 0; i < ABS(T->numind); i++ ) {
 			if ( t[FUNHEAD+1+2*i] != -SYMBOL ) break;
 		}
-		if ( i >= T->numind ) break;
+		if ( i >= ABS(T->numind) ) break;
 		t += t[1];
 	}
 	if ( t >= stopper ) {
@@ -365,12 +374,32 @@ WORD DoTableExpansion(WORD *term, WORD level)
 			r = m;
 			if ( t[1] == FUNHEAD+2 && t[FUNHEAD+1] <= -FUNCTION ) {
 				*m++ = -t[FUNHEAD+1];
-				*m++ = FUNHEAD+T->numind*2;
-				for ( j = 2; j < FUNHEAD; j++ ) *m++ = 0;
-				tp = T->tablepointers + (T->numind+TABLEEXTENSION)*i;
-				for ( j = 0; j < T->numind; j++ ) {
-					*m++ = -SNUMBER; *m++ = *tp++;
+				tp = T->tablepointers + (ABS(T->numind)+TABLEEXTENSION)*i;
+				if ( T->numind < 0 ) {
+					xx = tp[0]+1;
+					*m++ = FUNHEAD+xx*2;
+					for ( j = 2; j < FUNHEAD; j++ ) *m++ = 0;
+					for ( j = 0; j < xx; j++ ) {
+						*m++ = -SNUMBER; *m++ = *tp++;
+					}
 				}
+				else {
+					*m++ = FUNHEAD+T->numind*2;
+					for ( j = 2; j < FUNHEAD; j++ ) *m++ = 0;
+					for ( j = 0; j < T->numind; j++ ) {
+						*m++ = -SNUMBER; *m++ = *tp++;
+					}
+				}
+			}
+			else if ( T->numind < 0 ) {
+				tp = T->tablepointers + (ABS(T->numind)+TABLEEXTENSION)*i;
+				xx = tp[0]+1;
+				*m++ = SYMBOL; *m++ = 2+xx*2; mm = t + FUNHEAD+1;
+				for ( j = 0; j < xx; j++, mm += 2, tp++ ) {
+					if ( *tp != 0 ) { *m++ = mm[1]; *m++ = *tp; }
+				}
+				r[1] = m-r;
+				if ( r[1] == 2 ) m = r;
 			}
 			else {
 				*m++ = SYMBOL; *m++ = 2+T->numind*2; mm = t + FUNHEAD+1;
@@ -398,21 +427,42 @@ WORD DoTableExpansion(WORD *term, WORD level)
 			any risks. There is still one problem. We have not checked
 			that the prototype matches.
 */
+			tp = T->tablepointers + (ABS(T->numind)+TABLEEXTENSION)*i
+					+ABS(T->numind);
+			numsubexp = tp[0]; numbuf = tp[1];
 			r = m;
+#ifdef WITHPTHREADS
+			tp = T->prototype[identity];
+#else
+			tp = T->prototype;
+#endif
+			for ( j = 0; j < tp[1]; j++ ) *m++ = tp[j];
+			r[2] = numsubexp; r[4] = numbuf;
+/*
+			r = m;
+			tp = T->tablepointers + (ABS(T->numind)+TABLEEXTENSION)*i;
 			*m++ = -t[FUNHEAD];
-			*m++ = t[1] - 1;
+			if ( T->numind < 0 ) {
+				xx = tp[0]+1;
+				*m++ = t[1] - xx - T->numind - 1;
+			}
+			else {
+				xx = T->numind;
+				*m++ = t[1] - 1;
+			}
 			for ( j = 2; j < FUNHEAD; j++ ) *m++ = t[j];
-			tp = T->tablepointers + (T->numind+TABLEEXTENSION)*i;
-			for ( j = 0; j < T->numind; j++ ) {
+			for ( j = 0; j < xx; j++ ) {
 				*m++ = -SNUMBER; *m++ = *tp++;
 			}
 			tp = t + FUNHEAD + 1 + 2*T->numind;
 			mm = t + t[1];
 			while ( tp < mm ) *m++ = *tp++;
 			r[1] = m-r;
+*/
 /*
 			From now on is old code
 */
+			mm = t + t[1];
 			while ( mm < tstop ) *m++ = *mm++;
 			*termout = m - termout;
 			AT.WorkPointer = m;
@@ -420,6 +470,7 @@ WORD DoTableExpansion(WORD *term, WORD level)
 				MesCall("DoTableExpand");
 				return(-1);
 			}
+			AT.WorkPointer = termout;
 		}
 	}
 	else {
@@ -520,6 +571,7 @@ WORD DoTableExpansion(WORD *term, WORD level)
 	TB,options;
 	Options are:
 		Open "File.tbl";                   Open for R/W
+		Open "File.tbl", readonly;         Open for R
 		Create "File.tbl";                 Create for write
 		Load "File.tbl", tablename;        Loads stubs of table
 		Load "File.tbl";                   Loads stubs of all tables
@@ -539,8 +591,8 @@ WORD DoTableExpansion(WORD *term, WORD level)
 	id tbl_(f?,?a) = f(?a);
 	When a tbl_ is used, automatically the corresponding element is compiled
 	at the start of the next module.
-	if TB,On,substitue [tablename], use of table RHS (if loaded)
-	if TB,Off,substitue [tablename], use of tbl_(table,...);
+	if TB,On,substitute [tablename], use of table RHS (if loaded)
+	if TB,Off,substitute [tablename], use of tbl_(table,...);
 
 
 	Still needed: Something like OverLoad to allow loading parts of a table
@@ -664,8 +716,8 @@ int SpareTable(TABLES TT)
 	T->reserved = 0;
 	T->tablenum = 0;
 	T->numdummies = 0;
-	T->mm = (MINMAX *)Malloc1(T->numind*sizeof(MINMAX),"table dimensions");
-	T->flags = (WORD *)Malloc1(T->numind*sizeof(WORD),"table flags");
+	T->mm = (MINMAX *)Malloc1(ABS(T->numind)*sizeof(MINMAX),"table dimensions");
+	T->flags = (WORD *)Malloc1(ABS(T->numind)*sizeof(WORD),"table flags");
 	ClearTableTree(T);
 	TT->spare = T;
 	TT->mode = 1;
@@ -720,12 +772,23 @@ int CoTBcreate(UBYTE *s)
 int CoTBopen(UBYTE *s)
 {
 	DBASE *d;
-	DUMMYUSE(s);
+	MLONG rw = 1;
+
+	SkipSpaces(&s);
+	
+	if ( *s ) {
+		if ( ConsumeOption(&s,"readonly") != 0 ) {
+			rw = 0;
+		} else {
+			MesPrint("&Invalid option for TableBase open: %s, ignoring", s);
+		}
+	}
+	
 	if ( ( d = FindTB(tablebasename) ) != 0 ) {
 		MesPrint("&There is already an open TableBase with the name %s",tablebasename);
 		return(-1);
 	}
-	d = GetDbase((char *)tablebasename);
+	d = GetDbase((char *)tablebasename, rw);
 	if ( CheckTableDeclarations(d) ) return(-1);
 	return(0);
 }
@@ -749,6 +812,11 @@ int CoTBaddto(UBYTE *s)
 		MesPrint("&No open tablebase with the name %s",tablebasename);
 		return(-1);
 	}
+	
+	if ( ( d->rwmode ) == 0 ) {
+		MesPrint("&Tablebase with the name %s opened in read only mode",tablebasename);
+		return(-1);
+	}
 	AO.DollarOutSizeBuffer = 32;
 	AO.DollarOutBuffer = (UBYTE *)Malloc1(AO.DollarOutSizeBuffer,
 							"TableOutBuffer");
@@ -776,7 +844,7 @@ int CoTBaddto(UBYTE *s)
 		*s = c;
 		if ( *s == '(' ) { /* Addition of single element */
 			s++; es = s;
-			for ( i = 0, w = AT.WorkPointer; i < T->numind; i++ ) {
+			for ( i = 0, w = AT.WorkPointer; i < ABS(T->numind); i++ ) {
 			  ParseSignedNumber(x,s);
 		      if ( FG.cTable[s[-1]] != 1 || ( *s != ',' && *s != ')' ) ) {
 				MesPrint("&Table arguments in TableBase addto statement should be numbers");
@@ -786,9 +854,9 @@ int CoTBaddto(UBYTE *s)
 			  if ( *s == ')' ) break;
 			  s++;
 			}
-			if ( *s != ')' || i < ( T->numind - 1 ) ) {
+			if ( *s != ')' || i < ( ABS(T->numind) - 1 ) ) {
 			  MesPrint("&Incorrect number of table arguments in TableBase addto statement. Should be %d"
-				,T->numind);
+				,ABS(T->numind));
 			  error = 1;
 			}
 			c = *s; *s = 0;
@@ -802,7 +870,7 @@ int CoTBaddto(UBYTE *s)
 			else {
 			  int dict = AO.CurrentDictionary;
 			  AO.CurrentDictionary = 0;
-			  sum = i + T->numind;
+			  sum = i + ABS(T->numind);
 /*
 			  See also commentary below
 */
@@ -840,9 +908,9 @@ int CoTBaddto(UBYTE *s)
 #else
 			if ( !T->sparse && T->tablepointers[TABLEEXTENSION*i] < 0 ) continue;
 #endif
-			sum = i * ( T->numind + TABLEEXTENSION );
+			sum = i * ( ABS(T->numind) + TABLEEXTENSION );
 			t = elementstring;
-			for ( j = 0; j < T->numind; j++, sum++ ) {
+			for ( j = 0; j < ABS(T->numind); j++, sum++ ) {
 				if ( j > 0 ) *t++ = ',';
 				num = T->tablepointers[sum];
 				t = NumCopy(num,t);
@@ -915,7 +983,7 @@ int CoTBenter(UBYTE *s)
 	int dict = AO.CurrentDictionary;
 	AO.CurrentDictionary = 0;
 	if ( ( d = FindTB(tablebasename) ) == 0 ) {
-		MesPrint("&No open tablebase with the name %s",tablebasename);
+		MesPrint("&No open tablebase with the name %s, check for existence of file or try readonly mode when opening.",tablebasename);
 		error = -1;
 		goto Endofall;
 	}
@@ -1343,10 +1411,11 @@ finishup:;
 	We need the arguments of TestUse to see for which tables it is to be done
 */
 
-WORD TestUse(WORD *term, WORD level)
+int TestUse(WORD *term, WORD level)
 {
 	WORD *tstop, *t, *m, *tstart, tabnum;
-	WORD *funs, numfuns, error = 0;
+	WORD *funs, numfuns;
+	int error = 0;
 	TABLES T;
 	LONG i;
 	CBUF *C = cbuf+AM.rbufnum;
@@ -1380,13 +1449,13 @@ WORD TestUse(WORD *term, WORD level)
 		Note: we only allow sparse tables (for now)
 */
 		m++;
-		for ( i = 0; i < T->numind; i++, m += 2 ) {
+		for ( i = 0; i < ABS(T->numind); i++, m += 2 ) {
 			if ( m >= t || *m != -SNUMBER ) break;
 		}
-		if ( ( i == T->numind ) &&
+		if ( ( i == ABS(T->numind) ) &&
 		 ( ( isp = FindTableTree(T,tstart+FUNHEAD+1,2) ) >= 0 ) ) {
-			if ( ( T->tablepointers[isp+T->numind+4] & ELEMENTLOADED ) == 0 ) {
-					T->tablepointers[isp+T->numind+4] |= ELEMENTUSED;
+			if ( ( T->tablepointers[isp+ABS(T->numind)+4] & ELEMENTLOADED ) == 0 ) {
+					T->tablepointers[isp+ABS(T->numind)+4] |= ELEMENTUSED;
 			}
 		}
 		else {
@@ -1571,7 +1640,7 @@ int CoTBuse(UBYTE *s)
 					Now translate the arguments and see whether we need
 					this one....
 */
-					for ( k = 0, w = AT.WorkPointer; k < T->numind; k++ ) {
+					for ( k = 0, w = AT.WorkPointer; k < ABS(T->numind); k++ ) {
 						ParseSignedNumber(x,p);
 						*w++ = x; p++;
 					}
@@ -1582,7 +1651,7 @@ int CoTBuse(UBYTE *s)
 						error = 1;
 						continue;
 					}
-					sum += T->numind + 4;
+					sum += ABS(T->numind) + 4;
 					mode = T->tablepointers[sum];
 					if ( ( mode & ELEMENTLOADED ) == ELEMENTLOADED ) {
 						T->tablepointers[sum] &= ~ELEMENTUSED;
@@ -1658,7 +1727,7 @@ int CoTBuse(UBYTE *s)
 					Now translate the arguments and see whether we need
 					this one....
 */
-					for ( k = 0, w = AT.WorkPointer; k < T->numind; k++ ) {
+					for ( k = 0, w = AT.WorkPointer; k < ABS(T->numind); k++ ) {
 						ParseSignedNumber(x,p);
 						*w++ = x; p++;
 					}
@@ -1669,7 +1738,7 @@ int CoTBuse(UBYTE *s)
 						error = 1;
 						continue;
 					}
-					sum += T->numind + 4;
+					sum += ABS(T->numind) + 4;
 					mode = T->tablepointers[sum];
 					if ( ( mode & ELEMENTLOADED ) == ELEMENTLOADED ) {
 						T->tablepointers[sum] &= ~ELEMENTUSED;
@@ -1828,7 +1897,7 @@ int CoTBhelp(UBYTE *s)
 	This gains one space. Hence we have to be very careful
 */
 
-VOID ReWorkT(WORD *term, WORD *funs, WORD numfuns)
+void ReWorkT(WORD *term, WORD *funs, WORD numfuns)
 {
 	WORD *tstop, *tend, *m, *t, *tt, *mm, *mmm, *r, *rr;
 	int i, j;
@@ -1909,7 +1978,7 @@ inc:		j = t[1];
   	#[ Apply :
 */
 
-WORD Apply(WORD *term, WORD level)
+void Apply(WORD *term, WORD level)
 {
 	WORD *funs, numfuns;
 	TABLES T;
@@ -1946,7 +2015,6 @@ WORD Apply(WORD *term, WORD level)
 	Note that we actually gain one space. 
 */
 	ReWorkT(term,funs,numfuns);
-	return(0);
 }
 
 /*
@@ -2067,26 +2135,26 @@ int ApplyExec(WORD *term, int maxtogo, WORD level)
 		Tpattern = T->pattern;
 #endif
 		p = Tpattern+FUNHEAD+1;
-		for ( i = 0; i < T->numind; i++, t += 2 ) {
+		for ( i = 0; i < ABS(T->numind); i++, t += 2 ) {
 			if ( *t != -SNUMBER ) break;
 		}
-		if ( i < T->numind ) { t = r; continue; }
+		if ( i < ABS(T->numind) ) { t = r; continue; }
 		isp = FindTableTree(T,t1+FUNHEAD+1,2);
 		if ( isp < 0 ) { t = r; continue; }
-		rhsnumber = T->tablepointers[isp+T->numind];
+		rhsnumber = T->tablepointers[isp+ABS(T->numind)];
 #if ( TABLEEXTENSION == 2 )
 		tbufnum = T->bufnum;
 #else
-		tbufnum = T->tablepointers[isp+T->numind+1];
+		tbufnum = T->tablepointers[isp+ABS(T->numind)+1];
 #endif
 		t = t1+FUNHEAD+2;
-		ii = T->numind;
+		ii = ABS(T->numind);
 		while ( --ii >= 0 ) {
 			*p = *t; t += 2; p += 2;
 		}
 /*
 		If there are more arguments we have to do some
-		pattern matching. This should be easy. We addapted the
+		pattern matching. This should be easy. We adapted the
 		pattern, so that the array indices match already.
 */
 #ifdef WITHPTHREADS
@@ -2185,7 +2253,7 @@ int ApplyExec(WORD *term, int maxtogo, WORD level)
   	#[ ApplyReset :
 */
 
-WORD ApplyReset(WORD level)
+void ApplyReset(WORD level)
 {
 	WORD *funs, numfuns;
 	TABLES T;
@@ -2213,7 +2281,6 @@ WORD ApplyReset(WORD level)
 			}
 		}
 	}
-	return(0);
 }
 
 /*
@@ -2221,7 +2288,7 @@ WORD ApplyReset(WORD level)
   	#[ TableReset :
 */
 
-WORD TableReset()
+void TableReset(void)
 {
 	TABLES T;
 	int i;
@@ -2231,7 +2298,6 @@ WORD TableReset()
 			functions[i].tabl = T->spare;
 		}
 	}
-	return(0);
 }
 
 /*
@@ -2248,7 +2314,7 @@ int LoadTableElement(DBASE *d, TABLE *T, WORD num)
 	Releases all TableBases
 */
 
-int ReleaseTB()
+int ReleaseTB(void)
 {
 	DBASE *d;
 	int i;
