@@ -7,7 +7,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -144,13 +144,14 @@ dovariable:		c = *in; *in = 0;
 										}
 /*[06nov2003 mt]:*/
 #ifdef WITHMPI
-										else/*RHSide*/
+										else { /*RHSide*/
 											/* NOTE: We always set AC.RhsExprInModuleFlag regardless of
 											 *       AP.PreAssignFlag or AP.PreInsideLevel because we have to detect
 											 *       RHS expressions even in those cases. */
 											AC.RhsExprInModuleFlag = 1;
-											if ( !AP.PreAssignFlag && !AP.PreInsideLevel )
-												Expressions[number].vflags |= ISINRHS;
+										}
+										if ( !AP.PreAssignFlag && !AP.PreInsideLevel )
+											Expressions[number].vflags |= ISINRHS;
 #endif
 /*:[06nov2003 mt]*/
 										if ( AC.exprfillwarning == 0 ) {
@@ -173,6 +174,11 @@ donumber:		i = 0;
 				*in = c;
 				break;
 			case 1:		/* 0-9 */
+			  {
+#ifdef WITHFLOAT
+				int spec;
+				UBYTE *in2;
+#endif
 				CHECKPOLY
 				s = in;
 				while ( *s == '0' && FG.cTable[s[1]] == 1 ) s++;
@@ -190,6 +196,31 @@ donumber:		i = 0;
 					object = 1;
 					break;
 				}
+#ifdef WITHFLOAT
+				in2 = CheckFloat(in,&spec);
+				if ( in2 > in ) {
+					if ( spec == -1 ) {
+						MesPrint("&The floating point system has not been started: %s",in);
+	                    if ( !error ) error = 1;
+					}
+					else {
+					  in = in2;
+dofloat:
+					  while ( out + (in-s) >= AC.toptokens ) {
+						LONG oldsize = (LONG)(out - AC.tokens);
+						SBYTE **ppp = &(AC.tokens); /* to avoid a compiler warning */
+						SBYTE **pppp = &(AC.toptokens);
+						DoubleBuffer((void **)ppp,(void **)pppp,sizeof(SBYTE),"more tokens");
+						out = AC.tokens + oldsize;
+						outtop = AC.toptokens - MAXNUMSIZE;
+					  }
+					  *out++ = TFLOAT;
+					  while ( s < in ) *out++ = *s++;
+					}
+				}
+				else
+#endif
+				{
 				*out++ = TNUMBER;
 				if ( ( i & 1 ) != 0 ) *out++ = (SBYTE)(*s++ - '0');
 				while ( out + (in-s)/2 >= AC.toptokens ) {
@@ -204,8 +235,10 @@ donumber:		i = 0;
 					*out++ = (SBYTE)(( *s - '0' ) * 10 + ( s[1] - '0' ));
 					s += 2;
 				}
+				}
 				object = 2;
-				break;
+			  }
+			  break;
 			case 2:		/* . $ _ ? # ' */
 				CHECKPOLY
 				if ( *in == '?' ) {
@@ -268,6 +301,27 @@ donumber:		i = 0;
 						object = 0;
 						in++;
 					}
+#ifdef WITHFLOAT
+					else if ( object == 0 || object == -1 ) {
+/*
+						Test for floating point number
+*/
+						int spec;
+						s = CheckFloat(in,&spec);
+						if ( s > in ) {
+							if ( spec == -1 ) {
+								MesPrint("&The floating point system has not been started: %s",in);
+								if ( !error ) error = 1;
+								in++;
+							}
+							else {
+								UBYTE *a = s; s = in; in = a;
+								goto dofloat;
+							}
+						}
+						else goto IllPos;
+					}
+#endif
 					else goto IllPos;
 				}
 				else if ( *in == '$' ) {	/* $ variable */
@@ -307,9 +361,14 @@ donumber:		i = 0;
 					}
 				}
 				else if ( *in == '#' ) {
+					in++;
 					if ( object == 1 ) {	/* follows a name */
 						*out++ = TCONJUGATE;
 					}
+					else {
+						MesPrint("&Illegal position for %#");
+						error = 1;
+					}
 				}
 				else goto IllPos;
 				break;
@@ -524,7 +583,7 @@ IllPos:			MesPrint("&Illegal character at this position: %s",in);
 */
 	if ( error == 0 && simp1token(AC.tokens) ) error = 1;
 /*
-	Collect wildcards for the prototype. Symplify the leftover wildcards
+	Collect wildcards for the prototype. Simplify the leftover wildcards
 */
 	if ( error == 0 && leftright == LHSIDE && simpwtoken(AC.tokens) )
 				error = 1;
@@ -548,10 +607,11 @@ IllPos:			MesPrint("&Illegal character at this position: %s",in);
 	if ( error == 0 && ( numexp = simp3atoken(AC.tokens,leftright) ) < 0 )
 		error = 1;
 	if ( numexp > 0 ) {
+		// Make some space at the beginning of AC.tokens, filled with TEMPTY.
 		SBYTE *tt;
 		out = AC.tokens;
 		while ( *out != TENDOFIT ) out++;
-		while ( out+numexp*9+20 > outtop ) {
+		while ( out+numexp*9 > outtop ) {
 			LONG oldsize = (LONG)(out - AC.tokens);
 			SBYTE **ppp = &(AC.tokens); /* to avoid a compiler warning */
 			SBYTE **pppp = &(AC.toptokens);
@@ -559,7 +619,7 @@ IllPos:			MesPrint("&Illegal character at this position: %s",in);
 			out = AC.tokens + oldsize;
 			outtop = AC.toptokens - MAXNUMSIZE;
 		}
-		tt = out + numexp*9+20;
+		tt = out + numexp*9;
 		while ( out >= AC.tokens ) { *tt-- = *out--; }
 		while ( tt >= AC.tokens ) { *tt-- = TEMPTY; }
 		if ( error == 0 && simp3btoken(AC.tokens,leftright) ) error = 1;
@@ -1233,6 +1293,12 @@ dofunction:
 				}
 				if ( denom < 0 || *t == TPOWER || *t == TPOWER1 ) numexp++;
 				break;
+#ifdef WITHFLOAT
+			case TFLOAT:
+				s++; while ( *s >= 0 ) s++;
+				if ( denom < 0 || *s == TPOWER || *s == TPOWER1 ) numexp++;
+				break;
+#endif
 			case TEXPRESSION:
 				s++; while ( *s >= 0 ) s++;
 				t = s;
@@ -1293,6 +1359,18 @@ dofunction:
 					case CVECTOR: goto dovector;
 					case CFUNCTION: goto dofunction;
 					case CNUMBER: goto dosymbol;
+					case CMODEL: 
+						if ( denom < 0 || *s == TPOWER ) {
+							MesPrint("&A model to a power or in denominator is illegal");
+							error = 1;
+						}
+						break;
+					case ANYTYPE: 
+						if ( denom < 0 || *s == TPOWER ) {
+							MesPrint("&A set without type to a power or in denominator is illegal");
+							error = 1;
+						}
+						break;
 					default: error = 1; break;
 				}
 				break;
@@ -1357,15 +1435,33 @@ int simp3btoken(SBYTE *s, int mode)
 	SBYTE *t, c, *fill, *ff, *ss;
 	LONG num;
 	WORD *prot;
+
+	// Work in a temporary buffer, to avoid clashes between input and output
+	SBYTE *tmptokens = (SBYTE*)Malloc1((AC.toptokens-AC.tokens)*sizeof(SBYTE), "simp3btoken scratch");
+	SBYTE* tmptoptokens = tmptokens + (AC.toptokens-AC.tokens);
+	fill = tmptokens;
+
 	if ( mode == RHSIDE ) {
 		prot = AC.ProtoType;
 		numprot = prot[1] - SUBEXPSIZE;
 		prot += SUBEXPSIZE;
 	}
 	else { prot = 0; numprot = 0; }
-	fill = s;
 	while ( *s == TEMPTY ) s++;
 	while ( *s != TENDOFIT ) {
+		// If we are near the end of the output buffer, we'd better reallocate
+		// both tmptokens and AC.tokens (which must fit the final result).
+		// Experimentally, fill can move by at most 5 per loop iteration, but
+		// this buffer of 20 of might not be enough for all cases!
+		while ( tmptoptokens - fill < 20 ) {
+			LONG tmppos;
+			tmppos = s - AC.tokens;
+			DoubleBuffer((void**)&(AC.tokens), (void**)&(AC.toptokens), sizeof(SBYTE), "simp3btoken double");
+			s = AC.tokens + tmppos;
+			tmppos = fill - tmptokens;
+			DoubleBuffer((void**)&tmptokens, (void**)&tmptoptokens, sizeof(SBYTE), "simp3btoken scratch double");
+			fill = tmptokens + tmppos;
+		}
 		if ( *s == TEMPTY ) { s++; continue; }
 		denom = 1;
 		if ( *s == TDIVIDE ) { denom = -1; *fill++ = *s++; }
@@ -1463,6 +1559,13 @@ dovector:
 				else if ( *s == TDOT ) goto dodot;
 				t = s;
 				goto dofunpower;
+#ifdef WITHFLOAT
+			case TFLOAT:
+				*fill++ = *s++; while ( *s >= 0 ) *fill++ = *s++;
+				t = s;
+				sube = 0;
+				goto dofunpower;
+#endif
 			case TFUNCTION:
 				*fill++ = *s++; while ( *s >= 0 ) *fill++ = *s++;
 				if ( *s == TWILDCARD ) *fill++ = *s++;
@@ -1665,6 +1768,8 @@ dofunpower:
 					case CVECTOR: goto dovector;
 					case CFUNCTION: goto dofunction;
 					case CNUMBER: goto dosymbol;
+					case CMODEL: break;
+					case ANYTYPE: break;
 					default: error = 1; break;
 				}
 				break;
@@ -1714,9 +1819,17 @@ nodot:				MesPrint("&Illegal second element in dotproduct");
 		}
 	}
 	*fill = TENDOFIT;
+	// Now copy the modified tokens back to the original buffer
+	fill = tmptokens;
+	s = AC.tokens;
+	do {
+		*s++ = *fill;
+	} while ( *fill++ != TENDOFIT );
+	M_free(tmptokens, "simp3btoken scratch");
 	return(error);
 doublepower:;
 	MesPrint("&Dubious notation with power of power");
+	M_free(tmptokens, "simp3btoken scratch");
 	return(-1);
 }
 
