@@ -4,7 +4,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -88,12 +88,12 @@ int CoTransform(UBYTE *in)
 {
 	GETIDENTITY
 	UBYTE *s = in, c, *ss, *Tempbuf;
-	WORD number, type, num, i, *work = AT.WorkPointer+2, *wp, range[2], one = 1;
+	WORD number, type, i, *work = AT.WorkPointer+2, *wp, range[2], one = 1;
 	WORD numdol, *wstart;
 	int error = 0, irhs;
 	LONG x;
 	while ( *in == ',' ) in++;
-	num = 0; wp = work + 1;
+	wp = work + 1;
 /*
   	#[ Sets :
 
@@ -118,7 +118,14 @@ int CoTransform(UBYTE *in)
 			if ( *in != ',' ) break;
 			c = *in; *in = 0;
 		    type = GetName(AC.varnames,s,&number,NOAUTO);
-			if ( type == CFUNCTION ) { number += MAXVARIABLES + FUNCTION; }
+			if ( type == CFUNCTION ) { 
+#ifdef WITHFLOAT
+				if ( (number+FUNCTION) == FLOATFUN ) {
+					MesPrint("&Illegal use of a transform statement and float_");
+					if ( error == 0 ) error = 1;
+				}
+#endif
+				number += MAXVARIABLES + FUNCTION; }
 			else if ( type != CSET ) {
 				MesPrint("& %s: A transform statement starts with sets of functions",s);
 				if ( error == 0 ) error = 1;
@@ -139,6 +146,17 @@ int CoTransform(UBYTE *in)
 				MesPrint("&A set in a transform statement should be a set of functions");
 				if ( error == 0 ) error = 1;
 			}
+#ifdef WITHFLOAT
+			WORD *r1, *r2;
+			r1 = SetElements + Sets[number].first;
+			r2 = SetElements + Sets[number].last;
+			while ( r1 < r2 ) {
+				if ( *r1++ == FLOATFUN ) {
+					MesPrint("&Illegal use of a transform statement and float_");
+					if ( error == 0 ) error = 1;
+				}
+			}
+#endif
 		  }
 		}
 		else if ( error == 0 ) error = 1;
@@ -146,7 +164,6 @@ int CoTransform(UBYTE *in)
 		Now write the number to the right place
 */
 		*wp++ = number;
-		num++;
 		while ( *in == ',' ) in++;
 	}
 	*work = wp - work;
@@ -164,7 +181,7 @@ int CoTransform(UBYTE *in)
 			return(error);
 		}
 		in = SkipAName(in);
-		if ( *in == '>' || *in == '<' ) in++;
+		if ( *in == '>' || *in == '<' || *in == '+' || *in == '-' ) in++;
 		ss = in;
 		c = *ss; *ss = 0;
 		if ( c != '(' ) {
@@ -549,7 +566,7 @@ illsize:					MesPrint("&Illegal value for base in encode/decode transformation")
 			type = ISLYNDON;
 			goto doreplace;
 		}
-		else if ( StrICmp(s,(UBYTE *)"islyndon+" ) == 0 ) {
+		else if ( StrICmp(s,(UBYTE *)"islyndon-" ) == 0 ) {
 			type = ISLYNDON;
 			goto doreplace;
 		}
@@ -557,7 +574,7 @@ illsize:					MesPrint("&Illegal value for base in encode/decode transformation")
 			type = ISLYNDONR;
 			goto doreplace;
 		}
-		else if ( StrICmp(s,(UBYTE *)"islyndon-" ) == 0 ) {
+		else if ( StrICmp(s,(UBYTE *)"islyndon+" ) == 0 ) {
 			type = ISLYNDONR;
 			goto doreplace;
 		}
@@ -569,7 +586,7 @@ illsize:					MesPrint("&Illegal value for base in encode/decode transformation")
 			type = TOLYNDON;
 			goto doreplace;
 		}
-		else if ( StrICmp(s,(UBYTE *)"tolyndon+" ) == 0 ) {
+		else if ( StrICmp(s,(UBYTE *)"tolyndon-" ) == 0 ) {
 			type = TOLYNDON;
 			goto doreplace;
 		}
@@ -577,7 +594,7 @@ illsize:					MesPrint("&Illegal value for base in encode/decode transformation")
 			type = TOLYNDONR;
 			goto doreplace;
 		}
-		else if ( StrICmp(s,(UBYTE *)"tolyndon-" ) == 0 ) {
+		else if ( StrICmp(s,(UBYTE *)"tolyndon+" ) == 0 ) {
 			type = TOLYNDONR;
 			goto doreplace;
 		}
@@ -660,6 +677,50 @@ illsize:					MesPrint("&Illegal value for base in encode/decode transformation")
 		}
 /*
  		#] selectarg : 
+ 		#[ ZtoH :
+*/
+		else if ( StrICmp(s,(UBYTE *)"ztoh") == 0 ) {
+/*
+				Subkeys: ?
+*/
+			type = ZTOHARG;
+			*ss = c;
+			if ( ( in = ReadRange(in,range,1) ) == 0 ) {
+				if ( error == 0 ) error = 1;
+				return(error);
+			}
+			*wp++ = ARGRANGE;
+			*wp++ = range[0];
+			*wp++ = range[1];
+			*wp++ = type;
+			*work = wp-work;
+			work = wp; *wp++ = 0;
+			s = in;
+		}
+/*
+ 		#] ZtoH : 
+ 		#[ HtoZ :
+*/
+		else if ( StrICmp(s,(UBYTE *)"htoz") == 0 ) {
+/*
+				Subkeys: ?
+*/
+			type = HTOZARG;
+			*ss = c;
+			if ( ( in = ReadRange(in,range,1) ) == 0 ) {
+				if ( error == 0 ) error = 1;
+				return(error);
+			}
+			*wp++ = ARGRANGE;
+			*wp++ = range[0];
+			*wp++ = range[1];
+			*wp++ = type;
+			*work = wp-work;
+			work = wp; *wp++ = 0;
+			s = in;
+		}
+/*
+ 		#] HtoZ : 
 */
 		else {
 			MesPrint("&Unknown transformation inside a Transform statement: %s",s);
@@ -686,11 +747,11 @@ illsize:					MesPrint("&Illegal value for base in encode/decode transformation")
 
 */
 
-WORD RunTransform(PHEAD WORD *term, WORD *params)
+int RunTransform(PHEAD WORD *term, WORD *params)
 {
 	WORD *t, *tstop, *w, *m, *out, *in, *tt, retval;
 	WORD *fun, *args, *info, *infoend, *onetransform, *funs, *endfun;
-	WORD *thearg = 0, *iterm, *newterm, *nt, *oldwork = AT.WorkPointer;
+	WORD *thearg = 0, *iterm, *newterm, *nt, *oldwork = AT.WorkPointer, sign = 1;
 	int i;
 	out = tstop = term + *term;
 	tstop -= ABS(tstop[-1]);
@@ -702,6 +763,9 @@ WORD RunTransform(PHEAD WORD *term, WORD *params)
 		if ( *t < FUNCTION ) {}
 		else if ( funs == endfun ) {  /* we do all functions */
 hit:;
+#ifdef WITHFLOAT
+			if ( *t == FLOATFUN ) goto next;
+#endif
 			while ( in < t ) *out++ = *in++;
 			tt = t + t[1]; fun = out;
 			while ( in < tt ) *out++ = *in++;
@@ -797,7 +861,7 @@ returnvalues:;
 							return(-1);
 						}
 						newterm = AT.WorkPointer;
-						if ( EndSort(BHEAD newterm,0) < 0 ) {}
+						if ( EndSort(BHEAD newterm,1) < 0 ) {}
 						if ( ( *newterm && *(newterm+*newterm) != 0 ) || *newterm == 0 ) {
 							MLOCK(ErrorMessageLock);
 							MesPrint("&yes/no information in islyndon/tolyndon does not evaluate into a single term");
@@ -866,6 +930,22 @@ abortlyndon:;
 						if ( RunSelectArg(BHEAD fun,args) ) goto abo;
 						out = fun + fun[1];
 						break;
+					case ZTOHARG:
+						{
+						  WORD s = RunZtoHArg(BHEAD fun,args);
+						  if ( s < 0 ) goto abo;
+						  if ( s == 1 ) sign = -sign;
+						  out = fun + fun[1];
+						}
+						break;
+					case HTOZARG:
+						{
+						  WORD s = RunHtoZArg(BHEAD fun,args);
+						  if ( s < 0 ) goto abo;
+						  if ( s == 1 ) sign = -sign;
+						  out = fun + fun[1];
+						}
+						break;
 					default:
 						MLOCK(ErrorMessageLock);
 						MesPrint("Irregular code in execution of transform statement");
@@ -891,9 +971,13 @@ abortlyndon:;
 			funs++;
 		  }
 		}
+#ifdef WITHFLOAT
+next:
+#endif
 		t += t[1];
 	}
 	tt = term + *term; while ( in < tt ) *out++ = *in++;
+	if ( sign == -1 ) out[-1] = -out[-1];
 	*tt = i = out - tt;
 /*
 	Now copy the whole thing back
@@ -919,7 +1003,7 @@ abo:
 		first or work with an array of pointers.
 */
 
-WORD RunEncode(PHEAD WORD *fun, WORD *args, WORD *info)
+int RunEncode(PHEAD WORD *fun, WORD *args, WORD *info)
 {
 	WORD base, *f, *funstop, *fun1, *t, size1, size2, size3, *arg;
 	int num, num1, num2, n, i, i1, i2;
@@ -1106,7 +1190,7 @@ CalledFrom:
  		#[ RunDecode :
 */
 
-WORD RunDecode(PHEAD WORD *fun, WORD *args, WORD *info)
+int RunDecode(PHEAD WORD *fun, WORD *args, WORD *info)
 {
 	WORD base, num, num1, num2, n, *f, *funstop, *fun1, size1, size2, size3, *t;
 	WORD i1, i2, i, sig;
@@ -1276,7 +1360,7 @@ CalledFrom:
 		The output is at first written after fun and in the end overwrites fun.
 */
 
-WORD RunReplace(PHEAD WORD *fun, WORD *args, WORD *info)
+int RunReplace(PHEAD WORD *fun, WORD *args, WORD *info)
 {
 	int n = 0, i, dirty = 0, totarg, nfix, nwild, ngeneral;
 	WORD *t, *tt, *u, *tstop, *info1, *infoend, *oldwork = AT.WorkPointer;
@@ -1319,7 +1403,7 @@ WORD RunReplace(PHEAD WORD *fun, WORD *args, WORD *info)
 		return(-1);
 	}
 	newterm = AT.WorkPointer;
-	if ( EndSort(BHEAD newterm,0) < 0 ) {}
+	if ( EndSort(BHEAD newterm,1) < 0 ) {}
 	if ( ( *newterm && *(newterm+*newterm) != 0 ) || *newterm == 0 ) {
 		MLOCK(ErrorMessageLock);
 		MesPrint("&information in replace transformation does not evaluate into a single term");
@@ -1581,7 +1665,7 @@ getthisone:;
 							return(-1);
 						}
 						term4 = AT.WorkPointer;
-						if ( EndSort(BHEAD term4,0) < 0 ) {}
+						if ( EndSort(BHEAD term4,1) < 0 ) {}
 						if ( ( *term4 && *(term4+*term4) != 0 ) || *term4 == 0 ) {
 							MLOCK(ErrorMessageLock);
 							MesPrint("&information in replace transformation does not evaluate into a single term");
@@ -1708,7 +1792,7 @@ dothisnow:;
 				return(-1);
 			}
 			term4 = AT.WorkPointer;
-			if ( EndSort(BHEAD term4,0) < 0 ) {}
+			if ( EndSort(BHEAD term4,1) < 0 ) {}
 			if ( ( *term4 && *(term4+*term4) != 0 ) || *term4 == 0 ) {
 				MLOCK(ErrorMessageLock);
 				MesPrint("&information in replace transformation does not evaluate into a single term");
@@ -1754,10 +1838,10 @@ nextt:;
 		Note that we restrict ourselves to short integers and/or single symbols
 */
 
-WORD RunImplode(WORD *fun, WORD *args)
+int RunImplode(WORD *fun, WORD *args)
 {
 	GETIDENTITY
-	WORD *tt, *tstop, totarg, arg1, arg2, num1, num2, i, i1, n;
+	WORD *tt, *tstop, totarg, arg1, arg2, num1, num2, i1, n;
 	WORD *f, *t, *ttt, *t4, *ff, *fff;
 	WORD moveup, numzero, outspace;
 	if ( functions[fun[0]-FUNCTION].spec != 0 ) return(0);
@@ -1798,7 +1882,7 @@ WORD RunImplode(WORD *fun, WORD *args)
 	We do it this way because otherwise stepping backwards (as in range=(4,1))
 	would be very difficult.
 */
-	tt = tstop; i = 0;
+	tt = tstop;
 	while ( n <= num2 ) {
 		if ( f >= tstop ) return(0);
 		if ( *f == -SNUMBER ) { *tt++ = -1; *tt++ = 0;
@@ -1832,7 +1916,7 @@ WORD RunImplode(WORD *fun, WORD *args)
 			}
 			f += *f;
 		}
-		i++; n++;
+		n++;
 	}
 	fff = f;
 /*
@@ -1955,7 +2039,7 @@ moveinto:
  		#[ RunExplode :
 */
 
-WORD RunExplode(PHEAD WORD *fun, WORD *args)
+int RunExplode(PHEAD WORD *fun, WORD *args)
 {
 	WORD arg1, arg2, num1, num2, *tt, *tstop, totarg, *tonew, *newfun;
 	WORD *ff, *f;
@@ -2069,7 +2153,7 @@ OverWork:;
  		#[ RunPermute :
 */
 
-WORD RunPermute(PHEAD WORD *fun, WORD *args, WORD *info)
+int RunPermute(PHEAD WORD *fun, WORD *args, WORD *info)
 {
 	WORD *tt, totarg, *tstop, arg1, arg2, n, num, i, *f, *f1, *f2, *infostop;
 	WORD *in, *iw, withdollar;
@@ -2296,7 +2380,7 @@ OverWork:;
  		#[ RunReverse :
 */
 
-WORD RunReverse(PHEAD WORD *fun, WORD *args)
+int RunReverse(PHEAD WORD *fun, WORD *args)
 {
 	WORD *tt, totarg, *tstop, arg1, arg2, n, num, i, *f, *f1, *f2, i1, i2;
 	if ( *args != ARGRANGE ) {
@@ -2383,7 +2467,7 @@ OverWork:;
  		#[ RunDedup :
 */
 
-WORD RunDedup(PHEAD WORD *fun, WORD *args)
+int RunDedup(PHEAD WORD *fun, WORD *args)
 {
 	WORD *tt, totarg, *tstop, arg1, arg2, n, i, j,k, *f, *f1, *f2, *fd, *fstart;
 	if ( *args != ARGRANGE ) {
@@ -2472,7 +2556,7 @@ WORD RunDedup(PHEAD WORD *fun, WORD *args)
  		#[ RunCycle :
 */
 
-WORD RunCycle(PHEAD WORD *fun, WORD *args, WORD *info)
+int RunCycle(PHEAD WORD *fun, WORD *args, WORD *info)
 {
 	WORD *tt, totarg, *tstop, arg1, arg2, n, num, i, j, *f, *f1, *f2, x, ncyc, cc;
 	if ( *args != ARGRANGE ) {
@@ -2624,7 +2708,7 @@ OverWork:;
  		#[ RunAddArg :
 */
 
-WORD RunAddArg(PHEAD WORD *fun, WORD *args)
+int RunAddArg(PHEAD WORD *fun, WORD *args)
 {
 	WORD *tt, totarg, *tstop, arg1, arg2, n, num, *f, *f1, *f2;
 	WORD scribble[10+ARGHEAD];
@@ -2643,6 +2727,8 @@ WORD RunAddArg(PHEAD WORD *fun, WORD *args)
 	}
 	tt = fun+FUNHEAD; tstop = fun+fun[1]; totarg = 0;
 	while ( tt < tstop ) { totarg++; NEXTARG(tt); }
+	/* ignore functions with no arguments */
+	if ( totarg == 0 ) return(0);
 	if ( FindRange(BHEAD args,&arg1,&arg2,totarg) ) return(-1);
 /*
 	We need to:
@@ -2677,7 +2763,7 @@ WORD RunAddArg(PHEAD WORD *fun, WORD *args)
 		}
 		n++;
 	}
-	if ( EndSort(BHEAD tstop+ARGHEAD,0) ) return(-1);
+	if ( EndSort(BHEAD tstop+ARGHEAD,1) < 0 ) return(-1);
 	num = 0;
 	f2 = tstop+ARGHEAD;
 	while ( *f2 ) { f2 += *f2; num++; }
@@ -2711,7 +2797,7 @@ WORD RunAddArg(PHEAD WORD *fun, WORD *args)
  		#[ RunMulArg :
 */
 
-WORD RunMulArg(PHEAD WORD *fun, WORD *args)
+int RunMulArg(PHEAD WORD *fun, WORD *args)
 {
 	WORD *t, totarg, *tstop, arg1, arg2, n, *f, nb, *m, i, *w;
 	WORD *scratch, argbuf[20], argsize, *where, *newterm;
@@ -2731,6 +2817,8 @@ WORD RunMulArg(PHEAD WORD *fun, WORD *args)
 	}
 	t = fun+FUNHEAD; tstop = fun+fun[1]; totarg = 0;
 	while ( t < tstop ) { totarg++; NEXTARG(t); }
+	/* ignore functions with no arguments */
+	if ( totarg == 0 ) return(0);
 	if ( FindRange(BHEAD args,&arg1,&arg2,totarg) ) return(-1);
 	if ( arg2 < arg1 ) { n = arg1; arg1 = arg2; arg2 = n; }
 	if ( arg1 > totarg ) return(0);
@@ -2823,7 +2911,7 @@ WORD RunMulArg(PHEAD WORD *fun, WORD *args)
 	NewSort(BHEAD0);
 	Generator(BHEAD scratch,AR.Cnumlhs);
 	newterm = AT.WorkPointer;
-	EndSort(BHEAD newterm+ARGHEAD,0);
+	EndSort(BHEAD newterm+ARGHEAD,1);
 	C->Pointer = C->Buffer+oldcpointer_pos;
 	C->numrhs = nb;
 	w = newterm+ARGHEAD; while ( *w ) w += *w;
@@ -2850,7 +2938,7 @@ WORD RunMulArg(PHEAD WORD *fun, WORD *args)
 		the numbers of the arguments in the range.
 */
 
-WORD RunIsLyndon(PHEAD WORD *fun, WORD *args, int par)
+int RunIsLyndon(PHEAD WORD *fun, WORD *args, int par)
 {
 	WORD *tt, totarg, *tstop, arg1, arg2, arg, num, *f, n, i;
 /*	WORD *f1; */
@@ -3040,7 +3128,7 @@ OverWork:;
  		#[ RunDropArg :
 */
 
-WORD RunDropArg(PHEAD WORD *fun, WORD *args)
+int RunDropArg(PHEAD WORD *fun, WORD *args)
 {
 	WORD *t, *tstop, *f, totarg, arg1, arg2, n;
 
@@ -3066,7 +3154,7 @@ WORD RunDropArg(PHEAD WORD *fun, WORD *args)
  		#[ RunSelectArg :
 */
 
-WORD RunSelectArg(PHEAD WORD *fun, WORD *args)
+int RunSelectArg(PHEAD WORD *fun, WORD *args)
 {
 	WORD *t, *tstop, *f, *tt, totarg, arg1, arg2, n;
 
@@ -3091,6 +3179,118 @@ WORD RunSelectArg(PHEAD WORD *fun, WORD *args)
 
 /*
  		#] RunSelectArg : 
+ 		#[ RunZtoHArg :
+*/
+
+int RunZtoHArg(PHEAD WORD *fun, WORD *args)
+{
+	WORD *tt, totarg, *tstop, arg1, arg2, n, i, *f, *f1;
+	int sign = 0;
+	WORD *t, *t1, *t2, *t3;
+	if ( *args != ARGRANGE ) {
+		MLOCK(ErrorMessageLock);
+		MesPrint("Illegal range encountered in RunZtoHArg.");
+		MUNLOCK(ErrorMessageLock);
+		Terminate(-1);
+	}
+	if ( functions[fun[0]-FUNCTION].spec != 0 ) {
+		MLOCK(ErrorMessageLock);
+		MesPrint("The ZtoH transformation can only be executed on regular functions with nonzero integer arguments.");
+		MUNLOCK(ErrorMessageLock);
+		Terminate(-1);
+	}
+	tt = fun+FUNHEAD; tstop = fun+fun[1]; totarg = 0;
+	while ( tt < tstop ) { totarg++; NEXTARG(tt); }
+	if ( FindRange(BHEAD args,&arg1,&arg2,totarg) ) return(-1);
+/*
+	Check the arguments. Should be -SNUMBER x!=0
+*/
+	f = fun+FUNHEAD; n = 1;
+	while ( n < arg1 ) { n++; NEXTARG(f) }
+	f1 = f;
+	for ( i = arg1; i <= arg2; i++, f += 2 ) {
+		if ( *f != -SNUMBER || f[1] == 0 ) return(-1);
+	}
+/*
+	Now we need a copy. 
+*/
+	t = f1; t1 = t2 = tt = TermMalloc("RunZtoHArg");
+	while ( t < f ) { *t1++ = *t++; *t1++ = *t++; }
+	t = f1; 
+	while ( t2 < t1 ) {
+		t += 2;
+		if ( t2[1] < 0 ) {
+			t3 = t;
+			while ( t3 < f ) { t3[1] = -t3[1]; t3 += 2; }
+		}
+		t2 += 2;
+	}
+	TermFree(tt,"RunZtoHArg");
+/*
+	Now the overall sign.
+*/
+	while ( f1 < f ) { if ( f1[1] < 0 ) sign = 1-sign; f1 += 2; }
+	return(sign);
+}
+
+/*
+ 		#] RunZtoHArg : 
+ 		#[ RunHtoZArg :
+*/
+
+int RunHtoZArg(PHEAD WORD *fun, WORD *args)
+{
+	WORD *tt, totarg, *tstop, arg1, arg2, n, i, *f, *f1, *f2;
+	int sign = 0;
+	WORD *t, *t1, *t2;
+	if ( *args != ARGRANGE ) {
+		MLOCK(ErrorMessageLock);
+		MesPrint("Illegal range encountered in RunZtoHArg.");
+		MUNLOCK(ErrorMessageLock);
+		Terminate(-1);
+	}
+	if ( functions[fun[0]-FUNCTION].spec != 0 ) {
+		MLOCK(ErrorMessageLock);
+		MesPrint("The HtoZ transformation can only be executed on regular functions with nonzero integer arguments.");
+		MUNLOCK(ErrorMessageLock);
+		Terminate(-1);
+	}
+	tt = fun+FUNHEAD; tstop = fun+fun[1]; totarg = 0;
+	while ( tt < tstop ) { totarg++; NEXTARG(tt); }
+	if ( FindRange(BHEAD args,&arg1,&arg2,totarg) ) return(-1);
+/*
+	Check the arguments. Should be -SNUMBER x!=0
+*/
+	f = fun+FUNHEAD; n = 1;
+	while ( n < arg1 ) { n++; NEXTARG(f) }
+	f2 = f1 = f;
+	for ( i = arg1; i <= arg2; i++, f += 2 ) {
+		if ( *f != -SNUMBER || f[1] == 0 ) return(-1);
+	}
+/*
+	First the overall sign.
+*/
+	while ( f2 < f ) { if ( f2[1] < 0 ) sign = 1-sign; f2 += 2; }
+/*
+	Now we need a copy. 
+*/
+	t = f1; t1 = tt = TermMalloc("RunHtoZArg");
+	while ( t < f ) { *t1++ = *t++; *t1++ = *t++; }
+/*
+	Now the transformation.
+*/
+	t = f1; t2 = tt + 2;
+	while ( t2 < t1 ) {
+		t += 2;
+		if ( t2[-1] < 0 ) t[1] = -t[1];
+		t2 += 2;
+	}
+	TermFree(tt,"RunHtoZArg");
+	return(sign);
+}
+
+/*
+ 		#] RunHtoZArg : 
  		#[ TestArgNum :
 
 		Looks whether argument n is contained in any of the ranges
@@ -3382,20 +3582,25 @@ int FindRange(PHEAD WORD *args, WORD *arg1, WORD *arg2, WORD totarg)
 				n[i] -= MAXPOSITIVE4; /* Now we have the number of the dollar variable   */
 			}
 			n[i] = DolToNumber(BHEAD n[i]);
-			if ( AN.ErrorInDollar ) goto Error;
+			if ( AN.ErrorInDollar ) {
+				MLOCK(ErrorMessageLock);
+				MesPrint("Illegal $ value in range while executing transform statement.");
+				MUNLOCK(ErrorMessageLock);
+				return(-1);
+			}
 			if ( fromlast ) n[i] = totarg-n[i];
 		}
 		else if ( n[i] >= MAXPOSITIVE4 ) { n[i] = totarg-(n[i]-MAXPOSITIVE4); }
-		if ( n[i] <= 0 ) goto Error;
+		if ( n[i] <= 0 ) {
+			MLOCK(ErrorMessageLock);
+			MesPrint("Illegal non-positive value in range (%d) while executing transform statement.", i+1);
+			MUNLOCK(ErrorMessageLock);
+			return(-1);
+		}
 	}
 	*arg1 = n[0];
 	*arg2 = n[1];
 	return(0);
-Error:
-	MLOCK(ErrorMessageLock);
-	MesPrint("Illegal $ value in range while executing transform statement.");
-	MUNLOCK(ErrorMessageLock);
-	return(-1);
 }
 
 /*
