@@ -11,7 +11,7 @@
  */
 /* #[ License : */
 /*
- *   Copyright (C) 1984-2023 J.A.M. Vermaseren
+ *   Copyright (C) 1984-2026 J.A.M. Vermaseren
  *   When using this file you are requested to refer to the publication
  *   J.A.M.Vermaseren "New features of FORM" math-ph/0010025
  *   This is considered a matter of courtesy as the development was paid
@@ -306,6 +306,7 @@ Seven:;
 				while ( t < u ) {
 					*m = *t;
 					subcount = 0;
+					/* Process the first vector of the DOTPRODUCT */
 					for ( si = 0; si < setflag; si += 2 ) {
 						if ( t == temp + setlist[si] ) goto ss2;
 					}
@@ -316,7 +317,17 @@ Seven:;
 								*m = s[3]; dirty = 1; break;
 							}
 							if ( *s == VECTOMIN ) {
-								*m = s[3]; dirty = 1; sgn += t[2]; break;
+								*m = s[3];
+								dirty = 1;
+								if ( ( ABS(t[2]) - 2*MAXPOWER ) < 0 ) {
+									/* The power is a number */
+									sgn += t[2];
+								}
+								else {
+									/* The power is a wildcard. Put a -, resolve later. */
+									sgn++;
+								}
+								break;
 							}
 							if ( *s == VECTOSUB ) {
 								*m = s[3]; dirty = 1; subcount = 1; break;
@@ -327,6 +338,7 @@ Seven:;
 ss2:
 					*++m = *++t;
 					s = subs;
+					/* Process the second vector of the DOTPRODUCT */
 					for ( si = 0; si < setflag; si += 2 ) {
 						if ( t == temp + setlist[si] ) goto ss3;
 					}
@@ -336,7 +348,17 @@ ss2:
 								*m = s[3]; dirty = 1; break;
 							}
 							if ( *s == VECTOMIN ) {
-								*m = s[3]; dirty = 1; sgn += t[1]; break;
+								*m = s[3];
+								dirty = 1;
+								if ( ( ABS(t[1]) - 2*MAXPOWER ) < 0 ) {
+									/* The power is a number */
+									sgn += t[1];
+								}
+								else {
+									/* The power is a wildcard. Put a -, resolve later. */
+									sgn++;
+								}
+								break;
 							}
 							if ( *s == VECTOSUB ) {
 								*m = s[3]; dirty = 1; subcount += 2; break;
@@ -345,12 +367,18 @@ ss2:
 						s += s[1];
 					}
 ss3:				*++m = *++t;
+					/* Process the power */
 					if ( ( ABS(*t) - 2*MAXPOWER ) < 0 ) goto RegPow;
 					s = subs;
 					for ( j = 0; j < i; j++ ) {
 						if ( ( ABS(*t) - 2*MAXPOWER ) == s[2] ) {
 							if ( *s == SYMTONUM ) {
 								*m = s[3];
+								/* Since the power is a wildcard, sgn is 0,1,2 depending whether
+									there were 0,1,2 VECTOMIN in the DOTPRODUCT. Multiply by the
+									power, which is positive currently. */
+								sgn *= *m;
+								/* Now flip the sign of the power, if the wildcard came with a - */
 								if ( *t < 0 ) *m = -*m;
 								dirty = 1;
 								break;
@@ -1330,7 +1358,7 @@ ss10:							*m++ = *t++;
 		substituted. This is ready for further wildcard substitutions.
 */
 
-WORD ResolveSet(PHEAD WORD *from, WORD *to, WORD *subs)
+int ResolveSet(PHEAD WORD *from, WORD *to, WORD *subs)
 {
 	GETBIDENTITY
 	WORD *m, *s, *w, j, i, ii, i3, flag, num;
@@ -1480,14 +1508,14 @@ GotOne:;
 
 /*
  		#] ResolveSet : 
- 		#[ ClearWild :			VOID ClearWild()
+ 		#[ ClearWild :			void ClearWild()
 
 	Clears the current wildcard settings and makes them ready for
 	CheckWild and AddWild.
 
 */
 
-VOID ClearWild(PHEAD0)
+void ClearWild(PHEAD0)
 {
 	GETBIDENTITY
 	WORD n, nn, *w;
@@ -1513,11 +1541,13 @@ VOID ClearWild(PHEAD0)
 
 */
 
-WORD AddWild(PHEAD WORD oldnumber, WORD type, WORD newnumber)
+int AddWild(PHEAD WORD oldnumber, WORD type, WORD newnumber)
 {
 	GETBIDENTITY
 	WORD *w, *m, n, k, i = -1;
 	CBUF *C = cbuf+AT.ebufnum;
+	WORD eattensor = type & EATTENSOR;
+	type = type & ~EATTENSOR;
 DEBUG(WORD *mm;)
 	AN.WildReserve = 0;
 	m = AT.WildMask;
@@ -1560,8 +1590,8 @@ DEBUG(MesPrint("Thread %w(d): m=(%d,%d,%d,%d)(%d)",mm[0],mm[1],mm[2],mm[3],C->nu
 				w[3] = C->numrhs;
 				w = AN.argaddress;
 DEBUG(mm=m;)
-				if ( ( newnumber & EATTENSOR ) != 0 ) {
-					n = newnumber & ~EATTENSOR;
+				if ( eattensor ) {
+					n = newnumber;
 					*m++ = n;
 					w = AN.argaddress;
 				}
@@ -1758,12 +1788,15 @@ FlipOn:
 		or-ed with EATTENSOR which is at least 8192.
 */
 
-WORD CheckWild(PHEAD WORD oldnumber, WORD type, WORD newnumber, WORD *newval)
+int CheckWild(PHEAD WORD oldnumber, WORD type, WORD newnumber, WORD *newval)
 {
 	GETBIDENTITY
 	WORD *w, *m, *s, n, old2, inset;
-	WORD n2, oldval, dirty, i, j, notflag = 0, retblock = 0;
+	WORD n2, oldval, dirty, i, j;
+	int notflag = 0, retblock = 0;
 	CBUF *C = cbuf+AT.ebufnum;
+	WORD eattensor = type & EATTENSOR;
+	type = type & ~EATTENSOR;
 	m = AT.WildMask;
 	w = AN.WildValue;
 	n = AN.NumWild;
@@ -1867,8 +1900,8 @@ WORD CheckWild(PHEAD WORD oldnumber, WORD type, WORD newnumber, WORD *newval)
 				if ( w[2] == oldnumber && *w == ARGTOARG ) {
 					if ( !*m ) return(0);		/* nihil obstat */
 					m = C->rhs[w[3]];
-					if ( ( newnumber & EATTENSOR ) != 0 ) {
-						n = newnumber & ~EATTENSOR;
+					if ( eattensor ) {
+						n = newnumber;
 						if ( *m != 0 ) {
 							if ( n == *m ) {
 								m++;
@@ -2225,6 +2258,17 @@ NoMnot:
 */
 		w = SetElements + Sets[j].first;
 		m = SetElements + Sets[j].last;
+		if ( w == m ) {
+/*
+			The set is empty! This is not a match unless we have !{}, in
+			which case it is.
+*/
+			if ( notflag ) return 0;
+			AN.oldtype = old2;
+			AN.oldvalue = oldval;
+			goto NoMatch;
+		}
+
 		if ( ( Sets[j].flags & ORDEREDSET ) == ORDEREDSET ) {
 /*
 			We search first and ask questions later
@@ -2521,7 +2565,7 @@ int DenToFunction(WORD *term, WORD numfun)
 			*t = numfun; t[2] |= DIRTYFLAG; action = 1;
 		}
 		tnext = t + t[1];
-		if ( *t >= FUNCTION && functions[*t-FUNCTION].spec == 0 ) {
+		if ( *t >= FUNCTION && functions[*t-FUNCTION].spec <= 0 ) {
 			arg = t + FUNHEAD;
 			while ( arg < tnext ) {
 				if ( *arg > 0 ) {
